#ifndef oslxBASE_H
#define oslxBASE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxBASE.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  11/06/97  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "slx.h"
#ifndef NOCCS
    #ifdef SUN_COMP
        #include <sys/types.h>
    #endif
    #include "err.h"
    #include "cmd.h"
    #include "oslxErrors.h"
    #include "slxErrors.h"
    #include "miscErrors.h"
    #include "ECCS.h"
#else
    #include "miscNoCcs.h"
    #include "miscNoCcsCpp.h"
    #include "oslxErrorsNoCcs.h"
    #include "slxErrorsNoCcs.h"
    #include "miscErrorsNoCcs.h"
#endif

// To support logging on stdout

/* The macro __OSLX_DBG__ produces a string in the format: "FileName:Line"
 * with the name and line of the file where it has been called.            
 * It is usefull in error messages                                         
 * (the other macros defined here are just for support, since I have not   
 * found a cleaner way to define __OSLX_DBG__ macro                       
 */   
#define _oslx_tostr(a) #a
#define _oslx_tostr_pass2(a) _oslx_tostr(a) 
#define __OSLX_DBG__   __FILE__ ":" _oslx_tostr_pass2(__LINE__)

#define oslxDEBUG_LEVEL_VAR  "OSLX_DEBUG"
#undef DBG0
#undef DBG1
#undef DBG2
#undef DBG3
#undef DBG4
#undef DBG5
typedef enum {
    DBG0 = 0,   // Debugging information is switched off
    DBG1,       // Standard information level
    DBG2,       // Test level 
    DBG3,       // Debug level 3 - standard debug information level
    DBG4,       // Debug level 4 - specific debug information level
    DBG5,       // Debug level 5 - very thorough debug level (bit print-out)
} isdDBG_LEVEL;

#undef DBG
#define DBG(level, info) { \
if (level <= debugLevel) \
    {\
    if (slxGenDayTimPrec(4, debugTime, &intCcsError) == FAILURE) \
        { \
        printf("\nOSLX Debugging> Problem in calling: slxGenDayTimPrec()"); \
        errCloseStack(&intCcsError); \
	} \
    else \
        printf("\n%s:%s:%s> %s", _instanceName, __OSLX_DBG__, \
               debugTime, info); \
    }\
}

#undef DBG1
#define DBG1(level, info, opt1) { \
if (level <= debugLevel) \
    { \
    if (slxGenDayTimPrec(4, debugTime, &intCcsError) == FAILURE) \
        { \
        printf("\nOSLX Debugging> Problem in calling: slxGenDayTimPrec()"); \
        errCloseStack(&intCcsError); \
	} \
    else \
        printf("\n%s:%s:%s> %s %s", _instanceName, __OSLX_DBG__, \
               debugTime, info, opt1); \
    } \
}

#undef DBG3
#define DBG3(level, info, opt1, opt2, opt3) { \
if (level <= debugLevel) \
    { \
    if (slxGenDayTimPrec(4, debugTime, &intCcsError) == FAILURE) \
        { \
        printf("\nOSLX Debugging> Problem in calling: slxGenDayTimPrec()"); \
        errCloseStack(&intCcsError); \
	} \
    else \
        printf("\n%s:%s:%s> %s %s %s %s", _instanceName, __OSLX_DBG__, \
               debugTime, info, opt1, opt2, opt3); \
    } \
}

#undef DBG4
#define DBG4(level, info, opt) { \
if (level <= debugLevel) \
    { \
    if (slxGenDayTimPrec(4, debugTime, &intCcsError) == FAILURE) \
        { \
        printf("\nOSLX Debugging> Problem in calling: slxGenDayTimPrec()"); \
        errCloseStack(&intCcsError); \
	} \
    else \
        printf("\n%s:%s:%s> %s %d", _instanceName, __OSLX_DBG__, \
               debugTime, info, opt); \
    } \
}

#undef DBG5
#define DBG5(level, info, opt) { \
if (level <= debugLevel) \
    { \
    if (slxGenDayTimPrec(4, debugTime, &intCcsError) == FAILURE) \
        { \
        printf("\nOSLX Debugging> Problem in calling: slxGenDayTimPrec()"); \
        errCloseStack(&intCcsError); \
	} \
    else \
        printf("\n%s:%s:%s> %s %f", _instanceName, __OSLX_DBG__, \
               debugTime, info, opt); \
    } \
}


class oslxBASE: public eccsERROR_CLASS
{
  public: 
    oslxBASE();
    ~oslxBASE();

    void              SetInstanceName(const slxKEYW_LINE  instanceName);

    ccsCOMPL_STAT     CheckVltEnv();

    void              VltEnvAvailable(const vltLOGICAL available);
    vltLOGICAL        VltEnvAvailable();

  protected:
    // Common, internal Error Stack for intermediate adding of errors
    ccsERROR          intCcsError;

    slxKEYW_LINE      _instanceName;

    vltLOGICAL        vltEnvAvailable_;

    /////////////////////////////////////////////////////////////////////////
    // Help variables/pointers
    /////////////////////////////////////////////////////////////////////////
    char              *charP, **charPP;
    vltDOUBLE         dblVal1, dblVal2;
    vltINT8           charVal;
    vltUINT8          *byteP;
    vltUINT8          byteVal;
    vltINT16          *int16P;
    vltINT16          int16Val;
    vltINT32          *intP;
    vltUINT32         *uintP;
    vltINT32          intVal1, intVal2, intVal3;
    vltFLOAT          *floatP;
    vltFLOAT          *floatP2;
    vltFLOAT          floatVal;
    vltLOGICAL        logVal1, logVal2;
    vltINT32          i, j, n, m;   
    vltINT32          no, no1, no2;
    slxFILENAME       tmpFileName1, tmpFileName2;
    slxKEYW_LINE      tmpBuf, tmpBuf1, tmpBuf2;
    vltLOGICAL        errFound, errFound1, errFound2;
    ccsSTACK_ELEM     *errDescription;

    // For debugging purposes
    vltINT32      debugLevel;
    slxKEYW_LINE  debugTime;
};

#endif /*!oslxBASE_H*/
