#ifndef oslxDICTIONARY_H
#define oslxDICTIONARY_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxDICTIONARY.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $" 
*
* oslxDictionary.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  05/03/95  Created
* jknudstr  01/10/95  Lots of changes ...
* epozna    05/05/00  compile on HP11: remove multiple including of <string.h>
* epozna    26/07/00  VLTSW00990319 consequence: LoadFromUrl() must be removed
* epozna    14/08/01  VLTSW20010118 added function: GetDicName()
*/

/************************************************************************
*  Header file for the OSLX Dictionary Class
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

//#include <string.h>   // EPO : included by oslxPRIVATE.h

#include "oslxPRIVATE.h"

class oslxDICTIONARY: public oslxBASE
{
public:
    oslxDICTIONARY();
    virtual ~oslxDICTIONARY();
    
    ccsCOMPL_STAT ResetDic();
  
    // Load/unload a Dictionary in the object
    ccsCOMPL_STAT LoadDic(const slxDICTIONARY  dictionary,
			  const vltLOGICAL     merge = FALSE);
    
    // VLTSW00990319 : remove this function
    //ccsCOMPL_STAT LoadFromUrl(const char       *url, 
    //			      const vltLOGICAL merge = FALSE);

    ccsCOMPL_STAT UnloadDic();

    ccsCOMPL_STAT SaveDic(const slxDICTIONARY dictionary);
    
    // Returns pointer to the Dictionary Hash Table
    miscHASH_EL    *GetDictionary() const;
    
    // Used to step through the dictionary.
    ccsCOMPL_STAT ResetRecordPtr();
    ccsCOMPL_STAT GetNextRecord();
    vltLOGICAL    NextRecord();
    slxDIC_HANDLE *GetDicList();
    slxDIC_HANDLE *PtrCurrRecord();

    ccsCOMPL_STAT Merge(oslxDICTIONARY *srcDic);

    // Looks up a keyword entered as a complete line
    ccsCOMPL_STAT LookUp(const slxKEYW_LINE keywordLine);
    ccsCOMPL_STAT _LookUp(const slxKEYW_LINE keywordLine,
			  ccsERROR           *error);

    // Looks up a keyword entered as individual parameters
    ccsCOMPL_STAT LookUp(const slxCATEGORY    cat, 
			 const slxSUBSYSTEM   subSyst, 
			 const slxPARAMETER   par);
    ccsCOMPL_STAT LookUp2(const char *keyEl1, ...);
 
    vltLOGICAL    KeyInDic(const slxKEYW_LINE  keyword);
     
    // Methods to access the members of the slxDIC_HANDLE structure
    vltLOGICAL    DicFoundKeyword() const;
    char          *KeywName() const;
    ccsCOMPL_STAT KeywName(const slxDIC_RECORD_ENTRY keyName);
    char          *KeywClass() const;
    ccsCOMPL_STAT KeywClass(const slxDIC_RECORD_ENTRY keywClass);
    char          *KeywContext() const;
    ccsCOMPL_STAT KeywContext(const slxDIC_RECORD_ENTRY context);
    slxPAR_TYPE   DicKeywType() const;
    ccsCOMPL_STAT DicKeywType(const slxPAR_TYPE type);
    char          *ValFormat() const;
    ccsCOMPL_STAT ValFormat(const slxDIC_RECORD_ENTRY format);
    char          *KeywUnit() const;      
    ccsCOMPL_STAT KeywUnit(const slxDIC_RECORD_ENTRY unit);
    char          *ComField() const;
    ccsCOMPL_STAT ComField(const slxDIC_RECORD_ENTRY com);
    char          *Description() const;
    ccsCOMPL_STAT Description(const char *descr);

    ccsCOMPL_STAT MatchKey(const slxKEYW_LINE key);
    char          *MatchKeyList();
    char          *BestFit();
    vltINT32      BestFitHits();

    ccsCOMPL_STAT StoreKeyword(const slxKEYW_LINE  keyword,
			       const slxKEYW_LINE  classList,
			       const slxKEYW_LINE  context,
			       const slxKEYW_LINE  type,
			       const slxKEYW_LINE  valueFormat,
			       const slxKEYW_LINE  unit,
			       const slxKEYW_LINE  comment,
			       const char          *description,
			       const vltLOGICAL    sort = TRUE);
    ccsCOMPL_STAT DeleteKeyword(const slxKEYW_LINE  keyword = NULL);
    ccsCOMPL_STAT SortList();

    char          *GetHeader();
    ccsCOMPL_STAT StoreHeader(const char *newHeader); 

    ccsCOMPL_STAT DumpRecord(const slxKEYW_LINE  keyword,
			     const vltLOGICAL    detailed = TRUE);
    ccsCOMPL_STAT DumpDictionary(const vltLOGICAL dumpOnlyKeys = TRUE);
    
    // Returns TRUE if the referenced Keyword Class is set for the 
    // current keyword looked up.
    vltLOGICAL KeywordClass(const slxKEYWORD_CLASS keywordClass) const;    
    
    miscHASH_TABLE  dicHashTable;   // Dictionary Hash Table
    
    ccsCOMPL_STAT GetDicName(slxDICTIONARY fullDictName);

  protected:
    ccsCOMPL_STAT  AddRecord(slxDIC_HANDLE *dicRec);

    ccsCOMPL_STAT  MatchKeyEls_(const char   *matchKeyEl,
				const char   *dicKeyEl,
				vltLOGICAL   *match);
    void           WildCardMatch_(const char  *matchKey,
				  const char  *dicKey,
				  vltLOGICAL  *match);

    ccsCOMPL_STAT  GenPafDic();

    vltINT8        dicLoaded;       // Indicates if dictionary is loaded
    slxDICTIONARY  dicName;         // Name of the dictionary loaded.
    slxDYN_BUF     dicHdr;

    // List of SLX Dictionary Handles
    slxDIC_HANDLE  dicList;

    // Current dictionary record to which there is referenced         
    slxDIC_HANDLE  *dicRecord;        

    slxDYN_BUF     matchKeyList;
    slxKEYW_LINE   bestFit;
    vltINT32       bestFitHits;
}; // end-class oslxDICTIONARY

#endif /*!oslxDICTIONARY_H*/





