#ifndef oslxKEYWORD_H
#define oslxKEYWORD_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxKEYWORD.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $" 
*
* oslxKEYWORD.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  05/03/95  Created
* jknudstr  05/10/95  Lots of changes ...
*/

/************************************************************************
*  Header file for the OSLX KEYWORD Class
*----------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#include "oslxPRIVATE.h"
#include "oslxDICTIONARY.h"

// Class to handle a single keyword
class oslxKEYWORD: public oslxBASE, public slxKEYW_INFO
{
  public:
    oslxKEYWORD();
    ~oslxKEYWORD();

    void           ClearKeyword();

    vltINT32       KeywordNo() const;

    ccsCOMPL_STAT  LoadDictionary(const slxDICTIONARY  dicName, 
				  const vltLOGICAL     pricateDic = FALSE);
    ccsCOMPL_STAT  SetDictionary(const oslxDICTIONARY  *dictionary,
				 const vltLOGICAL      pricateDic = FALSE);

    ccsCOMPL_STAT  SplitKeyword(const slxKEYW_LINE keywLine);

    vltLOGICAL     ValueValid();

    slxPAR_TYPE    Type() const;
    ccsCOMPL_STAT  Type(const slxPAR_TYPE type);
    char           *String();
    ccsCOMPL_STAT  String(const char *string);
    vltLOGICAL     Logical() const;
    ccsCOMPL_STAT  Logical(const vltLOGICAL logical);
    vltINT32       Integer() const;
    ccsCOMPL_STAT  Integer(const vltINT32 integer);
    vltDOUBLE      Double() const;
    ccsCOMPL_STAT  Double(const vltDOUBLE dbl);

    char           *Keyword() const;
    ccsCOMPL_STAT  Keyword(const slxKEYW_LINE keyword);
    char           *KeywordLine();
    ccsCOMPL_STAT  KeywordLine(const slxKEYW_LINE keywLine);
    char           *Category() const;
    ccsCOMPL_STAT  Category(const slxCATEGORY cat);
    char           *SubSystem(const vltINT32  no) const;
    ccsCOMPL_STAT  SubSystem(const vltINT32     no,
			     const slxSUBSYSTEM subSyst);
    char           *Parameter() const;
    ccsCOMPL_STAT  Parameter(const slxPARAMETER par);    
    char           *ValueString() const;
    char           ValueString(const char *valStr);
    char           *Comment() const;
    ccsCOMPL_STAT  Comment(const slxCOMMENT com);

    vltINT32       GetKeywordIndex(const char *keyword);
    
    // Return flag stating whether keyword is included or not
    vltLOGICAL     Included();

    oslxKEYWORD &operator = (const slxKEYW_INFO &keywordPtr);


    //////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////
    // OBSOLETE METHODS - KEPT FOR BACKWARDS COMPATABILITY REASONS
    //////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////
    char           *SubSystem() const;
    ccsCOMPL_STAT  SubSystem(const slxSUBSYSTEM subSyst);    
    char           *SubSystem2() const;
    ccsCOMPL_STAT  SubSystem2(const slxSUBSYSTEM subSyst);
    char           *SubSystem3() const;
    ccsCOMPL_STAT  SubSystem3(const slxSUBSYSTEM subSyst);
    char           *SubSystem4() const;
    ccsCOMPL_STAT  SubSystem4(const slxSUBSYSTEM subSyst);
    char           *SubSystem5() const;
    ccsCOMPL_STAT  SubSystem5(const slxSUBSYSTEM subSyst);

  protected:
    slxSETUP_FORMAT         keywordType;

    // Indicates if the value set according to the type of the keyword
    // is valid. This is the case after a successfull SplitKeyword() when
    // a Dictionary is available.
    vltLOGICAL              valueValid;

    ccsCOMPL_STAT           LoadCommonDic(const slxDICTIONARY  dicName);
    ccsCOMPL_STAT           LoadPrivateDic(const slxDICTIONARY  dicName);
    ccsCOMPL_STAT           UnloadCommonDic();
    ccsCOMPL_STAT           UnloadPrivateDic();

    ccsCOMPL_STAT           UpdateKeyword();

    // Parameters for handling the Dictionary common for all instances of
    // the oslxKEYWORD class
    ccsCOMPL_STAT           DicAvailable();

    static oslxDICTIONARY   *dictionary;  
    static vltLOGICAL       localDic;
    // Indicates if a Common Dictionary is available
    static vltLOGICAL       comDicAvail;
    
    // Parameter to handle a local Dictionary
    vltLOGICAL              priDic;
    oslxDICTIONARY          *priDictionary;  
    vltLOGICAL              priDicLocal;
};   // end-class oslxKEYWORD


#endif /*!oslxKEYWORD_H*/





