#ifndef oslxSETUP_H
#define oslxSETUP_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxSETUP.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $" 
*
* oslxSETUP.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  05/03/95  Created
* jknudstr  26/09/95  Lots of changes ...
* jknudstr  19/10/95  Introduced local oslxSETUP_HEADER in object
* jknudstr  05/02/96  Took out most methods and derived class from
*                     oslxSHORT_FITS
*/

/************************************************************************
*  Header file for the OSLX SETUP Class
*----------------------------------------------------------------------
*/
#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#include "oslxPRIVATE.h"
#include "oslxSHORT_FITS.h"
#include "oslxFITS_BASE.h"

// Various Check Types as strings
#define oslxNORMAL_SETUP_CHECK  "NORMAL"
#define oslxFULL_SETUP_CHECK    "FULL"
#define oslxSOFT_SETUP_CHECK    "SOFT"
#define oslxNO_SETUP_CHECK      "NONE"


// Contains methods and variables to handle a setup buffer in memory
class oslxSETUP: public oslxSHORT_FITS
{
  public: 
    // Constructor method that associates to the object a dictionary and
    // an alias table and loads one setup file.
    // If the dictionary or the alias conversion talbe are NULL, they are
    // not used and check is turned off.
    // The setup file name will be the name of the setup file for the obj
    oslxSETUP(const oslxDICTIONARY    *dictionary  = NULL, 
	      const oslxALIAS         *alias       = NULL);
    ~oslxSETUP();
   
    /**** Methods to access SETUP command attributes ****/
    ccsCOMPL_STAT  ExpoID(const vltINT32 expoID);
    vltINT32       ExpoID() const;
    ccsCOMPL_STAT  NoMoveFlag(const vltLOGICAL noMove);
    vltLOGICAL     NoMoveFlag() const;
    ccsCOMPL_STAT  CheckFlag(const vltLOGICAL check);
    vltLOGICAL     CheckFlag() const;

    /**** General methods acting on the whole setup  ****/

    // Resets the setup buffer of the object
    ccsCOMPL_STAT  ClearSetup();

    /**** Handling of buffers to/from SETUP commands ****/

    // Handle (read in) the parameters specified by the buffer of
    // a SETUP command the parameters are merged into the setup
    // buffer of the object
#ifndef NOCCS
    ccsCOMPL_STAT HandleSetupCmd(const char        *setupBuf,
				 const vltLOGICAL  updateOnly = FALSE);
#endif

    // Methods to access the SETUP command flags
    slxSETUP_FLAGS *GetSetupAttr();
    ccsCOMPL_STAT  ResetSetupAttr();
    ccsCOMPL_STAT  SetSetupAttr(const slxSETUP_FLAGS *setupAttrib);

    // Methods to generate a buffer which is suitable for sending via CCS.
    // The format of the generated buffer will be the position dependent
    // format as specified by the CDT for the SETUP command
    // The oslxKEYW_FILTER class can be used to extract special keywords.
    ccsCOMPL_STAT MakeMsgBuf(oslxKEYW_FILTER       *filter = NULL);
   
    // Makes a message buffer from the keywords marked as changed in the
    // Change Record. If the filter is specified only the changed keywords
    // fullfilling the filter criterias will be extracted.
    ccsCOMPL_STAT MakeMsgBufChRec(oslxKEYW_FILTER  *filter = NULL);
   
    // Methods to return pointers/lengths of the setup message buffer
    char       *SetupMsgBuf() const;
    vltUINT32   SetupMsgBufLen() const;

    /**** Load and save methods ****/

    // Method to load a setup file into the local setup buffer while
    // checking the keywords
    ccsCOMPL_STAT LoadSetup(const slxFILENAME     fileName,
			    const vltLOGICAL      globalSearch = FALSE);

    // Method to extract a setup from a FITS Data File Header
    ccsCOMPL_STAT ExtractSetupFromFitsHd(const slxFILENAME dataFile);

    // Sets/returns the file name for the setup-file
    ccsCOMPL_STAT SetSetupFile(const slxFILENAME fileName);

    // Save the setup parameters currently stored in the object into
    // the file with which the object is connected 
    // (creation or SetSetupFile()).
    ccsCOMPL_STAT Save(const slxSETUP_TYPE       setupType,
		       const char                *obsMode = NULL,
		       const vltLOGICAL          useChangeRecord = FALSE,
		       oslxKEYW_FILTER           *filter = NULL,
		       const vltLOGICAL          saveComments = FALSE,
		       const vltLOGICAL          saveIncludedKeys = FALSE);
    ccsCOMPL_STAT SaveAs(const slxFILENAME       fileName,
			 const slxSETUP_TYPE     setupType,
			 const char              *obsMode = NULL,
			 const vltLOGICAL        useChangeRecord = FALSE,
			 oslxKEYW_FILTER         *filter = NULL,
			 const vltLOGICAL        saveComments = FALSE,
			 const vltLOGICAL        saveIncludedKeys = FALSE);



  protected:

    ccsCOMPL_STAT   SearchFile(const slxFILENAME    baseName,
			       vltINT32             *numberFound,
			       slxSTRING_LIST       *fileList,
			       const vltLOGICAL     globalSearch,
			       const vltLOGICAL     stopFirstOccur,
			       const vltLOGICAL     skipInsUserDir);

    // Make some methods anavailable for the user of the oslxSETUP class
    ccsCOMPL_STAT LoadShortFitsFile(const slxFILENAME      fileName);
    ccsCOMPL_STAT SetShortFitsFile(const slxFILENAME   fileName);
    ccsCOMPL_STAT Save(const vltLOGICAL       useChRecord =FALSE,
		       oslxKEYW_FILTER        *filter = NULL);
    ccsCOMPL_STAT SaveAs(const slxFILENAME    fileName,
			 const vltLOGICAL     useChRecord = FALSE,
			 oslxKEYW_FILTER      *filter = NULL,
			 const oslxSHORT_FITS *header = NULL);

    slxDYN_BUF        msgBuf;         // Message buffer generated

    slxSETUP_FLAGS    setupFlags;     // Flags to use in connection with the 
                                      // SETUP command
 

};   // end-class oslxSETUP


#endif /*!oslxSETUP_H*/





