#ifndef oslxSHORT_FITS_H
#define oslxSHORT_FITS_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxSHORT_FITS.h,v 1.1 2007/02/26 14:51:42 cguirao Exp $" 
*
* oslxSHORT_FITS.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  24/01/96  created
* epozna    06/12/00  VLTSW20000469 added function SearchTsfxFile
* epozna    17/04/03  VLTSW20020388 added isLongStringValues
*/

/************************************************************************
*  Header file for the OSLX Short-FITS Class
*----------------------------------------------------------------------
*/
#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#include "oslxPRIVATE.h"
#include "oslxFITS_BASE.h"
#include "oslxALIAS.h"
#include "oslxDICTIONARY.h"
#include "oslxKEYWORD.h"
#include "oslxKEYW_FILTER.h"

// Structure to handle repeated keywords
typedef struct {
   vltINT32  keyCount;
} _oslxREP_KEY_HANDLE;
#define _oslxREP_KEY_TAG  "___#"

typedef enum {
    oslxREP_ALLOW_NO_KEYS = 0,
    oslxREP_ALLOW_ALL_KEYS,
    oslxREP_ALLOW_SOME_KEYS
} oslxREP_TYPE;

// Contains methods and variables to handle a Short-FITS keyword buffer 
// in memory
class oslxSHORT_FITS: public oslxFITS_BASE
{
  public: 
    // Constructor method that associates a dictionary and
    // an alias table to the object.
    oslxSHORT_FITS(const oslxDICTIONARY    *dictionary  = NULL, 
		   const oslxALIAS         *alias       = NULL);
    virtual ~oslxSHORT_FITS();
    
    virtual ccsCOMPL_STAT  LoadDictionary(const slxDICTIONARY  dicName,
					  const vltLOGICAL     merge = FALSE);
    virtual ccsCOMPL_STAT  SetDictionary(const oslxDICTIONARY *dictionary);

   
    // Methods to handle operation on PAFs with repeated keywords
    void          AllowRepeatedKeys(const oslxREP_TYPE state);
    ccsCOMPL_STAT AllowRepeatedKeys(const char *keyList); 
    oslxREP_TYPE  AllowRepeatedKeys() const; 
    ccsCOMPL_STAT NumberOfOccurences(const slxKEYW_LINE keyword,
				     vltINT32           *no);

    ccsCOMPL_STAT DumpKeywords(const vltLOGICAL listPafKeys = FALSE);
    void          ShowLists();

    /**** Methods to handle Data Blocks ****/
    ccsCOMPL_STAT ResetData();
    ccsCOMPL_STAT GetDataBlock(char            **dataPtr,
			       vltINT32        *dataLen,
			       const vltINT32  dataBlockNumber = 1);
    ccsCOMPL_STAT SetDataBlock(char            *dataPtr,
			       vltINT32        dataLen,
			       const vltINT32  dataBlockNumber = 1);
    vltINT32      NumberOfDataBlocks();
  
    // Can be used to add a suite of additional keywords needed for a certain
    // key, like is the case for the TSF keys:
    // DET.DIT: DET.DIT.VALUE, DET.DIT.RANGE ...
    virtual ccsCOMPL_STAT CheckAddDicKeys(const slxKEYW_LINE  keyword);
  
    /**** Access methods to current keyword ****/
  
    // All search and set operations on keyword act on the CURRENT KEYWORD
    // This contains the values to be stored and the retrieved values
    // for the last search performed
    oslxKEYWORD   &CurrKeyword();  
    oslxKEYWORD   &CK();

    virtual ccsCOMPL_STAT GetFormat(char* format);
    virtual ccsCOMPL_STAT GetFormat(char* keyword,char* format);

    /**** General methods acting on the whole object  ****/
  
    // Resets the keyword buffer of the object
    ccsCOMPL_STAT         ClearInstance();
    virtual ccsCOMPL_STAT ClearPri();
  
    // Merges a number of Short-FITS files, single keywords and slxDYN_BUF 
    // buffers containing keywords together in the objects keyword buffer.
    // Format is:
    // -k <keyword + value>[ <keyword + value>]
    // -b <slx Dyn Buf Ptr>[ <slx Dyn Buf Ptr>]
    // -f <Short-FITS  filename>[ <Short-FITS filename>]
    // Each type of 'media' must be separated by comma.
    ccsCOMPL_STAT Merge(const char   *dummy, ...);
    ccsCOMPL_STAT Update(vltINT32  *keywordsStored, ...);
  
    /**** Methods to monitor changes of single Short-FITS parameters ****/
  
    // NOTE: The first keyword line has number _1_ in the Change Record
    void          ResetChangeRec(const vltLOGICAL resetValue = FALSE);
    vltINT8       *GetChangeRec() const;
    vltLOGICAL    CurrKeywordChanged();
    vltLOGICAL    HasChanged(const slxKEYW_LINE keyword,
			     const vltINT32     keyNo = 0);
    vltLOGICAL    HasChanged(const vltUINT16 keywordNo);
    ccsCOMPL_STAT HasChanged(const slxKEYW_LINE keyword,
			     vltLOGICAL         *hasChanged,
			     const vltINT32     keyNo = 0);
    vltINT32      NoOfChangedKeywords() const;
  
    /**** Methods to extract keyword from the internal buffer ****/
  
    // Method that extract keywords and organizes them in the format
    // "keyword value keyword value ..."
    ccsCOMPL_STAT ExtractKeywords(const vltLOGICAL    useChangeRecord = FALSE,
				  oslxKEYW_FILTER     *filter = NULL,
				  const vltLOGICAL    buildBuf = TRUE);
    ccsCOMPL_STAT NextExtractedKeyword(const vltLOGICAL  init = FALSE);
    char          *ExtractedKeywords() const;
    vltINT32      ExtractedBufLen() const;
    vltINT32      NoExtractedKeywords() const;
  
    ccsCOMPL_STAT StoreExtrBuffer(const char *extrBuf);

    /**** Retrieve/store of keywords. Result is stored in CurrKeyword() ****/
  
    // Returns a specific keyword identified by its number in the buffer.
    // The first keyword being # 1. Comments do not count.
    // The members for CurrKeyword() are set accordingly.
    ccsCOMPL_STAT GetKeyword(const vltUINT16 keywordNo);
  
    // Check if a certain key is stored in the object
    vltLOGICAL KeyInBuffer(const slxKEYW_LINE keyword);

    // Searches for a keyword in the object's keyword buffer.
    // The members for CurrKeyword() are set accordingly.
    ccsCOMPL_STAT GetKeyword(const slxKEYW_LINE keyword,
			     const vltINT32     keyNo = 0);
  
    // Stores a keyword in the local keyword buffer. If the keyword is
    // represented as an alias the keyword is replaced with the alias
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_LINE keywordLine);    
    ccsCOMPL_STAT StoreKeywordAfter(const slxKEYW_LINE keywordLine,
				    const slxKEYW_LINE afterKey);
  
    // Method to remove a keyword from the object
    ccsCOMPL_STAT RemoveKeyword(const slxKEYW_LINE keyword,
				const vltINT32     keyNo = 0);
    ccsCOMPL_STAT RemoveKeyword(slxKEYW_INFO  *keyEl);

    // Stores a keyword based on its name and its value
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_LINE keyword,
			       const char         *stringVal,
			       const slxKEYW_LINE afterKey = NULL);    
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_LINE keyword,
			       const vltLOGICAL   logicalVal,
			       const slxKEYW_LINE afterKey = NULL);    
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_LINE keyword,
			       const vltINT32     integerVal,
			       const slxKEYW_LINE afterKey = NULL);    
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_LINE keyword,
			       const vltDOUBLE    doubleVal,
			       const slxKEYW_LINE afterKey = NULL);    
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_INFO *keyword);
    ccsCOMPL_STAT StoreKeyword(oslxKEYWORD        *keyword);
  
    // Returns number of Short-FITS keywords in the buffer
    vltINT32  NumberOfKeywords();

    // Methods to return the next sequential keyword by each call.
    // The members for CurrKeyword() are set accordingly.
    ccsCOMPL_STAT ResetKeywordPtr(const vltLOGICAL useChRec = FALSE);
    ccsCOMPL_STAT NextKeyword(const vltLOGICAL useChRec = FALSE,
			      oslxKEYW_FILTER  *filter = NULL);

    vltLOGICAL    Included(const slxKEYW_LINE keyword);
  
    /**** Load and save methods ****/
  
    // Return pointer to the filename currently stored in the object
    char *GetFileName();
  
    // Method to load a Short-FITS file into the local buffer while
    // checking the keywords
    ccsCOMPL_STAT LoadShortFitsFile(const slxFILENAME   fileName,
				    const char          *envVar = NULL,
				    oslxKEYW_FILTER     *filter = NULL,
				    const vltLOGICAL    updateOnly = FALSE,
				    const vltLOGICAL    globalSearch = FALSE);
  
    // Sets the file name for the Short-FITS File
    ccsCOMPL_STAT SetShortFitsFile(const slxFILENAME fileName);
  
    // Save the Short-FITS keywords currently stored in the object into
    // the file with which the object is connected 
    ccsCOMPL_STAT Save(const vltLOGICAL        useChRecord =FALSE,
		       oslxKEYW_FILTER         *filter = NULL,
		       const vltLOGICAL        saveComments = FALSE,
		       const vltLOGICAL        saveIncludedKeys = FALSE);
  
    ccsCOMPL_STAT SaveAs(const slxFILENAME      fileName,
			 const vltLOGICAL       useChRecord = FALSE,
			 oslxKEYW_FILTER        *filter = NULL,
			 const vltLOGICAL       saveComments = FALSE,
			 const vltLOGICAL       saveIncludedKeys = FALSE);
  
    // Method to transfer keywords between two oslxSHORT_FITS objects.
    // The object invoking the method is the destination object.
    ccsCOMPL_STAT Copy(oslxSHORT_FITS       *sourceObject,
		       const vltLOGICAL     useChRecord = FALSE,
		       oslxKEYW_FILTER      *filter = NULL);
  
    slxKEYW_INFO    *KeywordList();
  
    miscHASH_TABLE  &GetHashTable();  
  

    ////////////////////////////////////////////////////////////////
    // THESE METHODS ARE OBSOLETE
    // KEPT FOR BACKWARDS COMPATABILITY
    ////////////////////////////////////////////////////////////////
    ccsCOMPL_STAT  NormalCheck();
    ccsCOMPL_STAT  FullCheck();
    ccsCOMPL_STAT  SoftCheck();
    ccsCOMPL_STAT  NoCheck();
    ccsCOMPL_STAT  SetCheck(const char  *oslxCheckType);
    oslxCHECK_MODE CheckMode();
    ////////////////////////////////////////////////////////////////
    
  ccsCOMPL_STAT SearchTsfFile(const char *         filename,
  				slxFILENAME         fullFileName);
  
  protected: 
    ccsCOMPL_STAT         _DeleteCleanUp();
    virtual ccsCOMPL_STAT SearchFile(const slxFILENAME baseName,
				     vltINT32          *numberFound,
				     slxSTRING_LIST    *fileList,
				     const vltLOGICAL  globalSearch = FALSE,
				     const vltLOGICAL  stopFirstOccur = TRUE,
				     const vltLOGICAL  skipInsUserDir = FALSE);
  
  //ccsCOMPL_STAT SearchTsfFile(const char *         filename,
  //				slxFILENAME         fullFileName);

    ccsCOMPL_STAT LoadShortFitsInList(slxFILENAME       fileName,
				      const char        *envVar = NULL,
				      oslxKEYW_FILTER   *filter = NULL,
				      const vltLOGICAL  updateOnly = FALSE,
				      const vltLOGICAL  globalSearch = FALSE);
    void          RenumberKeyInfoStrs(const vltLOGICAL pafList);    
    ccsCOMPL_STAT GetFileBuf();

    slxDYN_BUF        *curFileBuf;
    miscHASH_TABLE    includeFileBufs;

    slxSTRING_LIST    *fileList;

    // Contains the complete name of the file currently being loaded
    slxFILENAME       completeNameLoading;
  
    // Hash Table used for searching for keywords
    miscHASH_TABLE    hashTable;      
    slxCHANGE_RECORD  changeRec;      // Keyword Change Record
    slxKEYW_INFO      keywords;       // Linked list of keyword structures

    // Number of files included directly into the current PAF (doesn't
    // count files included from included files)
    vltINT32          includeCount;
    // The level of inclusions
    vltINT32          includeLevel;
    
    // To store temporarily the filename of a file to include
    slxSTRING_LIST    *includeListTmp;
    vltINT32          includeTmpCount;

    // Handling of Data Blocks
    vltINT32          dataBlockCount;
    slxDYN_BUF        dataBuf;

    // Pointer to the Current Keyword usage is controlled by the user
    // (ResetKeywordPtr() ..)
    slxKEYW_INFO      *keywordPtrUsr; 

    // Used by OSLX to address current keyword
    slxKEYW_INFO      *keywordPtrTmp; 

    // Hash Table used for searching for PAF keywords
    miscHASH_TABLE    pafHashTable;     
    slxCHANGE_RECORD  pafChangeRec;      // PAF Change Record
    slxKEYW_INFO      pafKeywords;       // Linked list of keyword structures
    // Pointer to the Current Keyword usage is controlled by the user
    // (ResetKeywordPtr() ..)  
    slxKEYW_INFO      *pafKeywordPtrUsr;
    // Used by OSLX to address current keyword
    slxKEYW_INFO      *pafKeywordPtrTmp; 

    // Change Record to indicate which keywords that were extracted  
    slxCHANGE_RECORD  extrKeywordsChRec; 
    vltINT32          extrKeysChRecIndex;
    slxDYN_BUF        extrKeywords;   // Buffer to hold extracted keywords
    vltINT32          noOfExtrKeys;   // The number of keywords extracted
  
    slxFILENAME       fileName;       // Current filename referenced
  
    // Convert header in old 'SUP' format to new 'PAF' format
    ccsCOMPL_STAT ConvertSup2Paf(slxKEYW_LINE  keywordLine);
  
    // Various help variables
    vltINT32          intVal, intVal1, intVal2, intVal3, intVal4;
    char              *chrPtr1, *chrPtr2;
    slxFILENAME       tmpFileName;
    slxKEYW_LINE      tmpBuf1, tmpBuf2, tmpBuf3;
    slxKEYW_INFO      *keyPtr1, *keyPtr2;
  
    // Method that carries out a check of a keyword line according to the
    // Check Mode of the object. The keyword is stored if OK or ignored
    // if Soft Check is requested. Else an error is returned.
    ccsCOMPL_STAT     CheckStoreKeyword(slxKEYW_LINE       keywordLine,
					const vltLOGICAL   updateOnly = FALSE,
					slxDYN_BUF         *comBuf = NULL,
					const slxKEYW_LINE afterKey = NULL,
					slxDYN_BUF         *dataBuf = NULL);

    // To handle repeated keywords
    ccsCOMPL_STAT     UpdateRepeatedKeysTable(const slxKEYW_LINE keyword);

    oslxREP_TYPE      allowRepeatedKeys;
    miscHASH_TABLE    repeatedKeysTable;
    miscHASH_TABLE    allowedRepeatedKeys;

    ccsCOMPL_STAT SavePafFileList(vltLOGICAL        storeGenFile,
				  slxFILENAME       fileName,
				  slxKEYW_INFO      *keywordList,
				  miscHASH_TABLE    pafHdHashTable,
				  slxCHANGE_RECORD  pafChRec,
				  slxCHANGE_RECORD  filterList,  
				  miscHASH_TABLE    dictionary,
				  const vltLOGICAL  saveComments,
				  const vltLOGICAL  saveIncludKeys,
				  vltINT32          includeCount);

    ccsCOMPL_STAT _GetKeyword(const slxKEYW_LINE keyword,
			      const vltINT32     keyNo,
			      ccsERROR           *intError);  

    ccsCOMPL_STAT Merge_v(const vltLOGICAL  updateOnly,
			  va_list           argList);

    // Indicates if any keywords were stored in the object
    vltINT32     keywordsStored;

    // Buffer in which to store comments written before each key
    slxDYN_BUF  comBuf;

    // Checks the keyword line given in as input parameters according to
    // the Check Mode and returns a possible error. If the keyword of the
    // keyword line is an alias this is replaced with the actual Short-FITS
    // keyword.
    ccsCOMPL_STAT     CheckKeyword(slxKEYW_LINE keywordLine);
    ccsCOMPL_STAT     _CheckKeyword(slxKEYW_LINE keywordLine,
				    ccsERROR     *intError);
  
    ccsCOMPL_STAT     StoreKeywordInList(const slxKEYW_LINE keywordLine,
					 oslxKEYW_FILTER    *filter = NULL,
					 slxDYN_BUF         *comBuf = NULL,
					 const slxKEYW_LINE afterKey = NULL,
					 slxDYN_BUF         *dataBuf = NULL);

    // Current keyword
    oslxKEYWORD       currKeyword;   

    // Keyword object to be used for various purposes
    oslxKEYWORD       utilKeyword;

    // Indicates the current level of check
    // to be performed by the object
    oslxCHECK_MODE    checkMode;      

    vltLOGICAL isDeleteSosKw;
    vltLOGICAL isLongStringValues; // VLTSW20020388

};   // end-class oslxSHORT_FITS


#endif /*!oslxSHORT_FITS_H*/





