global env

class oslxBASE {

    constructor {} {
	global  env oslx 
	set oslx(comFile)      $env(HOME)/.oslxServer
	set oslx(reply)        ""
	set oslx(errorStack)   ""    
	set oslx(result)       ""
	set oslx(fdIn)         ""
	set oslx(fdOut)        ""
	set oslx(timeOut)      20000
	set oslx(debug)        0
	if {![info exists oslx(startServer)]} {
	    set oslx(startServer)  1
	}
	set oslx(errPrefix)    "oslxError:\n"
	set oslx(errPrefixLen) [string length $oslx(errPrefix)]

	# Define errors
	set oslx(oslxERR_TCL_WRITE_SOCK) \
		"Error writing socket"
	set oslx(oslxERR_TCL_FORMAT_REPLY) \
		"Format of reply from OSLX Server is wrong"
	set oslx(oslxERR_TCL_START_SERV) \
		"Could not start the OSLX Server"
	set oslx(oslxERR_TCL_OPEN_FILE) \
		"Could not open file"
	set oslx(oslxERR_TCL_OPEN_CON) \
		"Could not open socket connection to OSLX Server"

	Debug "In contructor method of oslxBASE"
    }

    destructor {
	Debug "In destructor method of oslxBASE"
	exit
    }

    public    method    Debug {msg}    
    public    method    SendMsg {cmd parameter data}
    public    method    AnalyzeReply {rep}
    public    method    ReceiveMsg {}
    public    method    ErrStack {}
    public    method    ErrInStack {errMnemonic}
    public    method    ErrResetStack {}
    public    method    ErrCloseStack {}
    public    method    ErrPrintStack {}

    private   method    OpenConnection {} 
    private   method    CloseConnection {}
}

body oslxBASE::Debug {msg} {
    global oslx

    if {$oslx(debug)} {
	puts "oslx> $msg"
    }
}

body oslxBASE::SendMsg {cmd parameter data} {
    global env oslx

    set oslx(errorStack) ""
    set oslx(reply) ""

    if {[string length $oslx(fdIn)] == 0} {
	if {[OpenConnection] == -1} {
	    return -1
	}
    }

    # Create the message
    set msg "$cmd $parameter $data"
    set msg "[string length $msg]|$msg"
    Debug "Message to OSLX Server: $msg"    

    # Write the message
    if {[catch {puts -nonewline $oslx(fdIn) $msg} res]} {
	set oslx(errorStack) \
		"$oslx(oslxERR_TCL_WRITE_SOCK): \"Server Input Socket\""
	CloseConnection
	return -1
    }
    flush $oslx(fdIn)

    # Wait for the reply 
    set res [select $oslx(fdOut) {} {} [expr $oslx(timeOut) / 1000]]
    Debug "Select unblocked: $res"
    
    set count 0
    while {1} {   
	# Message format: <len>|<data>
	# Read in the message size
	set run 1
	set msgLen ""
	while {$run} {
	    set inBuf [read $oslx(fdOut) 1]
	    if {[string compare $inBuf "|"] == 0} {
		set run 0
	    } elseif {[string length $inBuf] > 0} {
		set msgLen "${msgLen}${inBuf}"
	    } else {
		Debug "Error in format of reply"
		set oslx(errorStack) "$oslx(oslxERR_TCL_FORMAT_REPLY): \
			\"$msgLen\""
		CloseConnection
		if {$count == 1} {
		    return -1
		} else {
		    if {[OpenConnection] == -1} {
			return -1
		    }
		    set oslx(errorStack) ""
		    incr count
		}
	    }	
	}
	set oslx(reply) ""
	set oslx(reply) [read $oslx(fdOut) $msgLen]
	if {[AnalyzeReply $oslx(reply)] == -1} {
	    return -1
	} else {
	    Debug "Received from OSLX Server: $oslx(reply)"
	    return $oslx(reply)
	}
    }
}


body oslxBASE::AnalyzeReply {rep} {
    global oslx

    # Check for $oslx(errPrefix)
    set res [string first $oslx(errPrefix) $rep]
    if {$res != -1} {
	set err [string range $rep $oslx(errPrefixLen) end]
	Debug "Error Occurred: $err"
	set oslx(errorStack) $err"
	return -1
    } else {
	return 0
    }
}

body oslxBASE::OpenConnection {} {
    global env oslx
    
    set comFileIn  "$oslx(comFile)"
    set comFileIn  "${comFileIn}In"
    set comFileOut "$oslx(comFile)"
    set comFileOut "${comFileOut}Out"

    # Prepare the communication files
    if {$oslx(startServer)} {
	Debug "Deleting Socket Port Number File: $comFileIn"
	catch {exec rm $comFileIn}
	Debug "Deleting Socket Port Number File: $comFileOut"
	catch {exec rm $comFileOut}

	# Start the OSLX Server
	if {[catch {exec oslxServer &}]} {
	    puts "Problems starting the OSLX Server"
	    set oslx(errorStack) "$oslx(oslxERR_TCL_START_SERV)"
	    return -1
	}
    }

    # Poll for the Server to create an socket (and write the
    # port number in the file
    set run 1
    while {$run} {
	Debug "Checking if file \"$comFileIn\" exists ..."
	if {[file exist $comFileIn] == 1} {
	    Debug "File exists!"
	    set run 0
	} else {
	    Debug "File does not exist"
	    after 500    
	    incr run
	}
	if {$run == 20} {
	    set oslx(errorStack) "$oslx(oslxERR_TCL_START_SERV)"
	    return -1
	}
    }

    # Read the port number, create the socket and delete the file
    Debug "Opening Socket Port Number File $comFileIn"
    set fd [open $comFileIn]
    if {$fd == -1} {
	set oslx(errorStack) "$oslx(oslxERR_TCL_OPEN_FILE): \"$comFileIn\""
	return -1
    }
    set inBuf [read $fd]
    if {$inBuf == -1} {
    }
    Debug "Read from Socket Port Number File: $inBuf"
    set portNoIn $inBuf
    close $fd
    if {$oslx(startServer)} {
	catch {exec rm $comFileIn}
    }
    Debug "Opening Socket Port Number File $comFileOut"
    set fd [open $comFileOut]
    if {$fd == -1} {
	set oslx(errorStack) "$oslx(oslxERR_TCL_OPEN_FILE): \"$comFileOut\""
	return -1
    }
    set inBuf [read $fd]
    if {$inBuf == -1} {
    }
    Debug "Read from Socket Port Number File: $inBuf"
    set portNoOut $inBuf
    close $fd
    if {$oslx(startServer)} {
	catch {exec rm $comFileOut}
    }

    Debug "Creating socket connections ..."
    #set oslx(fdIn) [server_connect -nobuf $env(HOST) $portNoIn]
    set oslx(fdIn) [socket $env(HOST) $portNoIn]
    Debug "Got socket fd: $oslx(fdIn)"
    if {($oslx(fdIn) == -1) || ([string first "Error:" $oslx(fdIn)] != -1)} {
	set oslx(errorStack) "$oslx(oslxERR_TCL_OPEN_CON)"
	return -1
    }
    #set oslx(fdOut) [server_connect -nobuf $env(HOST) $portNoOut]
    set oslx(fdOut) [socket $env(HOST) $portNoOut]
    Debug "Got socket fd: $oslx(fdOut)"
    if {($oslx(fdOut) == -1) || ([string first "Error:" $oslx(fdOut)] != -1)} {
	set oslx(errorStack) "$oslx(oslxERR_TCL_OPEN_CON)"
	return -1
    }

    Debug "Wait for a while to enable the OSLX Server to select()"
    after 1000
}


body oslxBASE::CloseConnection {} {
    global gvar oslx

    close $oslx(fdIn)
    close $oslx(fdOut)
    set   oslx(fdIn)   ""
    set   oslx(fdOut)  ""
}


class oslxALIAS {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	Debug "Sending ALIAS cmd: parameter = $parameter, data = $data"
	if {[SendMsg ALIAS $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {	    
	    return $oslx(reply)
	}
    }
}

class oslxCONFIG {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	if {[SendMsg CONFIG $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxDICTION {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx
	
	if {[SendMsg DICTION $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxGEN {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	if {[SendMsg GEN $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxINS_DIR {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	if {[SendMsg INSDIR $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxFILTER {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	if {[SendMsg FILTER $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxFITS {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	if {[SendMsg FITS $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxSETUP {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx
	
	if {[SendMsg OSETUP $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxSHORT_FITS {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	if {[SendMsg SHOFITS $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}

class oslxTSF {
    inherit oslxBASE
    
    public method Do {parameter data} {
	global oslx

	if {[SendMsg TSF $parameter $data] == -1} {
	    return $oslx(errorStack)
	} else {
	    return $oslx(reply)
	}
    }
}


# --- oOo ---


