/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuFITS.C,v 1.2.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *   NAME
 *   ftuBASE - Base class for the FTU library.
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *   The class is the base class for (most of) the FTU classes and provides,
 *   the basic, common things.
 *
 *   PUBLIC METHODS
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuFITS.C,v 1.2.2.1 2008/09/11 08:11:40 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include "ftuFITS.h"


ftuFITS::ftuFITS()
{ 
    _status = 0;
    _statusStr = "";
    _filePtr = 0;
    _filename = "";
    _dictionary = 0;
}

ftuFITS::~ftuFITS()
{
    if (_filePtr != 0)
	{
	fits_close_file(_filePtr, &_status);
	_filePtr = 0;
	}

}

utilSTAT ftuFITS::Open(const string &filename,
		       const int    ioMode)
{
    _filename = "";
    if ((this->_status = fits_open_file(&_filePtr, filename.c_str(), 
					ioMode, &_status)) != 0)
	return FAILURE;
    _filename = filename;

    return SUCCESS;
}


utilSTAT ftuFITS::Save(void)
{
    if (fits_flush_buffer(_filePtr, 0, &_status) != 0)
	{
	FtuErrAddGen(utilERR_FATAL, "Error saving (flushing) file: %s. "
		     "Error: %s", Filename().c_str(), GetStatusStr().c_str());
	return FAILURE;
	}
    return SUCCESS;
}
    

string    &ftuFITS::Filename(void)
{
    return _filename;
}

int  ftuFITS::GetStatus(void)
{
    return _status;
}

string &ftuFITS::GetStatusStr(void)
{
    char tmpStat[128];

    fits_get_errstatus(_status, tmpStat);
    _statusStr = tmpStat;
    return _statusStr;
}


utilSTAT ftuFITS::CheckFile(void)
{
    if (_filePtr == 0)
	{
	FtuErrAddGen(utilERR_FATAL, "No file associated to ftuFITS object");
	return FAILURE;
	}
    // IMPL: Implement better check of if file is valid etc.
    return SUCCESS;
}


void  ftuFITS::SetDictionary(oslxDICTIONARY  *dictionary)
{
    _dictionary = dictionary;
}


oslxDICTIONARY  *ftuFITS::Dictionary(void)
{
    return _dictionary;
}


int      ftuFITS::NoOfHdus(void)
{
    int noOfHdus;

    if (CheckFile() != SUCCESS) return FAILURE;
    if (fits_get_num_hdus(_filePtr, &noOfHdus, &_status) != 0)
	{
	FtuErrAddGen(utilERR_FATAL, "Error querying number of HDUs: %s",
		     GetStatusStr().c_str());
	return FAILURE;
	}
    return noOfHdus;
}


utilSTAT  ftuFITS::Mv2Hdu(const int hdu)
{
    int hduType;

    if (CheckFile() != SUCCESS) return FAILURE;
    if (fits_movabs_hdu(_filePtr, hdu, &hduType, &_status) != 0)
	{
	FtuErrAddGen(utilERR_FATAL, "Error querying number of HDUs: %s",
		     GetStatusStr().c_str());
	return FAILURE;
	}
    return SUCCESS;
}

bool ftuFITS::KeyInHdu(const string &keyName)
{
    if (CheckFile() != SUCCESS) return FAILURE;
    int   tmpStatus;
    char  tmpVal[128], tmpCom[128];
    if (fits_read_keyword(_filePtr, (char *)keyName.c_str(), tmpVal, tmpCom,
			  &tmpStatus) != 0)
	return false;
    else
	return true;
}

utilSTAT  ftuFITS::GetKey(const string &keyName,
			  string       &valStr,
			  string       &comment)
{
    if (CheckFile() != SUCCESS) return FAILURE;
    char  tmpVal[128], tmpCom[128];
    if (fits_read_keyword(_filePtr, (char *)keyName.c_str(), tmpVal, tmpCom,
			  &_status) != 0)
	{
	FtuErrAddGen(utilERR_FATAL, "Error requesting keyword: %s. Error: %s",
		     keyName.c_str(), GetStatusStr().c_str());
	return FAILURE;
	}		  
    valStr = tmpVal;
    comment = tmpCom;

    return SUCCESS;
}


utilSTAT  ftuFITS::GetKey(const string &keyName,
			  int          *val,
			  string       &comment)
{
    if (CheckFile() != SUCCESS) return FAILURE;
    string tmpVal;
    if (GetKey(keyName, tmpVal, comment) == FAILURE) return FAILURE;
    *val = atoi(tmpVal.c_str());
    return SUCCESS;
}


utilSTAT  ftuFITS::GetKey(const string &keyName,
			  double       *val,
			  string       &comment)
{
    if (CheckFile() != SUCCESS) return FAILURE;
    string tmpVal;
    if (GetKey(keyName, tmpVal, comment) == FAILURE) return FAILURE;
    *val = atof(tmpVal.c_str());
    return SUCCESS;
}


utilSTAT  ftuFITS::GenComment(const string  &keyName,
			      const string  &val,
			      string        &comment)
{
    if (Dictionary() != 0)
	{
	// This is somewhat a detour in order to get the value converted
	// if a value conversion is defined in the dictionary.
	slxFITS_VALUE  fitsValue;
	if (Dictionary()->LookUp(keyName.c_str()) == FAILURE)
	    return FAILURE;
	switch (Dictionary()->DicKeywType())
	    {
	    case slxSTRING:
		fitsValue.s = (char *)val.c_str();
		break;
	    case slxLOGICAL:
		if (val == "T")
		    fitsValue.l = 1;
		else
		    fitsValue.l = 0;
		break;
	    case slxINTEGER:
		fitsValue.i = atoi(val.c_str());
	    default:
		fitsValue.d = atof(val.c_str());
	    }
	slxKEYW_LINE   fitsLine;
	if (slxCreateFits2(Dictionary()->dicHashTable, fitsValue, 0,
			   fitsLine, &ErrStack(), keyName.c_str()) == FAILURE)
	    return FAILURE;
	slxKEYW_LINE        keyElements[8];
	vltINT32            noOfKeyElements;
	slxVALUE            tmpVal;
	slxCOMMENT          tmpCom;
	if (slxSplitFitsDyn(fitsLine, 8, keyElements, &noOfKeyElements, tmpVal,
			    tmpCom, &ErrStack()) == FAILURE)
	    return FAILURE;
	comment = tmpCom;
	}
    else
	comment = "";
    return SUCCESS;
}


utilSTAT  ftuFITS::WriteKey(const string  &keyName,
			    const string  &val,
			    const string  *comment)
{
    if (CheckFile() != SUCCESS) return FAILURE;
    string comLoc;
    if (comment != 0)
	comLoc = *comment;
    else
	{
	if (GenComment(keyName, val, comLoc) == FAILURE) return FAILURE;
	}
    if (fits_update_key(_filePtr, TSTRING, (char *)keyName.c_str(), 
			(char *)val.c_str(), (char *)comLoc.c_str(),
			&_status) != 0)
	{
	FtuErrAddGen(utilERR_FATAL, "Error writing comment: %s in file: %s. "
		     "Error: %s", comLoc.c_str(), Filename().c_str(), 
		     GetStatusStr().c_str());
	return FAILURE;
	}

    return SUCCESS;
}


utilSTAT  ftuFITS::WriteComment(const string &comment)
{
    if (CheckFile() != SUCCESS) return FAILURE;
    if (fits_write_comment(_filePtr, comment.c_str(), &_status) != 0)
	{
	FtuErrAddGen(utilERR_FATAL, "Error writing comment: %s in file: %s. "
		     "Error: %s", comment.c_str(), Filename().c_str(), 
		     GetStatusStr().c_str());
	return FAILURE;
	}
    return SUCCESS;
}


utilSTAT  ftuFITS::RemoveKey(const string  &keyName)
{
    if (CheckFile() != SUCCESS) return FAILURE;
    if (fits_delete_key(_filePtr, (char *)keyName.c_str(), &_status) != 0)
	{
	FtuErrAddGen(utilERR_FATAL, "Error removing key: %s from file: %s. "
		     "Error: %s", keyName.c_str(), Filename().c_str(), 
		     GetStatusStr().c_str());
	return FAILURE;
	}

    return SUCCESS;
}


// EOF
