/****************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuFTT.C,v 1.1.2.2 2009/12/22 09:51:20 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *  NAME
 *  ftuFTT - Class to handle a FITS Translation Table (FTT).
 * 
 *  SYNOPSIS
 *   
 *  PARENT CLASS
 * 
 *  DESCRIPTION
 *  The class is used to parse and to access the information contained
 *  in an FTT.
 *
 *  PUBLIC METHODS
 *
 *  PUBLIC DATA MEMBERS
 *
 *  PROTECTED METHODS
 *
 *  PROTECTED DATA MEMBERS
 *
 *  FILES
 *
 *  ENVIRONMENT
 *
 *  RETURN VALUES
 *
 *  CAUTIONS 
 *
 *  EXAMPLES
 *
 *  SEE ALSO
 *
 *  BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuFTT.C,v 1.1.2.2 2009/12/22 09:51:20 cguirao Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#define _ALL_SOURCE 1
#define __EXTENSIONS__ 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "utilPAF.h"

#include "ftuDefs.h"
#include "ftuFTT.h"
#include "ftuTRANSLATION.h"
#include "ftuSwap.h"


// Callbacks to handle deletion of varios class when stored in hash tables.
void fttFileNameSetHashDeleteEl(void *elPtr) {
    delete (fttFILENAME_SET *)elPtr;
}
void fttOslxFitsDeleteEl(void *elPtr) {
    delete (oslxFITS *)elPtr;
}


ftuFTT::ftuFTT(ftuCONDITIONS  &conditions)
{
    if (BuildFttDic() == FAILURE)
	{
	printf("\nProblem initializing FTU environment!\n\n");
	errPrint(&ErrStack());
	exit(1);
	}
    fttTbl_.DeleteTable();

    char      *bufPtr;
    slxVALUE  date1, date2;
    slxGenDayTim(date1);
    if (slxGetNextField2('T', date1, sizeof(slxVALUE), date2, &bufPtr,
			 &ErrStack()) == FAILURE)
	errCloseStack(&ErrStack());
    Date(date2);

    conditions_ = &conditions; 
    *fttFileName_ = '\0';
    SetDumpHdr(ftuHDR_DUMP_NONE);
    ResetNoOfTransCount();
    ResetFileIndex();
}


ftuFTT::~ftuFTT()
{
}


ftuCONDITIONS  &ftuFTT::Conditions()
{
    return *conditions_;
}


utilSTAT ftuFTT::BuildFttDic()
{
    char *fttKeyList[] = {"INCLUDE", "DICTIONARY",
			   
			  "DIC.KEY", "DIC.TYPE", "DIC.FORMAT", "DIC.UNIT", 
			  "DIC.COMMENT",
			  
			  "HDR.DUMP", 

			  "FILE.SPLIT.PREFIX", "FILE.SPLIT.HDUS",
			  "FILE.SPLIT.CONCAT.KEY", 
			  "FILE.SPLIT.CONCAT.KEY.HDR",
			  "FILE.SPLIT.CONCAT.CONST", "FILE.SPLIT.CONCAT.FCT", 

			  "FILE.RENAME.PREFIX", "FILE.RENAME.CONCAT.KEY", 
			  "FILE.RENAME.CONCAT.KEY.HDR",
			  "FILE.RENAME.CONCAT.CONST", "FILE.RENAME.CONCAT.FCT",
			  
			  "FILE.PATH.BASE", "FILE.PATH.CONCAT.KEY",
			  "FILE.PATH.CONCAT.KEY.HDR",
			  "FILE.PATH.CONCAT.CONST", "FILE.PATH.CONCAT.FCT",

			  "ADD.KEY", "ADD.HDR", "ADD.VALUE.KEY",
			  "ADD.VALUE.FCT", "ADD.VALUE.CONST", "ADD.FCT", 
			  
			  "CHANGE.KEY", "CHANGE.HDR", "CHANGE.VALUE.KEY",
			  "CHANGE.VALUE.FCT", "CHANGE.VALUE.CONST", 
			  "CHANGE.FCT",
			  
			  "RENAME.SRCKEY", "RENAME.TRGKEY", "RENAME.HDR",
			  "RENAME.FCT",
			  
			  "MOVE.KEY", "MOVE.SRCHDR", "MOVE.TRGHDR", "MOVE.FCT",
			  
			  "COPY.SRCKEY", "COPY.SRCHDR", "COPY.TRGKEY",
			  "COPY.TRGHDR", "COPY.FCT",
			  
			  "REMOVE.KEYLIST", "REMOVE.HDR", NULL};

    for (int i = 0; fttKeyList[i] != NULL; i++) 
	{
	if (fttDic_.StoreKeyword(fttKeyList[i], "private", "FTT", "string",
				 "%s", "", "FTT Key", "FTT Key", 
				 FALSE) == FAILURE)
	    return FAILURE;
	}

    return SUCCESS;
}


utilSTAT ftuFTT::BuildFitsDic()
{
    char *fitsKeyList[] = {"CHECKSUM", "string", "%s", 
			   "ASCII 1's complement checksum",

			   "DATAMIN", "double", "%.6f",
			   "Minimum pixel value",

			   "DATAMAX", "double", "%.6f",
			   "Maximum pixel value",

			   "DATAMEAN", "double", "%.6f",
			   "Mean Pixel Value",

			   "DATAMED", "double", "%.6f",
			   "Median Pixel Value",

			   "DATARMS", "double", "%.6f",
			   "RMS of Pixel Values", 

			   "COMMENT", "string", "%s", 
			   "Dummy DID Entry",

			   NULL};

    // Add records for the keywords that might be added by the FTU. If these
    // are redefined in a dictionary being loaded, the definition here will
    // be overwritten.
    for (int j = 0; fitsKeyList[j] != NULL; j += 4)
	{
	if (fitsDicStat_.StoreKeyword(fitsKeyList[j], "header", "FTT", 
				      fitsKeyList[j+1],fitsKeyList[j+2],"", 
				      fitsKeyList[j+3],"",FALSE) == FAILURE)
	    return FAILURE;
	}    

    return SUCCESS;
}


utilSTAT         ftuFTT::Load(const slxFILENAME  fttFile,
			      const int          recursiveCall)
{
    utilPAF  fttPaf;
    
    fttPaf.AllowRepeatedKeys(oslxREP_ALLOW_ALL_KEYS);

    // If it is a recursive call, i.e., when an FTT is included into another,
    // we should not delete the table and FITS Dictionary.
    if (!recursiveCall)
	{
	fttTbl_.DeleteTable();
	fitsDicStat_.ResetDic();
	fitsDicDyn_.ResetDic();
	BuildFitsDic();
	rename_.Clear();
	split_.Clear();
	path_.Clear();
	splitSelHdus_ = 0;
	}

    fttPaf.SetDictionary(&fttDic_);

    if (*fttFile == '\0')
	return SUCCESS;

    if (fttPaf.Load(fttFile) == FAILURE)
	{
	if (errErrInStack("slxERR_LOCATE_FILE_GEN", &ErrStack()))
	    {
	    sprintf(errBuf_, "Could not open or locate FTT: %s.", fttFile);
	    ErrStackReset();
	    }
	else
	    sprintf(errBuf_, "Could not read FTT: %s.", fttFile);
	ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	return FAILURE;
	}

    slxFILENAME    path, fn, ext;
    *ext = '\0';
    utilFILE_MGR::SplitFileName(fttFile, path, fn, ext);
    if (*ext != '\0')
	sprintf(fttFileName_, "%s.%s", fn, ext);
    else
	strcpy(fttFileName_, fn);

    fttPaf.ResetKeyPtr();
    fttPaf.NextKey();
    while (fttPaf.TimesNextKeyCalled() <= fttPaf.NumberOfKeywords())
	{
	if (strcmp(fttPaf.CK().Keyword(), "INCLUDE") == 0)
	    {
	    if (HandleIncludeKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "DICTIONARY") == 0)
	    {
	    if (HandleDictionaryKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "DIC.KEY") == 0)
	    {
	    if (HandleDicKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "HDR.DUMP") == 0)
	    {
	    if (HandleHdrKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "FILE.SPLIT.PREFIX") == 0)
	    {
	    if (HandleFileSplitKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "FILE.RENAME.PREFIX") == 0)
	    {
	    if (HandleFileRenameKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "FILE.PATH.BASE") == 0)
	    {
	    if (HandleFilePathKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "ADD.KEY") == 0)
	    {
	    if (HandleAddKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "CHANGE.KEY") == 0)
	    {
	    if (HandleChangeKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "RENAME.SRCKEY") == 0)
	    {
	    if (HandleRenameKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "MOVE.KEY") == 0)
	    {
	    if (HandleMoveKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "COPY.SRCKEY") == 0)
	    {
	    if (HandleCopyKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else if (strcmp(fttPaf.CK().Keyword(), "REMOVE.KEYLIST") == 0)
	    {
	    if (HandleRemoveKey(fttPaf) == FAILURE)
		return FAILURE;
	    }
	else
	    {
	    sprintf(errBuf_, "Unknown tag %s found in FTT %s",
		    fttPaf.CK().Keyword(), fttFile);
	    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	    return FAILURE;
	    }
	}   // end-while (fttPaf.KeysReturned() <= fttPaf.NumberOfKeywords())

    return SUCCESS;
}


char            *ftuFTT::FttFileName()
{
    return fttFileName_;
}


utilSTAT  ftuFTT::HandleIncludeKey(utilPAF &fttPaf) 
{
    // Expect: INCLUDE
    slxFILENAME  fttName;
    strcpy(fttName, fttPaf.CK().String());
    slxTrimString(fttName, " \t\n");
    INFO(1,"Handling include statement - including FTT: %s.",fttName);
    chronometer_.Start("IncludeFtt");
    if (Load(fttName, TRUE) == FAILURE)
	return FAILURE;
    INFO(1,"Included FTT: %s. Time: %f.",fttName, 
	 chronometer_.Stop("IncludeFtt"));
    fttPaf.NextKey();

    return SUCCESS;
}    


utilSTAT  ftuFTT::HandleDictionaryKey(utilPAF &fttPaf)
{  
    char  *completePath;
    if (slxResolvePath(fttPaf.CK().String(), &completePath, 
		       &ErrStack()) == FAILURE)
	return FAILURE;
    slxFILENAME  dicName;
    strcpy(dicName, completePath);
    slxTrimString(dicName, " \n\t");
    INFO(2,"Loading dictionary: %s.",dicName);
    if (fitsDicStat_.LoadDic(dicName, TRUE) == FAILURE)
	return FAILURE;
    fttPaf.NextKey();

    return SUCCESS;
}


utilSTAT        ftuFTT::HandleDicKey(utilPAF &fttPaf)
{
    slxKEYW_LINE  key, type, format, unit, comment;


    // Expect: "DIC.KEY", "DIC.TYPE", "DIC.FORMAT", ["DIC.TYPE",] "DIC.COMMENT"

    // Get keyword name.
    strcpy(key, fttPaf.CK().String());
    slxTrimString(key, " \n\t");

    // Get keyword type.
    fttPaf.NextKey();
    if (CheckExpKey("DIC.TYPE", fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    strcpy(type, fttPaf.CK().String());
    slxTrimString(type, " \n\t");

    // Get keyword value format.
    fttPaf.NextKey();
    if (CheckExpKey("DIC.FORMAT", fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    strcpy(format, fttPaf.CK().String());
    slxTrimString(format, " \n\t");

    // Get type if defined.
    fttPaf.NextKey();
    if (strcmp(fttPaf.CK().Keyword(), "DIC.UNIT") == 0)
        {
	strcpy(unit, fttPaf.CK().String());
	slxTrimString(unit, " \n\t");
	fttPaf.NextKey();
	}
    else
        *unit = '\0';

    // Get keyword comment.
    if (CheckExpKey("DIC.COMMENT", fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    strcpy(comment, fttPaf.CK().String());
    slxTrimString(comment, " \n\t");
    fttPaf.NextKey();

    // Add the keyword in the FITS dictionary.
    INFO(3,"Adding Key: %s, Type: %s, Format: %s, Unit: %s, "
	 "Comment: %s, in FITS Dictionary.", key, type, format, unit, comment);
    if (fitsDicStat_.StoreKeyword(key, "header", "FTT", type, format,
				  unit, comment, NULL, FALSE) == FAILURE)
        return FAILURE;

    /* REMOVE THIS:
    if (0)
        {
	// IMPL.: Add this place we would like to use the method
	// oslxDICTIONARY::StoreKeyword() to add the keyword. Unfortunately
	// The Value Conversion is not handled properly in this method.
	// We therefore make a small, temporary DID in an slxDynBuf and
	// use some basic functions to handle this. This could be changed
	// later so that oslxDICTIONARY::StoreKeyword() is used directly
	// if this is fixed.
	
	// A keyword record in ASCII format is e.g. like this:
	//
	// Parameter Name:    RA
	// Class:             header
	// Context:           FITS
	// Type:              double
	// Value Format:      %.6f
	// Unit:              deg
	// Comment Field:     %HOURANG RA (J2000) pointing
	// Description:	      ---

	// Replace dots with blanks.
	char  *chrPtr;
	chrPtr = key;
	while (*chrPtr != '\0')
	    {
	    if (*chrPtr == '.') *chrPtr = ' ';
	    chrPtr++;
	    }
	char    tmpBuf[1024];
	memset(tmpBuf, 0, sizeof(tmpBuf));
	sprintf(tmpBuf, 
		"Parameter Name:    %s\n"
		"Class:             header\n"
		"Context:           FITS\n"
		"Type:              %s\n"
		"Value Format:      %s\n"
		"Unit:              %s\n"
		"Comment Field:     %s\n"
		"Description:       -FTU-GENERATED-DEF-\n", 
		key, type, format, unit, comment);

	printf("\n>>>>>>> DicBuf=\n%s\n", tmpBuf);

	if (slxStoreBufDynBuf(&tmpDicBuf, tmpBuf, strlen(tmpBuf), 
			      &ErrStack()) == FAILURE)
	    goto errExit;
	if (slxInstallDicBuf(&tmpDicBuf, fitsDicStat_.dicHashTable, NULL, 
			     &ErrStack()) == FAILURE)
	    goto errExit;
	}
    */

    return SUCCESS;
}


utilSTAT  ftuFTT::CheckExpKey(const slxKEYW_LINE   expKeys,
			      const slxKEYW_LINE   actualKey)
{
    if (strstr(expKeys, actualKey) == NULL)
	{
	sprintf(errBuf_, "Unknown key encountered. Expected (one of) %s, "
		"found %s.", expKeys, actualKey);
	ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	return FAILURE;
	}
    return SUCCESS;
}


utilSTAT        ftuFTT::HandleHdrKey(utilPAF &fttPaf)
{
    SetDumpHdr(ftuHDR_DUMP_NONE);
 
    slxTrimString((char *)fttPaf.CK().String(), " \n");
    if (strcmp(fttPaf.CK().String(), ftuHDR_DUMP_NL_STR) == 0)
	SetDumpHdr(ftuHDR_DUMP_NL);
    else if (strcmp(fttPaf.CK().String(), ftuHDR_DUMP_NO_NL_STR) == 0)
	SetDumpHdr(ftuHDR_DUMP_NO_NL);
    else
	{
	sprintf(errBuf_, "Illegal tag %s for key HDR.DUMP.",
		fttPaf.CK().String());
	INFO(1,errBuf_);
	ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	return FAILURE;
	}
    fttPaf.NextKey();

    return SUCCESS;
}


utilSTAT        ftuFTT::HandleFileSplitRenamePathKey(utilPAF          &fttPaf,
						     ftuTRANSLATION   &trans,
						     const ftuACTION  transId,
						     const char       *initKey)
{
    if (((transId == ftuFILE_SPLIT) && 	
	 (rename_.GetAction() == ftuFILE_RENAME)) ||
	((transId == ftuFILE_RENAME) && (split_.GetAction() == ftuFILE_SPLIT)))
	{
	sprintf(errBuf_, "Cannot use File Splitting and File Renaming in the "
		"same FTT.");
	goto errorExit;
	}

    // Check that we have only one initiator of each type of file action.
    if ((transId == ftuFILE_SPLIT) && (split_.GetAction() == ftuFILE_SPLIT))
	{
	sprintf(errBuf_, "File splitting defined repeatedly. Remove "
		"additional occurrences of FILE.SPLIT.PREFIX.");
	goto errorExit;
	}
    else if ((transId == ftuFILE_RENAME) && 
	     (rename_.GetAction() == ftuFILE_RENAME))
	{
	sprintf(errBuf_, "File renaming defined repeatedly. Remove "
		"additional occurrences of FILE.RENAME.PREFIX.");
	goto errorExit;
	}
    else if ((transId == ftuFILE_PATH) &&
	     (path_.GetAction() == ftuFILE_PATH))
	{
	sprintf(errBuf_, "File path defined repeatedly. Remove "
		"additional occurrences of FILE.PATH.BASE.");
	goto errorExit;
	}

    trans.SetAction(transId);

    /* Expect:
     *    FILE.SPLIT/RENAME.PREFIX|FILE.PATH.BASE, possibly in conjunction with
     *    FILE.SPLIT.HDUS
     *    FILE.SPLIT/RENAME/PATH.CONCAT.KEY
     *    [FILE.SPLIT/RENAME.CONCAT.KEY.HDR|FILE.PATH.CONCAT.KEY.HDR]
     *    FILE.SPLIT/RENAME/PATH.CONCAT.CONST
     *    FILE.SPLIT/RENAME/PATH.CONCAT.FCT
     */
    if (CheckExpKey(initKey, fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    if (trans.AddValueSrcAdr(fttPaf.CK().Keyword(),
			     fttPaf.CK().String()) == FAILURE)
	return FAILURE;
    slxKEYW_LINE  prevKey;
    *prevKey = '\0';
    fttPaf.NextKey();
    while ((fttPaf.TimesNextKeyCalled() <= fttPaf.NumberOfKeywords()) &&
	   ((strstr(fttPaf.CK().Keyword(), ".CONCAT.KEY") != NULL)||
	    (strstr(fttPaf.CK().Keyword(), ".CONCAT.FCT") != NULL) ||
	    (strstr(fttPaf.CK().Keyword(), ".CONCAT.CONST") != NULL) ||
	    ((transId == ftuFILE_SPLIT) && 
	     (strstr(fttPaf.CK().Keyword(),"FILE.SPLIT.HDUS") != NULL))))
	{
	slxKEYW_LINE keyword;
	strcpy(keyword, fttPaf.CK().Keyword());

	// Check that there are no repeated keys like e.g.:
	// FILE.PATH.CONCAT.KEY.HDR
	// FILE.PATH.CONCAT.KEY.HDR
	if ((strstr(keyword, ".CONCAT.KEY.HDR") != NULL) &&
	    (strstr(prevKey, ".CONCAT.KEY.HDR") != NULL))
	    {
	    sprintf(errBuf_, "Sequence of keywords %s->%s is illegal. "
		    "Rejecting FTT.", prevKey, keyword);
	    goto errorExit;
	    }

	slxVALUE     value;
	int          header = -1;
	strcpy(value, fttPaf.CK().String());

	// Check if it's a FILE.SPLIT.HDUS key.
	if (strstr(keyword, "FILE.SPLIT.HDUS") != NULL)
	    {
	    slxKEYW_LINE     hdus[64];
	    vltINT32         noOfHdus;
	    if (slxSplit(",", value, 64, hdus, &noOfHdus,
			 &ErrStack()) == FAILURE)
		goto errorExit;
	    for (int hduNo = 0; hduNo < noOfHdus; hduNo++)
		hduList_[hduNo] = atoi(hdus[hduNo]);
	    splitSelHdus_ = 1;
	    fttPaf.NextKey();
	    }
	else
	    {
	    // If the next keyword is *.*.CONCAT.HDR, we should also add the
	    // source. A source of -1 means the current header or that it is
	    // not relevant to refer to a specific header in the context.
	    fttPaf.NextKey();
	    if (strstr(fttPaf.CK().Keyword(), ".CONCAT.KEY.HDR") != NULL)
		{
		header = atoi(fttPaf.CK().String());
		fttPaf.NextKey();
		}
	    if (trans.AddValueSrcAdr(keyword, value, header) == FAILURE)
		return FAILURE;
	    }
	strcpy(prevKey, keyword);
	}   // end-while ((fttPaf.TimesNextKeyCalled() <= ...

    return SUCCESS;

  errorExit:
    INFO(1,errBuf_);
    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
    return FAILURE;
}


utilSTAT        ftuFTT::HandleFileSplitKey(utilPAF &fttPaf)
{
    if (HandleFileSplitRenamePathKey(fttPaf, split_, ftuFILE_SPLIT,
				     "FILE.SPLIT.PREFIX") == FAILURE)
	return FAILURE;

    return SUCCESS;
}
   

utilSTAT        ftuFTT::HandleFileRenameKey(utilPAF &fttPaf)
{
    if (HandleFileSplitRenamePathKey(fttPaf, rename_, ftuFILE_RENAME,
				     "FILE.RENAME.PREFIX") == FAILURE)
	return FAILURE;

    return SUCCESS;
}


utilSTAT        ftuFTT::HandleFilePathKey(utilPAF &fttPaf)
{
    if (HandleFileSplitRenamePathKey(fttPaf, path_, ftuFILE_PATH,
				     "FILE.PATH.BASE") == FAILURE)
	return FAILURE;   

    return SUCCESS;
}


utilSTAT ftuFTT::AddValueSrcs(utilPAF          &fttPaf,
			      ftuTRANSLATION   *transEl)
{     
    while ((fttPaf.TimesNextKeyCalled() <= fttPaf.NumberOfKeywords()) &&
	   ((strstr(fttPaf.CK().Keyword(), ".VALUE.KEY") != NULL)||
	    (strstr(fttPaf.CK().Keyword(), ".VALUE.FCT") != NULL) ||
	    (strstr(fttPaf.CK().Keyword(), ".VALUE.CONST") != NULL)))
	{
	slxTrimString((char *) fttPaf.CK().Keyword(), " \n\t");
	slxTrimString((char *) fttPaf.CK().String(),  " \n\t");
	INFO(3,"Adding Value Source: %s and Reference: %s", 
	     fttPaf.CK().Keyword(), fttPaf.CK().String());
	if (transEl->AddValueSrcAdr(fttPaf.CK().Keyword(), 
				    fttPaf.CK().String()) == FAILURE)
	    return FAILURE;
	fttPaf.NextKey();
	}

    return SUCCESS;
}


utilSTAT        ftuFTT::HandleAddKey(utilPAF &fttPaf)
{
    /* Expect:
     *   ADD.KEY,ADD.HDR,(ADD.VALUE.KEY|ADD.VALUE.FCT|
     *   ADD.VALUE.CONST)+,[ADD.FCT]
     */
    ftuTRANSLATION *transTmp = new ftuTRANSLATION(ftuADD);
    
    // Set keyword.
    transTmp->SetSrcKey(fttPaf.CK().String());
    
    // Get/set header reference.
    fttPaf.NextKey();
    if (CheckExpKey("ADD.HDR", fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    if (transTmp->SetSrcHdr(fttPaf.CK().String()) == FAILURE)
	return FAILURE;
    
    // Handle set source/address key(s).
    fttPaf.NextKey();
    if (CheckExpKey("ADD.VALUE.KEY,ADD.VALUE.FCT,ADD.VALUE.CONST", 
		    fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    else
	{
	if (AddValueSrcs(fttPaf, transTmp) == FAILURE)
	    return FAILURE; 
	}
    
    // Handle the ADD.FCT key.
    if (strcmp(fttPaf.CK().Keyword(), "ADD.FCT") == 0)
	{
	if (transTmp->SetFct(fttPaf.CK().String()) == FAILURE)
	    return FAILURE;
	fttPaf.NextKey();
	}
    
    // Add the translation rule in the translation hash table.
    if (fttTbl_.AddEl(fttTbl_.NoOfEls(), (void **)&transTmp) == FAILURE)
	return FAILURE;
    
    return SUCCESS;
}


utilSTAT        ftuFTT::HandleChangeKey(utilPAF    &fttPaf)
{
    /* Expect:
     *   CHANGE.KEY,CHANGE.HDR,(CHANGE.VALUE.KEY|
     *   CHANGE.VALUE.FCT|CHANGE.VALUE.CONST)+,[CHANGE.FCT]
     */
    ftuTRANSLATION *transTmp = new ftuTRANSLATION(ftuCHANGE);

    // Set keyword.
    transTmp->SetSrcKey(fttPaf.CK().String());

    // Get/set header reference.
    fttPaf.NextKey();
    if (CheckExpKey("CHANGE.HDR", fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    if (transTmp->SetSrcHdr(fttPaf.CK().String()) == FAILURE)
	return FAILURE;

    // Handle set source/address key(s).
    fttPaf.NextKey();
    if (CheckExpKey("CHANGE.VALUE.KEY,CHANGE.VALUE.FCT,CHANGE.VALUE.CONST", 
		    fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    else
	{
	if (AddValueSrcs(fttPaf, transTmp) == FAILURE)
	    return FAILURE; 
	}

    // Handle the CHANGE.FCT key.
    if (strcmp(fttPaf.CK().Keyword(), "CHANGE.FCT") == 0)
	{
	if (transTmp->SetFct(fttPaf.CK().String()) == FAILURE)
	    return FAILURE;
	fttPaf.NextKey();
	}

    // Add the translation rule in the translation hash table.
    if (fttTbl_.AddEl(fttTbl_.NoOfEls(), (void **)&transTmp) == FAILURE)
	return FAILURE;
  
    return SUCCESS;
}


utilSTAT        ftuFTT::HandleRenameKey(utilPAF   &fttPaf)
{
    // Expect: RENAME.SRCKEY,RENAME.TRGKEY,RENAME.HDR,RENAME.FCT
    ftuTRANSLATION *transTmp = new ftuTRANSLATION(ftuRENAME);

    // Set source keyword.
    transTmp->SetSrcKey(fttPaf.CK().String());

    // Set target keyword.
    fttPaf.NextKey();
    if (CheckExpKey("RENAME.TRGKEY", fttPaf.CK().Keyword()) == FAILURE)
	goto cleanExit;
    if (transTmp->SetTargKey(fttPaf.CK().String()) == FAILURE)
	goto cleanExit;

    // Set header reference.
    fttPaf.NextKey();
    if (CheckExpKey("RENAME.HDR", fttPaf.CK().Keyword()) == FAILURE)
	goto cleanExit;
    if (transTmp->SetSrcHdr(fttPaf.CK().String()) == FAILURE)
	goto cleanExit;
    
    // Add Value Source.
    if (transTmp->AddValueSrcAdr(".KEY", transTmp->GetSrcKey()) == FAILURE)
	goto cleanExit;

    // Check if a function should be applied.
    fttPaf.NextKey();
    if (strcmp(fttPaf.CK().Keyword(), "RENAME.FCT") == 0)
	{
	if (transTmp->SetFct(fttPaf.CK().String()) == FAILURE)
	    goto cleanExit;
	fttPaf.NextKey();
	}

    // Add the translation rule in the translation hash table.
    if (fttTbl_.AddEl(fttTbl_.NoOfEls(), (void **)&transTmp) == FAILURE)
	goto cleanExit;
    
    return SUCCESS;

  cleanExit:
    delete transTmp;
    return FAILURE;
}


utilSTAT        ftuFTT::HandleMoveKey(utilPAF   &fttPaf)
{
    // Expect: MOVE.KEY,MOVE.SRCHDR,MOVE.TRGHDR,MOVE.FCT
    ftuTRANSLATION *transTmp = new ftuTRANSLATION(ftuMOVE);

    // Get keyword.
    transTmp->SetSrcKey(fttPaf.CK().String());
   
    // Get/set source header.
    fttPaf.NextKey();
    if (CheckExpKey("MOVE.SRCHDR", fttPaf.CK().Keyword()) == FAILURE)
	goto cleanExit;
    if (transTmp->SetSrcHdr(fttPaf.CK().String()) == FAILURE)
	goto cleanExit;

    // Set the value-source.
    if (transTmp->AddValueSrcAdr("MOVE.VALUE.KEY",
				 transTmp->GetSrcKey()) == FAILURE)
	return FAILURE;

    // Get/set target header.
    fttPaf.NextKey();
    if (CheckExpKey("MOVE.TRGHDR", fttPaf.CK().Keyword()) == FAILURE)
	goto cleanExit;
    if (transTmp->SetTargHdr(fttPaf.CK().String()) == FAILURE)
	goto cleanExit;

    // Check if a function should be applied.
    fttPaf.NextKey();
    if (strcmp(fttPaf.CK().Keyword(), "MOVE.FCT") == 0)
	{
	if (transTmp->SetFct(fttPaf.CK().String()) == FAILURE)
	    goto cleanExit;
	fttPaf.NextKey();
	}

    // Add the translation rule in the translation hash table.
    if (fttTbl_.AddEl(fttTbl_.NoOfEls(), (void **)&transTmp) == FAILURE)
	goto cleanExit;

    return SUCCESS;

  cleanExit:
    delete transTmp;
    return FAILURE;
}


utilSTAT        ftuFTT::HandleCopyKey(utilPAF   &fttPaf)
{
    // Expect: COPY.SRCKEY,COPY.SRCHDR,COPY.TRGKEY,COPY.TRGHDR,COPY.FCT
    ftuTRANSLATION *transTmp = new ftuTRANSLATION(ftuCOPY);

    // Get keyword.
    slxKEYW_LINE  srcKey;
    strcpy(srcKey, fttPaf.CK().String());
    transTmp->SetSrcKey(fttPaf.CK().String());
   
    // Get source header.
    fttPaf.NextKey();
    if (CheckExpKey("COPY.SRCHDR", fttPaf.CK().Keyword()) == FAILURE)
	goto cleanExit;
    if (transTmp->SetSrcHdr(fttPaf.CK().String()) == FAILURE)
	goto cleanExit;

    // Get target keyword.
    fttPaf.NextKey();
    if (CheckExpKey("COPY.TRGKEY", fttPaf.CK().Keyword()) == FAILURE)
	goto cleanExit;
    if (transTmp->SetTargKey(fttPaf.CK().String()) == FAILURE)
	goto cleanExit;

     // Get target header.
    fttPaf.NextKey();
    if (CheckExpKey("COPY.TRGHDR", fttPaf.CK().Keyword()) == FAILURE)
	goto cleanExit;
    if (transTmp->SetTargHdr(fttPaf.CK().String()) == FAILURE)
	goto cleanExit;

    // Indicate that we take the value from the source keyword. This
    // is a trick to make the SW handle this case as the other cases.
    if (transTmp->AddValueSrcAdr("COPY.VALUE.KEY", srcKey) == FAILURE)
	return FAILURE;

    // Check if a function should be applied.
    fttPaf.NextKey();
    if (strcmp(fttPaf.CK().Keyword(), "COPY.FCT") == 0)
	{
	if (transTmp->SetFct(fttPaf.CK().String()) == FAILURE)
	    goto cleanExit;
	fttPaf.NextKey();
	}

    // Add the translation rule in the translation hash table.
    if (fttTbl_.AddEl(fttTbl_.NoOfEls(), (void **)&transTmp) == FAILURE)
	goto cleanExit;

    return SUCCESS;

  cleanExit:
    delete transTmp;
    return FAILURE;
}


utilSTAT        ftuFTT::HandleRemoveKey(utilPAF  &fttPaf)
{
    // Expect: REMOVE.KEYLIST,REMOVE.HDR

    // Get keyword(s). The list of keyword is separated by comma.
    slxKEYW_LINE     keys[64];
    vltINT32         noOfSubStr;
    if (slxSplit(", ", fttPaf.CK().String(), 64, keys, &noOfSubStr,
		 &ErrStack()) == FAILURE)
	return FAILURE;
  
    // Get header.
    fttPaf.NextKey();
    if (CheckExpKey("REMOVE.HDR", fttPaf.CK().Keyword()) == FAILURE)
	return FAILURE;
    slxKEYW_LINE  header;
    strcpy(header, fttPaf.CK().String());
    fttPaf.NextKey();

    // Add a translation element for each key to be removed.
    ftuTRANSLATION *transTmp;
    for (int keyNo = 0; keyNo < noOfSubStr; keyNo++)
	{
	transTmp = new ftuTRANSLATION(ftuREMOVE);

	transTmp->SetSrcKey(keys[keyNo]);
	if (transTmp->SetSrcHdr(header) == FAILURE)
	    goto cleanExit;

	// Add the translation rule in the translation hash table.
	if (fttTbl_.AddEl(fttTbl_.NoOfEls(), (void **)&transTmp) == FAILURE)
	    goto cleanExit;
	}   // end-for (int keyNo = 0; keyNo < noOfSubStr; keyNo++)

    return SUCCESS;

  cleanExit:
    delete transTmp;
    return FAILURE;
}


int             ftuFTT::NoOfTranslations()
{
    return fttTbl_.NoOfEls();
}


ftuTRANSLATION *ftuFTT::GetTranslation(const int number)
{
    ftuTRANSLATION  *transPtr;

    if (fttTbl_.GetEl(number, (void **)&transPtr) == FAILURE)
	{
	ErrStackReset();
	return NULL;
	}
    return transPtr;
}


utilLOGICAL ftuFTT::Split(oslxFITS  &fitsObj)
{
    return ((split_.GetAction() == ftuFILE_SPLIT) && 
	    (fitsObj.NoOfHdrs() > 1));
}



void                ftuFTT::SetDumpHdr(const ftuHDR_DUMP_TYPES dump)
{
    dumpHdr_ = dump;
}


ftuHDR_DUMP_TYPES   ftuFTT::GetDumpHdr()
{
    return dumpHdr_;
}


/*___oOo___*/
