/****************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuFTT_Calc.C,v 1.1.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *  NAME
 *  ftuFTT_Cakc - Contains methods to handle the calculations needed for
 *                for the translation.
 * 
 *  SYNOPSIS
 *   
 *  PARENT CLASS
 * 
 *  DESCRIPTION
 *
 *  PUBLIC METHODS
 *
 *  PUBLIC DATA MEMBERS
 *
 *  PROTECTED METHODS
 *
 *  PROTECTED DATA MEMBERS
 *
 *  FILES
 *
 *  ENVIRONMENT
 *
 *  RETURN VALUES
 *
 *  CAUTIONS 
 *
 *  EXAMPLES
 *
 *  SEE ALSO
 *
 *  BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuFTT_Calc.C,v 1.1.2.1 2008/09/11 08:11:40 cguirao Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#define _ALL_SOURCE 1
#define __EXTENSIONS__ 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "utilPAF.h"

#include "ftuDefs.h"
#include "ftuFTT.h"
#include "ftuTRANSLATION.h"
#include "ftuSwap.h"

extern "C" {
    #include "mmap_i.h"

    ftuINT8  median_pixelvalue_INT8(ftuINT8  *a, int n);
    ftuINT16 median_pixelvalue_INT16(ftuINT16 *a, int n);
    ftuINT32 median_pixelvalue_INT32(ftuINT32 *a, int n);
    ftuFP32  median_pixelvalue_FP32(ftuFP32  *a, int n);
    ftuFP64  median_pixelvalue_FP64(ftuFP64  *a, int n);
}


utilSTAT ftuFTT::GetDataSize(oslxFITS               &fits,
			     int                    *dataSize)
{
    int     imageSize;
    int     bitPix;
    int     naxis;
    double  bscale;
    double  bzero;
    if (ImageInfo(fits, &imageSize, dataSize, &bitPix, &naxis, &bscale,
		  &bzero) == FAILURE)
	return FAILURE;

    return SUCCESS;
}


utilSTAT  ftuFTT::GetFctValue(const char *function,
			      slxVALUE   result)
{
    slxVALUE dummyValues[16];

    if (ComputeValue(function, 0, dummyValues, result) == FAILURE)
	return FAILURE;

    return SUCCESS;
}


utilSTAT  ftuFTT::ComputeValue(const char      *function,
			       const int       noOfValues,
			       const slxVALUE  values[],
			       slxVALUE        result)
{
    
    // Check if Built-In or a Plug-In Conversion Function we are about to
    // invoke (Plug-In = shell commands, have prefix "SHELL:<function name>".
    if (strstr(function, "SHELL:") == function)
	{
	// Get the function name.
	slxVALUE fctName;
	char     *bufPtr;
	bufPtr = (char *)function;
	for (int i = 0; i < 2; i++)
	    {
	    if (slxGetNextField2(':', bufPtr, sizeof(slxVALUE),
				 fctName, &bufPtr, &ErrStack()) == FAILURE)
		return FAILURE;
	    }

	char  execBuf[1024];
	strcpy(execBuf, fctName);
	for (int valNo = 0; valNo < noOfValues; valNo++)
	    {
	    strcat(execBuf, " ");
	    strcat(execBuf, values[valNo]);
	    }
	if (ExecPlugIn(execBuf, result) == FAILURE)
	    return FAILURE;

	return SUCCESS;
	}
    else
	{
	// We indentify the function.
	INFO(3,"Invoking Internal Function: %s.",function);
	if (strcmp(function, "ConcatString") == 0) 
	    strcpy(result, ConcatString(noOfValues, values));
	else if (strcmp(function, "AndLogical") == 0)
	    strcpy(result, AndLogical(noOfValues, values));
	else if (strcmp(function, "OrLogical") == 0)
	    strcpy(result, OrLogical(noOfValues, values));
	else if (strcmp(function, "SumInteger") == 0)
	    strcpy(result, SumInteger(noOfValues, values));
	else if (strcmp(function, "MultiplyInteger") == 0)
	    strcpy(result, MultiplyInteger(noOfValues, values));
	else if (strcmp(function, "SumDouble") == 0)
	    strcpy(result, SumDouble(noOfValues, values));
	else if (strcmp(function, "MultiplyDouble") == 0)
	    strcpy(result, MultiplyDouble(noOfValues, values));
	else if (strcmp(function, "Ut2IsoTime") == 0)
	    strcpy(result, Ut2IsoTime(values[0]));
	else
	    {
	    sprintf(errBuf_, "Unknown Built-In Conversion Function "
		    "referenced: %s.", function);
	    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	    return FAILURE;
	    }	
	INFO(3,"Result returned by Internal Conversion Function: %s",result);
	}   // end-else 

    return SUCCESS;
}


char            *ftuFTT::ConcatString(const int        noOfValues,
				      const slxVALUE   pars[])
{
    static char    val[512];
    
    *val = '\0';
    for (int parNo = 0; parNo < noOfValues; parNo++)
	{
	if (strcmp(pars[parNo], "$BLANK") == 0)
	    strcat(val, " ");
	else
	    strcat(val, pars[parNo]);
	}
    return val;
}


char            *ftuFTT::AndLogical(const int        noOfValues,
				    const slxVALUE   pars[])
{ 
    static slxVALUE  val;
    int              result;

    result = slxStr2Logical(pars[0]);
    for (int parNo = 1; parNo < noOfValues; parNo++)
	result &=  slxStr2Logical(pars[parNo]);
    if (result)
	*val = 'T';
    else
	*val = 'F';
    return val; 
}


char            *ftuFTT::OrLogical(const int        noOfValues,
				   const slxVALUE   pars[])
{
    static slxVALUE  val;
    int              result;

    result = slxStr2Logical(pars[0]);
    for (int parNo = 1; parNo < noOfValues; parNo++)
	result |=  slxStr2Logical(pars[parNo]);
    if (result)
	*val = 'T';
    else
	*val = 'F';
    return val; 
}


char            *ftuFTT::SumInteger(const int        noOfValues,
				    const slxVALUE   pars[])
{
    static slxVALUE  val;
    int              result;

    result = 0;
    for (int parNo = 0; parNo < noOfValues; parNo++)
	result += atoi(pars[parNo]);
    sprintf(val, "%d", result);
    return val;
}


char            *ftuFTT::MultiplyInteger(const int        noOfValues,
					 const slxVALUE   pars[])
{
    static slxVALUE  val;
    int              result;

    result = 1;
    for (int parNo = 0; parNo < noOfValues; parNo++)
	result *= atoi(pars[parNo]);
    sprintf(val, "%d", result);
    return val;
}


char            *ftuFTT::SumDouble(const int        noOfValues,
				   const slxVALUE   pars[])
{
    static slxVALUE  val;
    double           result;

    result = 0;
    for (int parNo = 0; parNo < noOfValues; parNo++)
	result += atof(pars[parNo]);
    sprintf(val, "%.12E", result);
    return val;
}


char            *ftuFTT::MultiplyDouble(const int        noOfValues,
					const slxVALUE   pars[])
{
    static slxVALUE  val;
    double           result;

    result = 1;
    for (int parNo = 0; parNo < noOfValues; parNo++)
	result *= atof(pars[parNo]);
    sprintf(val, "%.12E", result);
    return val;
}


utilSTAT ftuFTT::Secs2Time(const double          ut,
			   const vltINT32        precision,
			   slxKEYW_LINE          utcTime)
{
    slxKEYW_LINE  tmpBuf;
    vltDOUBLE     dblVal;
    unsigned long tv_sec;
    long          tv_usec;
    vltLOGICAL    negative;

    dblVal = ut;
    if (dblVal < 0)
	{
	negative = TRUE;
	dblVal *= -1;
	}
    else
	negative = FALSE;
    tv_sec = (unsigned long)dblVal;
    tv_usec = (long)(1e6 * (dblVal - tv_sec));

    slxGenUTC(tv_sec, tv_usec, precision, tmpBuf);
    if (negative)
	sprintf(utcTime, "-%s", tmpBuf);
    else
	sprintf(utcTime, "%s", tmpBuf);

    return SUCCESS;
}


char            *ftuFTT::Ut2IsoTime(const slxVALUE  ut)
{
    static slxVALUE  utcTime; 
    Secs2Time(atof(ut), 3, utcTime);
    return utcTime;
}


utilSTAT ftuFTT::CheckKeyAndValTypes(const char    *key,
				     const char    *value,
				     utilLOGICAL   *match)
{
    slxPAR_TYPE keyType, valType;

    if (fitsDicDyn_.LookUp(key) == FAILURE)
	return FAILURE;
    keyType = fitsDicDyn_.DicKeywType();

    valType = utilDetermineType(value);

    if ((keyType != valType) && 
	!((keyType == slxSTRING) && (valType == slxLOGICAL)))
	{
	if (match == NULL)
	    {
	    slxVALUE  keyTypeStr, valTypeStr;
	    strcpy(keyTypeStr, utilSlxType2Str(keyType));
	    strcpy(valTypeStr, utilSlxType2Str(valType));
	    INFO(1,"WARNING: Type of key %s (%s), and type of value |%s| "
		 "(~%s) incompatible.", key, keyTypeStr, value, valTypeStr);
	    }
	else
	    *match = FALSE;
	}
    else if (match != NULL)
	*match = TRUE;

    return SUCCESS;
}


utilSTAT         ftuFTT::ImageInfo(oslxFITS   &fits,
				   int        *imageSize,
				   int        *totalImageSize,
				   int        *bitPix,
				   int        *naxis,
				   double     *bscale,
				   double     *bzero)
{
    // Image Size = |(BITPIX / 8)| * NAXIS1 * NAXIS2 * ...  .
    int  imSize;
    if (fits.KeyInBuffer("NAXIS") && fits.KeyInBuffer("BITPIX"))
	{
	fits.GetKeyword("NAXIS");
	*naxis = fits.CK().Integer();

	fits.GetKeyword("BITPIX");
	*bitPix = fits.CK().Integer();

	if (*naxis == 0)
	    *imageSize = *totalImageSize = 0;
	else
	    {
	    imSize = 1;
	    imSize *= (int)fabs(*bitPix / 8);
	    slxKEYW_LINE  tmpKey;
	    for (int naxisNo = 1; naxisNo <= *naxis; naxisNo++)
		{
		sprintf(tmpKey, "NAXIS%d", naxisNo);
		if (fits.GetKeyword(tmpKey) == FAILURE)
		    {
		    sprintf(errBuf_, "BITPIX/NAXIS* not correctly defined for "
			    "FITS file: %s.", fits.FitsFileName());
		    ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
		    return FAILURE;
		    }
		else
		    imSize *= fits.CK().Integer();
		}
	    *imageSize = *totalImageSize = imSize;
	    // The actual image size has to be corrected to be a multiple 
	    // of 2880.
	    int modulus;
	    if ((modulus = (*totalImageSize % ftuHDU_BLOCK)) != 0)
		*totalImageSize += (ftuHDU_BLOCK - modulus);
	    }
	}
    else
	*imageSize = *bitPix = *totalImageSize = 0;

    if (fits.KeyInBuffer("BSCALE"))
	{
	fits.GetKeyword("BSCALE");
	// Should be defined as a double, but some FITS files have
	// it wrongly written. E.g. "1" instead of "1.".
	if (fits.CK().Type() == slxDOUBLE)
	    *bscale = fits.CK().Double();
	else  
	    *bscale = fits.CK().Integer();
	}
    else
	*bscale = 1.0;

    if (fits.KeyInBuffer("BZERO"))
	{
	fits.GetKeyword("BZERO");
	if (fits.CK().Type() == slxDOUBLE)
	    *bzero = fits.CK().Double();
	else
	    *bzero = fits.CK().Integer();
	}
    else
	*bzero = 0.0;

    
    return SUCCESS;
}


// The "CONV_MACRO" is necessary on Linux to convert from Little Endian to Big
// Endian.
// We have two macros for this to avoid the multiplication and summation
// when BSCALE=1 and BZERO=0.
#define ftuSTAT_GEN(BSCALE, BZERO, PTR, END_MARKER, CONV_MACRO) \
{ \
    minGen = DBL_MAX; \
    maxGen = DBL_MIN; \
    while (PTR < END_MARKER) \
        { \
        valGen = ((BSCALE * CONV_MACRO(*PTR)) + BZERO); \
        if (minGen > valGen) \
            minGen = valGen; \
        if (maxGen < valGen) \
            maxGen = valGen; \
        sumLoc += valGen; \
        sqrSum += (valGen * valGen); \
        PTR++; \
        } \
    *min = minGen; \
    *max = maxGen; \
}


#define ftuSTAT(MIN_VAL, MAX_VAL, MIN_VAR, MAX_VAR, \
                TMP_VAL, PTR, END_MARKER, CONV_MACRO) \
{ \
    MIN_VAR = MAX_VAL; \
    MAX_VAR = MIN_VAL; \
    while (PTR < END_MARKER) \
	{ \
	TMP_VAL = CONV_MACRO(*PTR); \
	if (TMP_VAL < MIN_VAR) \
	    MIN_VAR = TMP_VAL; \
	if (MAX_VAR < TMP_VAL) \
	    MAX_VAR = TMP_VAL; \
	sumLoc += TMP_VAL; \
	sqrSum += (TMP_VAL * TMP_VAL); \
	PTR++; \
	} \
    *min = MIN_VAR; \
    *max = MAX_VAR; \
}


utilSTAT  ftuFTT::GenStatistics(const int   bitPix,
				const char  *buf,
				const int   len,
				double      bscale,
				double      bzero,
				double      *min,
				double      *max,
				double      *mean,
				double      *rms,
				double      *sum)
{
    register double  valGen, minGen, maxGen;
    register double  sumLoc = 0, sqrSum = 0;
    int              pixels;


    INFO(2,"Generating min./max./mean/rms ...");
    chronometer_.Start("GenStat");
    switch (bitPix)
	{
	case -32:
	    register ftuFP32  *fp32Ptr, *fp32EndMarker;
	    fp32Ptr = (ftuFP32 *)buf;
	    fp32EndMarker = (ftuFP32 *)(buf + len);
	    pixels = (len / 4);
	    if ((bscale != 1) || (bzero != 0))
		{
		ftuSTAT_GEN(bscale, bzero, fp32Ptr, fp32EndMarker, SWAP_FP32);
		}
	    else
		{	
		register ftuFP32  fp32MinVal, fp32MaxVal, fp32Val;
		ftuSTAT(FLT_MIN, FLT_MAX, fp32MinVal, fp32MaxVal, 
			fp32Val, fp32Ptr, fp32EndMarker, SWAP_FP32);
		}
	    break;
	case 32:
	    register ftuINT32  *int32BufPtr, *int32EndMarker;
	    int32BufPtr = (ftuINT32 *)buf;
	    int32EndMarker = (ftuINT32 *)(buf + len);
	    pixels = (len / 4);
	    if ((bscale != 1) || (bzero != 0))
		{
		ftuSTAT_GEN(bscale, bzero, int32BufPtr, int32EndMarker, 
			    SWAP_INT32);
		}
	    else
		{	
		register ftuINT32  int32MinVal, int32MaxVal, int32Val;
		ftuSTAT(INT_MIN, INT_MAX, int32MinVal, int32MaxVal, int32Val,
			int32BufPtr, int32EndMarker, SWAP_INT32);
		}
	    break;
	case 16:
	    register ftuINT16  *int16BufPtr, *int16EndMarker;
	    int16BufPtr = (ftuINT16 *)buf;
	    int16EndMarker = (ftuINT16 *)(buf + len);
	    pixels = (len / 2);
	    if ((bscale != 1) || (bzero != 0))
		{
		ftuSTAT_GEN(bscale, bzero, int16BufPtr, int16EndMarker, 
			    SWAP_INT16);
		}
	    else
		{	
		register ftuINT16  int16MinVal, int16MaxVal, int16Val;
		ftuSTAT(SHRT_MIN, SHRT_MAX, int16MinVal, int16MaxVal, 
			int16Val, int16BufPtr, int16EndMarker, SWAP_INT16);
		}
	    break;
	case -64:
	    register ftuFP64  *fp64BufPtr, *fp64EndMarker;
	    fp64BufPtr = (ftuFP64 *)buf;
	    fp64EndMarker = (ftuFP64 *)(buf + len);
	    pixels = (len / 8);
	    if ((bscale != 1) || (bzero != 0))
		{
		ftuSTAT_GEN(bscale, bzero, fp64BufPtr,fp64EndMarker,SWAP_FP64);
		}
	    else
		{	
		register ftuFP64  fp64MinVal, fp64MaxVal, fp64Val;
		ftuSTAT(DBL_MIN, DBL_MAX, fp64MinVal, fp64MaxVal, fp64Val, 
			fp64BufPtr, fp64EndMarker, SWAP_FP64);
		}
	    break;
	default:
	    register ftuINT8  *int8BufPtr, *int8EndMarker;
	    int8BufPtr = (ftuINT8 *)buf;
	    int8EndMarker = (ftuINT8 *)(buf + len);
	    pixels = len;
	    if ((bscale != 1) || (bzero != 0))
		{
		ftuSTAT_GEN(bscale, bzero, int8BufPtr, int8EndMarker, NOP);
		}
	    else
		{	
		register ftuINT8  int8MinVal, int8MaxVal, int8Val;
		ftuSTAT(0, UCHAR_MAX, int8MinVal, int8MaxVal, int8Val, 
			int8BufPtr, int8EndMarker, NOP);
		}
	    break;
	}
    *mean = (sumLoc / (double)pixels);
    sqrSum /= (double)pixels;
    *rms = sqrt(sqrSum - (*mean * *mean));
    *sum = sumLoc;
    INFO(2,"Values generated are - Min: %f, Max: %f, Mean: %f, Rms: %f, "
	 "Sum: %.12E. Time: %.3fs.", *min, *max, *mean, *rms, sumLoc,
	 chronometer_.Stop("GenStat"));
	
    return SUCCESS;
}


utilSTAT ftuFTT::CheckKeyTypes(const char *srcKey,
			       const char *targKey)
{ 
    slxPAR_TYPE  srcKeyType, targKeyType;

    if (fitsDicDyn_.LookUp(srcKey) == FAILURE)
	return FAILURE;
    srcKeyType = fitsDicDyn_.DicKeywType();
    if (fitsDicDyn_.LookUp(targKey) == FAILURE)
	return FAILURE;
    targKeyType = fitsDicDyn_.DicKeywType();

    if (srcKeyType != targKeyType)
	{
	INFO(1,"WARNING: Source keyword and target keyword do not have the "
	     "same type (source key: %s, target key: %s).",
	     srcKey, targKey);
	}

    return SUCCESS;
}



/*___oOo___*/
