/******************************************************************************
 * E.S.O. - Archive System
 *
 * "@(#) $Id: ftuVALUE_SRC.C,v 1.1.2.1 2008/09/11 08:11:40 cguirao Exp $"
 *
 * Who       When        What
 * --------  ----------  ----------------------------------------------
 * jknudstr  20/01/2000  Created 
 */

/************************************************************************
 *
 *   NAME
 *   ftuVALUE_SRC - To contain information about a value source.
 * 
 *   SYNOPSIS
 *   
 *   PARENT CLASS
 * 
 *   DESCRIPTION
 *
 *   PUBLIC METHODS
 *   utilSTAT         SetSource(const char *source)
 *   ftuVALUE_SOURCE  GetSource()
 *   Set/get the source for the value.
 *
 *   utilSTAT         SetReference(const char  *reference)
 *   char             *GetReference()
 *   Set/get reference for the value.
 *
 *   utilSTAT         SetAttribute(const int  value)
 *   int              GetAttribute()  
 *   Set/get the attribute describing the value.
 *
 *   PUBLIC DATA MEMBERS
 *
 *   PROTECTED METHODS
 *
 *   PROTECTED DATA MEMBERS
 *
 *   FILES
 *
 *   ENVIRONMENT
 *
 *   RETURN VALUES
 *
 *   CAUTIONS 
 *
 *   EXAMPLES
 *
 *   SEE ALSO
 *
 *   BUGS   
 * 
 *------------------------------------------------------------------------
 */

static const char *rcsId="@(#) $Id: ftuVALUE_SRC.C,v 1.1.2.1 2008/09/11 08:11:40 cguirao Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

#include <strings.h>
#include <string.h>

#include "ftuVALUE_SRC.h"

// Callback to handle deletion of the ftuVALUE_SRC_HASH class.
void fttValueSrcDeleteEl(void *elPtr) {
    delete (ftuVALUE_SRC *)elPtr;
}


ftuVALUE_SRC::ftuVALUE_SRC()
{ 
    source_ = ftuSRC_UNKNOWN;
    *reference_ = '\0'; 
    SetAttribute(ftuUNDEFINED);
    SetHeader(ftuUNDEFINED);
}

    
ftuVALUE_SRC::~ftuVALUE_SRC()
{
}


utilSTAT            ftuVALUE_SRC::SetSource(const char *source)
{
    if (strstr(source, ".KEY") != 0)
	source_ = ftuSRC_KEY;
    else if (strstr(source, ".DB") != NULL)
	source_ = ftuSRC_DB;
    else if (strstr(source, ".FCT") != NULL)
	source_ = ftuSRC_FCT;
    else if (strstr(source, ".CONST") != NULL)
	source_ = ftuSRC_CONST;
    else if (strstr(source, "FILE.RENAME.PREFIX") != NULL)
	source_ = ftuSRC_CONST;
    else if (strstr(source, "FILE.SPLIT.PREFIX") != NULL)
	source_ = ftuSRC_CONST;
    else if (strstr(source, "FILE.PATH.BASE") != NULL)
	source_ = ftuSRC_CONST;
    else
	{
	sprintf(errBuf_, "Unknown source tag %s found in FTT file.", source);
	ErrAdd("ftu", ftuERR_GEN, utilERR_FATAL, errBuf_);
	return FAILURE;
	}

    return SUCCESS;
}


ftuVALUE_SOURCE ftuVALUE_SRC::GetSource()
{
    return source_;
}


utilSTAT        ftuVALUE_SRC::SetReference(const slxKEYW_LINE  reference)
{
    strcpy(reference_, reference);
    slxTrimString(reference_, " \n\t");
    return SUCCESS;
}


char            *ftuVALUE_SRC::GetReference()
{
    return reference_;
}


utilSTAT         ftuVALUE_SRC::SetAttribute(const int  value)
{
    attribute_ = value;
    return SUCCESS;
}


int             ftuVALUE_SRC::GetAttribute()
{
    return attribute_;
} 


utilSTAT         ftuVALUE_SRC::SetHeader(const int header)
{
    header_ = header;
    return SUCCESS;
}


int              ftuVALUE_SRC::GetHeader()
{
    return header_;
}

 
/*___oOo___*/
