#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ftuTest.py,v 1.4 2005/12/01 17:33:58 jknudstr Exp $"
#
# who       when        what
# --------  --------    ----------------------------------------------
# jknudstr  28/02/2000  Created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

import os, sys
import tstCom

# Declare common variables.
pwd              = os.environ["PWD"]
ftuTestBase      = pwd+"/"+"ftuTestFile_tmp.fits"
ftuSimpleFits    = pwd+"/"+"ftuSimpleFitsFile.fits"
ftuSmallExtFits1 = pwd+"/"+"ftuTestFile2H_3I_1H_1I_2H_2I.fits"
ftuSmallExtFits2 = pwd+"/"+"ftuTestFile2H_0I_1H_1I_2H_2I.fits"
ftuIsaacFile     = pwd+"/"+"ftuTestIsaac.fits"

# Test paramereters.
test = tstCom.TestUtil()
test.parseInputPars(sys.argv)
development = 0
if (development):
    test.setVerbose(1)
    test.setCleanUp(0)
    displayDiff = 1
    disableTest = 1
else:
    test.setVerbose(0)
    test.setCleanUp(1)
    displayDiff = 0
    disableTest = 0

test.log("Executing Tests ...")

###########################################################################
# 1_1: Basic Test:
#
# Tests Performed:
# 1.1, 1.2
# 2.2, 2.3
# 3.1, 3.2, 3.3, 3.4, 3.5, 3.6
# 4.1, 4.2
# 5.1, 5.2
###########################################################################
if not disableTest or 0:
    ftuRef1_1 = pwd+"/"+"ftuRef1_1.fits"
    cmd = "fitsTranslate -noComment -noHistory -ftt "+pwd+ \
          "/ftuTestFtt_1_1.ftt "+ftuTestBase
    test.stdTest("Test-1_1-Basic-Test",
                 cmd,
                 ftuSmallExtFits1,
                 ftuTestBase,
                 ftuTestBase,
                 ftuRef1_1)
###########################################################################


###########################################################################
# 1_2: Convert Standard FITS File.
#
# Tests Performed:
# 1.3
# 6.1
###########################################################################
if not disableTest or 0:
    ftuRef1_2 = pwd+"/"+"ftuRef1_2.fits"
    cmd = "fitsTranslate -noComment -noHistory -ftt "+pwd+\
          "/ftuTestFtt_1_2.ftt "+ftuTestBase
    baseFile = ftuIsaacFile
    targetFile = pwd+"/"+"ftuISAAC_ESO-PARANALTestFileHello_tmp.fits"
    test.stdTest("Test-1_2-Convert-ISAAC-File",
                 cmd,
                 baseFile,
                 ftuTestBase,
                 targetFile,
                 ftuRef1_2)
###########################################################################


###########################################################################
# 1_3: Split up a file using the FITS Image Extension.
#
# Tests Performed:
# 1.4
###########################################################################
if not disableTest or 0:
    ftuRef1_3_1 = pwd+"/"+"ftuRef1_3_1.fits"
    cmd = "fitsTranslate -noComment -noHistory -ftt "+pwd+\
          "/ftuTestFtt_1_3.ftt "+ftuTestBase
    baseFile = ftuSmallExtFits2
    targetFile = pwd+"/"+"ftuTestSplit_20_72_TestFile_Hello_tmp.fits"
    test.stdTest("Test-1_3_1-Split-File",
                 cmd,
                 baseFile,
                 ftuTestBase,
                 targetFile,
                 ftuRef1_3_1)
###########################################################################
if not disableTest or 0:
    ftuRef1_3_2 = pwd+"/"+"ftuRef1_3_2.fits"
    baseFile = ftuSmallExtFits2
    targetFile = pwd+"/"+"ftuTestSplit_60_48_TestFile_Hello_tmp.fits"
    test.testFiles("Test-1_3_2-Split-File",
                   baseFile,
                   ftuTestBase,
                   targetFile,
                   ftuRef1_3_2)
###########################################################################


###########################################################################
# 1_4: Handle several input FITS files.
#
# Tests Performed:
#    1.5: Test input of several files.
#    4.3: Test -noStat parameter.
#    7.2: Test -test parameter.
#    4.4: Test -noCheckSum parameter.
###########################################################################
if not disableTest or 0:
    testBaseFiles   = [ftuSimpleFits,
                       ftuSmallExtFits2,
                       ftuSmallExtFits1]
    
    testInputFiles  = [pwd+"/"+"ftuTest_1_4_InputFile1_tmp.fits",
                       pwd+"/"+"ftuTest_1_4_InputFile2_tmp.fits",
                       pwd+"/"+"ftuTest_1_4_InputFile3_tmp.fits"]
    
    testTargetFiles = [pwd+"/"+"ftuTest_1_4_InputFile1_tmp_test.fits",
                       pwd+"/"+"ftuTest_1_4_InputFile2_tmp_test.fits",
                       pwd+"/"+"ftuTest_1_4_InputFile3_tmp_test.fits"]
    
    testRefFiles    = [pwd+"/"+"ftuTest_1_4_Ref_1.fits",
                       pwd+"/"+"ftuTest_1_4_Ref_2.fits",
                       pwd+"/"+"ftuTest_1_4_Ref_3.fits"]

    cmd = "fitsTranslate -test -noComment -noHistory -noStat -noCheckSum "+ \
          "-ftt "+pwd+"/"+"ftuTestFtt_1_4.ftt "+ \
          testInputFiles[0]+" "+testInputFiles[1]+" "+testInputFiles[2]

    test.stdTestDyn("Test-1_4-Handle-Mulitple-Files",
                    cmd,
                    testBaseFiles,
                    testInputFiles,
                    testTargetFiles,
                    testRefFiles)
###########################################################################


###########################################################################
# 1_5: Test Handling of Type Conversion.
###########################################################################
if not disableTest or 0:
    testBaseFile    = pwd+"/ftuTestTypeConv_1_5.fits"    
    testInputFile   = pwd+"/ftuTest_1_5.fits"
    testTargetFile  = testInputFile
    testRefFile     = pwd+"/ftuRef_1_5.fits"
    testFtt         = pwd+"/ftuTestTypeConv_1_5.ftt"
       
    cmd = "fitsTranslate -noComment -noHistory -ftt "+testFtt+" "+\
          testInputFile
    
    test.stdTest("Test-1_5-Type-Conversion",
                 cmd,
                 testBaseFile,
                 testInputFile,
                 testTargetFile,
                 testRefFile)
###########################################################################


###########################################################################
# 1_6_1: Exaction of selected FITS Image Extension/Ext#1.
#
# Tests Performed:
# 
###########################################################################
if not disableTest or 0:
    ftuRef1_6_1 = pwd+"/"+"ftuRef1_6_1.fits"
    cmd = "fitsTranslate -noComment -noHistory -ftt "+pwd+\
          "/ftuTestFtt_1_6_1.ftt "+ftuTestBase
    baseFile = ftuSmallExtFits2
    targetFile = pwd+"/"+"ftuTestExtractHdus_ESO_2_20_tmp.fits"
    test.stdTest("Test-1_6_1-Ex-Sel-Ext",
                 cmd,
                 baseFile,
                 ftuTestBase,
                 targetFile,
                 ftuRef1_6_1,
                 displayDiff)
###########################################################################

###########################################################################
# 1_6_2: Exaction of selected FITS Image Extension/Ext#1.
#
# Tests Performed:
# 
###########################################################################
if not disableTest or 0:
    ftuRef1_6_2 = pwd+"/"+"ftuRef1_6_2.fits"
    cmd = "fitsTranslate -noComment -noHistory -ftt "+pwd+\
          "/ftuTestFtt_1_6_2.ftt "+ftuTestBase
    baseFile = ftuSmallExtFits2
    targetFile = pwd+"/"+"ftuTestExtractHdus_ESO_3_60_tmp.fits"
    test.stdTest("Test-1_6_2-Ex-Sel-Ext",
                 cmd,
                 baseFile,
                 ftuTestBase,
                 targetFile,
                 ftuRef1_6_2,
                 displayDiff)
###########################################################################


test.log("Tests Executed")
sys.exit(0)

#
# ___oOo___
