#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tstCom.py,v 1.1 2004/09/06 14:38:38 cguirao Exp $"
#
# who       when          what
# --------  ----------    ----------------------------------------------
# jknudstr  28/02/2000    Created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

import os, sys
import pcc, PccUtUtils

# Class providing utilities for carrying out the tests.
class TestUtil:
    "Class providing various services for carrying out the tests."

    verbose_ = 0
    cleanUp_ = 1

    def parseInputPars(self, argv):
        if len(argv) == 1:
            return 0
        for i in range(len(argv)):
            if argv[i] == "-noCleanUp":
                self.setCleanUp(0)
            elif argv[i] == "-v":
                self.verbose(1)
            else:
                self.usage(argv[0])
                sys.exit(1)

    def usage(self, scriptName):
        print "\nCorrect usage is:\n\n"
        print "python %s [-v] [-noCleanUp]\n\n" % scriptName

    def setVerbose(self, verbose):
        self.verbose_ = verbose

    def getVerbose(self):
        return self.verbose_

    def setCleanUp(self, state):
        self.cleanUp_ = state

    def getCleanUp(self):
        return self.cleanUp_

    def log(self, msg):
        if self.getVerbose() > 0:
            print "TEST-LOG> "+msg

    def diff(self, file1, file2):
        cmd = "diff "+file1+" "+file2
        self.log("Executing command: "+cmd)
        exitCode, stdOut, stdErr = PccUtUtils.execCmd(cmd)
        #fd = os.popen(cmd, "r")
        #res = fd.read()
        #fd.close()
        #return res
        if (stdOut != ""):
            return stdOut
        elif (exitCode == 0):
            return ""
        else:
            return stdErr

    def compareFiles(self, file1, file2, testName, displayDif = 0):
        res = self.diff(file1, file2)
        if res == "":
            print("RESULT: "+testName+": OK")
        else:
            if displayDif == 1:
                print "RESULT: "+testName+": FAILED\n"+res
            else:
                print "RESULT: "+testName+": FAILED"        
                
    def removeFile(self, file):
        cmd = "rm -f "+file
        self.log("Removing file:")
        self.log(cmd)
        os.system(cmd)

    def stdTest(self,
                testName,
                testCmd,
                testBaseFile,
                testInputFile,
                testTargetFile,
                testRefFile,
                displayDif = 0):

        # Clean up existing files.
        self.log("Removing existing input/output files ...")
        self.log("rm -f "+testInputFile+" "+testTargetFile)
        os.system("rm -f "+testInputFile+" "+testTargetFile)

        # Make copy of test base file to the test input file.
        self.log("Generating new input file: "+testInputFile)
        self.log("cp "+testBaseFile+" "+testInputFile)
        os.system("cp "+testBaseFile+" "+testInputFile)

        # Execute the test.
        self.log("Executing the test command: "+testCmd)
        os.system(testCmd)

        # Make a diff() of the Test Target File and the Reference File.
        self.compareFiles(testTargetFile, testRefFile, testName, displayDif)

        # Clean up test files.
        if self.getCleanUp() == 1:
            self.removeFile(testInputFile)
            self.removeFile(testTargetFile)
        else:
            self.log("Leaving test files on disk. Files used in test are:")
            self.log("    Test Base File:      "+testBaseFile)
            self.log("    Test Input File:     "+testInputFile)
            self.log("    Test Target File:    "+testTargetFile)
            self.log("    Test Reference File: "+testRefFile)

    # Method that handles lists of the various test files.
    def stdTestDyn(self,
                   testName,
                   testCmd,
                   testBaseFiles,
                   testInputFiles,
                   testTargetFiles,
                   testRefFiles,
                   displayDif = 0):

        # Clean up existing files.
        self.log("Removing existing input/output files ...")
        for i in range(len(testInputFiles)):
            self.removeFile(testInputFiles[i])
            self.removeFile(testTargetFiles[i])

        # Make copy of test base file to the test input file.
        for i in range(len(testBaseFiles)):
            self.log("Generating new input file: "+testInputFiles[i])
            os.system("cp "+testBaseFiles[i]+" "+testInputFiles[i])

        # Execute the test.
        self.log("Executing the test command: "+testCmd)
        os.system(testCmd)

        # Make a diff() of the Test Target File and the Reference File.
        for i in range(len(testTargetFiles)):
            name = testName+"/"+str(i + 1)
            self.compareFiles(testTargetFiles[i], testRefFiles[i],
                              name, displayDif)

        # Clean up test files.
        if self.getCleanUp() == 1:
            for i in range(len(testInputFiles)):
                self.removeFile(testInputFiles[i])
                self.removeFile(testTargetFiles[i])
        else:
            self.log("Leaving test files on disk. Files used in test are:")
            for i in range(len(testBaseFiles)):
                self.log("    Test Base File:      "+testBaseFiles[i])
                self.log("    Test Input File:     "+testInputFiles[i])
                self.log("    Test Target File:    "+testTargetFiles[i])
                self.log("    Test Reference File: "+testRefFiles[i])

    # Method to test files already generated and to clean up afterwards.
    def testFiles(self,
                  testName,
                  testBaseFile,
                  testInputFile,
                  testTargetFile,
                  testRefFile,
                  displayDif = 0):

        # Make a diff() of the Test Target File and the Reference File.
        self.compareFiles(testTargetFile, testRefFile, testName, displayDif)

        # Clean up test files.
        if self.getCleanUp() == 1:
            self.removeFile(testInputFile)
            self.removeFile(testTargetFile)
        else:
            self.log("Leaving test files on disk. Files used for in test are:")
            self.log("    Test Base File:      "+testBaseFile)
            self.log("    Test Input File:     "+testInputFile)
            self.log("    Test Target File:    "+testTargetFile)
            self.log("    Test Reference File: "+testRefFile)    

#
# ___oOo___
