#ifndef slxBasics_H
#define slxBasics_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: slxBasics.h,v 1.145 2005/07/27 08:53:27 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/10/96  created
* epozna    13/06/02  fixed compillation warning on te35  
*                     do not use ## in macros
* - - - - - - - - - - JAN2006 - - - - - - - - - - - - - - - - - - - 
* mnylund   12/05/05  VLTSW20050188 Cannot have more than 4096 
*                     configuration keywords
*/
/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdarg.h>

#ifndef NOCCS
    #include "ccs.h"
    #include "msg.h"
    #include "miscHash.h"
#else
    #include "miscNoCcs.h"
#endif

/* Constant declarations
 */
#ifndef TRUE
#define TRUE ccsTRUE
#endif
#ifndef FALSE
#define FALSE ccsFALSE
#endif

#define slxMODULE_NAME           "slx"
#define slxBIG_CHUNK             50000
#define slx10_BLANKS             "          "
#define slxRECV_TIMEOUT          10000
#define slxMAX_NO_OF_ALIASES     300
#define slxALIAS_FILE_EXT        ".alias"
#define slxINS_ROOT              "INS_ROOT"
#define slxINS_SETUPPATH_NAME    "INS_SETUPPATH"
#define slxINS_USER              "INS_USER"
#define slxINS_USER_PATH         "INS_USER_PATH"
#define slxSYST_SETUP_FILE_DIR   "/SYSTEM/COMMON/SETUPFILES"
#define slxSETUP_FILE_DIR        "/SETUPFILES"
#define slxDET_SETUP_FILE_DIR    "/DET"
#define slxINS_SETUP_FILE_DIR    "/INS"
#define slxREF_SETUP_FILE_DIR    "/REF"
#define slxTARG_SETUP_FILE_DIR   "/TARG"
#define slxINS_PATH_DEFAULT      "./"
#define slxPROT_REG_MARKER       "SUP.PROT"
#define slxSTART_PROT_REG_SHORT  "SUP.PROT;                      # Start protected region           \n"
#define slxEND_PROT_REG_SHORT    "SUP.PROT;                      # End protected region             \n"
#define slxKEYW_INFO_LEN         200    /* Before: 64 */
#define slxFILENAME_LEN          256
#define slxNO_ERROR              0
#define slxKEYW_LINE_LEN         256    /* Before: 128 */
#define slxNO_OF_CH_REC_ELS      8191   /* VLTSW20050188, Before 4096 */
#define slxMAX_NO_OF_KEY_ELS     7
#define slxMAX_SUB_SYSTS         (slxMAX_NO_OF_KEY_ELS - 2)

#define slxMAX_NO_OF_WATCHES     25

/* Constants for FITS keyword generation
 */
#define slxSTD_FITS_EQ_POS       9
#define slxHIER_FITS_EQ_POS      30
#define slxSTD_FITS_CH_START     11
#define slxFITS_ESO_PREFIX       "HIERARCH ESO "
#define slxMAX_FITS_LI_LEN       80
#define slxSHORT_FITS_MAX_LI_LEN slxMAX_FITS_LI_LEN
#define slxEXTRA_FITS_LI_LEN     20
#define slxSTD_FITS_COM_POS      41
#define slxHIER_FITS_COM_POS     45
#define slxSTD_FITS_COM_MAX_LEN  (slxMAX_FITS_LI_LEN - slxSTD_FITS_COM_POS - 1)
#define slxSTD_FITS_MAX_STR_LEN  (slxSTD_FITS_COM_POS - slxSTD_FITS_EQ_POS - 5)
#define slxHIER_FITS_COM_MAX_LEN (slxMAX_FITS_LI_LEN - slxSTD_FITS_COM_POS - 1)
#define slxHIER_FITS_MAX_STR_LEN (slxSTD_FITS_COM_POS - slxHIER_FITS_EQ_POS- 5)
#define slxSTD_FITS_MAX_STR_LEN  (slxSTD_FITS_COM_POS - slxSTD_FITS_EQ_POS - 5)
#define slxCATEGORY_LEN          3
#define slxFITS_MIN_STR_LEN      8
#define slxFITS_HD_BLOCK_LEN     2880
#define slxNULL_STR              ""
#define slxNULL_NO               0
#define slxSTD_FITS              ""
#define slxCAT_BUF_LEN           16
#define slxSUBSYST_BUF_LEN       32
#define slxPAR_BUF_LEN           32
#define slxFITS_FILE_EXT         ".fits"
#define slxCOMMENT_PREFIX        "COMMENT"
#define slxLOG_PREFIX            "ESO-LOG"

/* Constants for accessing dictionary
 */
#define slxSYST_DIC_DIR             "SYSTEM/Dictionary"
#define slxDICO_PREFIX              "ESO-VLT-DIC."
#define slxNOT_SET                  "        "
#define slxSTR_UNDEF                slxNOT_SET

#define slxDICO_HD_LAST_ENTRY       "Remarks:"
#define slxDICO_HD_DICO_NAME        "Dictionary Name:"
#define slxDICO_PAR_NAME_PREFIX     "Parameter Name:"
#define slxDICO_DESCR_PREFIX        "Description:"

#define slxSTRING_TYPE_STR          "string"
#define slxLOGICAL_TYPE_STR         "logical"
#define slxINTEGER_TYPE_STR         "integer"
#define slxDOUBLE_TYPE_STR          "double"
#define slxUNKNOWN_TYPE_STR         "<unknown>"
#define slxDICO_LI_LEN              100
#define slxDIC_BUF_LEN              128

/* Set-Up file extensions
 */
#define slxNO_EXTENSION            ""
#define slxREF_EXTENSION           ".ref"
#define slxINS_EXTENSION           ".ins"
#define slxTARG_EXTENSION          ".targ"
#define slxDET_EXTENSION           ".det"

/* File Types
 */
/* NEW Types:
 */
#define slxPAF_TYPE_REF_SETUP      "Reference Setup"
#define slxPAF_TYPE_INS_SETUP      "Instrument Setup"
#define slxPAF_TYPE_DET_SETUP      "Detector Setup"
#define slxPAF_TYPE_TARG_SETUP     "Telescope Setup"
#define slxPAF_TYPE_REF_CONFIG     "Reference Configuration"
#define slxPAF_TYPE_CONFIG         "Configuration"
#define slxPAF_TYPE_AMBIENT_DATA   "Ambient Data"
#define slxPAF_TYPE_TEMPL_PAF      "Template Signature"
#define slxPAF_OB_DESCR            "OB Description"
/* For backwards compatability: */
#define slxPAF_TYPE_TEMPL          slxPAF_OB_DESCR

/* OLD Types:
 */
#define slxREF_SETUP_TYPE          slxPAF_TYPE_REF_SETUP
#define slxINS_SETUP_TYPE          slxPAF_TYPE_INS_SETUP
#define slxDET_SETUP_TYPE          slxPAF_TYPE_DET_SETUP 
#define slxTARG_SETUP_TYPE         slxPAF_TYPE_TARG_SETUP
typedef enum {
    slxREFERENCE_SETUP = 1,
    slxINSTRUMENT_SETUP,
    slxTARGET_SETUP,
    slxDETECTOR_SETUP,
    slxUNKNOWN_SETUP_TYPE
} slxSETUP_TYPE;

/* Configuration File Extensions
 */
#define slxINS_CONFIG_FILE_EXT     ".cfg"
#define slxREF_CONFIG_FILE_EXT     ".refcfg"
#define slxCONFIG_FILE_DIR         "CONFIGFILES"
#define slxSEQ_FILE_DIR            "TEMPLATES"
#define slxOBD_DIR                 "OBD"
#define slxSEQ_DIR                 "SEQ"
#define slxTSF_DIR                 "TSF"
#define slxTSFX_DIR                "TSF"

/* OLD Configuration File Types:
 */
#define slxINS_CONFIG_FILE_TYPE    slxPAF_TYPE_CONFIG
#define slxREF_CONFIG_FILE_TYPE    slxPAF_TYPE_REF_CONFIG

/* FITS Keyword Classes (as written in the DID, all with small letters)
 */
#define slxSETUP_CLASS_STR         "setup"
#define slxHEADER_CLASS_STR        "header"
#define slxCONFIG_CLASS_STR        "config"
#define slxCONF_LOG_CLASS_STR      "conf-log"
#define slxCOND_LOG_CLASS_STR      "cond-log"
#define slxOPS_LOG_CLASS_STR       "ops-log"
#define slxREDUC_LOG_CLASS_STR     "reduc-log"
#define slxPRIVATE_CLASS_STR       "private"
#define slxTEMPLATE_CLASS_STR      "template"
#define slxMAINT_HEADER_CLASS_STR  "maint-header"
#define slxALL_CLASS_STR           "all"

typedef enum {
    slxANY_CLASS = 0,
    slxSETUP_CLASS,
    slxHEADER_CLASS,
    slxCONFIG_CLASS,
    slxCONF_LOG_CLASS,
    slxCOND_LOG_CLASS,
    slxOPS_LOG_CLASS,
    slxREDUC_LOG_CLASS, 
    slxPRIVATE_CLASS,
    slxTEMPLATE_CLASS,
    slxMAINT_HEADER_CLASS,
    slxALL_CLASS,

    /* Used to indicate the number of classes
     */
    slxNO_KEYW_CLASSES
} slxKEYWORD_CLASS;

/* Types for FITS keywords
 */
typedef enum {
    slxSTRING  = 1,
    slxLOGICAL,
    slxINTEGER,
    slxDOUBLE,
    slxUNKNOWN
} slxPAR_TYPE;

/* Types for data in FITS files
 */
typedef enum {
    slxINT8 = 1,
    slxUINT8,
    slxINT16,
    slxUINT16,
    slxINT32,
    slxUINT32,
    slxFLOAT32,
    slxFLOAT64
} slxDATA_TYPES;

/* Hierarchical FITS keywords categories
 */
#define slxGENERAL                "GEN"
#define slxTELESCOPE              "TEL"
#define slxADAPTER                "ADA"
#define slxINSTRUMENT             "INS"
#define slxDETECTOR               "DET"
#define slxARCHIVE                "ARC"
#define slxPROCESS                "PRO"
#define slxSETUP                  "SUP"

/* Types for Template Files
 */
typedef enum {
    slxTEMPLATE_TSF = 1,
    slxTEMPLATE_SEQ,
    slxTEMPLATE_OBD,
    slxEXT_TEMPLATE_TSF
} slxTEMPLATE_FILES;

/* Macro to build a keyword name from components given as
 * constant strings (such as defines)
 */
/* comp te35 gcc3.0.4
 * #define slxX_KEYW_NAME(a,b,c) a ## "." ## b "." ## c 
 */
#define slxX_KEYW_NAME(a,b,c) a "." b "." c
#define slxKEYW_NAME(a,b,c) slxX_KEYW_NAME(a,b,c)

/* Macro to glue components together
 */
/*gcc 3.0.4
 *#define _slxGLUE_2(a,b) a ## b
 */
#define _slxGLUE_2(a,b) a  b
#define slxGLUE_2(a,b) _slxGLUE_2(a,b)

/* Macro that copies over a maximum number of bytes to a destination buffer.
 * In case of buffer overflow a pointer to an Error Structure with the name 
 * "error" is set with the appropriate error message
 */
#define slxCOPY_MAX_N_BYTES(destBuf, srcBuf, maxBytes, error) { \
    if ((vltINT32)strlen(srcBuf) >= (vltINT32)maxBytes) \
       { \
       errAdd(error, slxMODULE_NAME, slxERR_BUF_OVERFLOW, __FILE__, NULL); \
       errAdd(error, slxMODULE_NAME, slxERR_VALUE_FORMAT, __FILE__, NULL, \
	      srcBuf); \
       return FAILURE; \
       } \
    strcpy(destBuf, srcBuf); \
}
#define slxCOPYN_MAX_N_BYTES(destBuf, srcBuf, noBytes, maxBytes, error) { \
    if ((vltINT32)noBytes >= (vltINT32)maxBytes) \
       { \
       errAdd(error, slxMODULE_NAME, slxERR_BUF_OVERFLOW, __FILE__, NULL); \
       errAdd(error, slxMODULE_NAME, slxERR_VALUE_FORMAT, __FILE__, NULL, \
	      srcBuf); \
       return FAILURE; \
       } \
    strncpy(destBuf, srcBuf, noBytes); \
    *(destBuf + noBytes) = '\0'; \
}
#define slxSTRCAT_MAX_N_BYTES(destBuf, srcBuf, maxBytes, error) { \
    if (((vltINT32)strlen(srcBuf) + strlen(destBuf)) >= (vltINT32)maxBytes) \
       { \
       errAdd(error, slxMODULE_NAME, slxERR_BUF_OVERFLOW, __FILE__, NULL); \
       errAdd(error, slxMODULE_NAME, slxERR_VALUE_FORMAT, __FILE__, NULL, \
	      srcBuf); \
       return FAILURE; \
       } \
    strcat(destBuf, srcBuf); \
    *(destBuf + (strlen(srcBuf) + strlen(destBuf))) = '\0'; \
}

/* Zero out a memory area
 */
#define slxZERO(buffer) { \
    memset(buffer, 0, sizeof(buffer)); \
}

/* Type definitions
 */
typedef union {
    char        *s;
    vltLOGICAL  l;
    vltINT32    i;
    vltDOUBLE   d;
} slxFITS_VALUE;

typedef enum {
    slxNORMAL_FITS    = 1,      /* Set-up file in normal hierarch. FITS */
    slxSHORT_FITS     = 2,      /* Set-up file in the Short-FITS format */
    slxUNKNOWN_FORMAT           /* Set-up file in unknown format */
} slxSETUP_FORMAT;

typedef enum {
    slxINS_CONFIG_FILE = 1,
    slxREF_CONFIG_FILE,
    slxANY_CONFIG_FILE
} slxCONFIG_TYPE;

typedef char       slxALIAS[64];

typedef char       slxINSTR_NAME[16];

typedef char       slxDICO_TYPE[80];

typedef char       slxDIC_RECORD_ENTRY[slxDIC_BUF_LEN];

typedef char       slxFITS_CAT[16];

typedef char       slxUT_DATE[32];

typedef char       slxCATEGORY[slxCAT_BUF_LEN];

typedef char       slxSUBSYSTEM[slxSUBSYST_BUF_LEN];

typedef char       slxPARAMETER[slxPAR_BUF_LEN];

typedef char       slxVALUE[slxKEYW_LINE_LEN];

typedef char       slxFITS_STRING[slxKEYW_LINE_LEN];

typedef char       slxCOMMENT[slxKEYW_LINE_LEN];

typedef char       slxKEYW_LINE[slxKEYW_LINE_LEN];

typedef char       slxFILENAME[slxFILENAME_LEN];

typedef char       slxDICTIONARY[slxFILENAME_LEN];

typedef char       slxALIAS_TABLE[slxFILENAME_LEN];

/* VLTSW20050188, 
Before: 
typedef vltINT8    slxCHANGE_RECORD[slxNO_OF_CH_REC_ELS] 

The additional element is added to avoid problems with indexing 
outside the limit in several oslxRoutines
*/
typedef vltINT8    slxCHANGE_RECORD[slxNO_OF_CH_REC_ELS+1]; 

typedef vltINT8    slxFILE_PERMIS[3];

typedef vltLOGICAL slxKEYWORD_CLASS_VEC[slxNO_KEYW_CLASSES];

typedef struct _slxKEYW_INFO {
    slxPAR_TYPE          parType;
    slxKEYWORD_CLASS_VEC parClassVec;
    vltUINT8             found;
    vltUINT16            number;
    slxKEYW_LINE         keyword;
    slxKEYW_LINE         keywordLine;
    slxCATEGORY          category;
    slxSUBSYSTEM         subSystem;
    slxSUBSYSTEM         subSystem2;
    slxSUBSYSTEM         subSystem3;
    slxSUBSYSTEM         subSystem4;
    slxSUBSYSTEM         subSystem5;
    slxPARAMETER         parameter;
    char                 value[slxKEYW_INFO_LEN];
    slxCOMMENT           comment;
    char                 stringVal[slxKEYW_INFO_LEN];
    vltLOGICAL           logicVal;
    vltINT32             intVal;
    vltDOUBLE            doubleVal;
    char                 *commentPtr;
    char                 *dataPtr;
    vltLOGICAL           included;

    /* Private stuff
     */
    struct _slxKEYW_INFO *next;
    struct _slxKEYW_INFO  *prev;
} slxKEYW_INFO;

typedef struct {
    vltINT32    expoID; 
    vltLOGICAL  noMove;
    vltLOGICAL  check;
} slxSETUP_FLAGS;

#ifdef __cplusplus
}
#endif

#endif /*!slxBasics_H*/
