#ifndef slxDictionary_H
#define slxDictionary_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: slxDictionary.h,v 1.145 2005/07/27 08:54:06 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/10/96  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
#ifdef __cplusplus
extern "C" {    
#endif

#include "slxBasics.h"
#include "slxDynBuf.h"
#include "miscHash.h"

/* Dictionary Units and pseudo units allowed in Dictionaries
 */
typedef enum {
    slxUNIT_NONE = 0,

    /* Basic (SI) Units:
     */
    slxUNIT_LENGTH_M,
    slxUNIT_LENGTH_MM,
    slxUNIT_LENGTH_MUM,
    slxUNIT_LENGTH_NM,
    slxUNIT_MASS,
    slxUNIT_TIME,
    slxUNIT_PLANE_ANGLE,
    slxUNIT_SOLID_ANGLE,
    slxUNIT_TEMP,
    slxUNIT_ELEC_CURRENT,

    /* IAU Recognized Units:
     */
    slxUNIT_FREQ,
    slxUNIT_ENERGY,
    slxUNIT_POWER,
    slxUNIT_ELEC_POT,
    slxUNIT_FORCE,
    slxUNIT_PRESSURE,

    /* Additional 'pseudo' units allowed
     */
    slxUNIT_DEGS_OF_ARC,
    slxUNIT_MINS_OF_ARC,
    slxUNIT_SECS_OF_ARC,
    slxUNIT_MAGNITUDE,
    slxUNIT_FLUX_FREQ,
    slxUNIT_FLUX_WAVE,
    slxUNIT_MJD
} slxDIC_UNITS;

/* Unit Names:
 */
#define slxUNIT_NONE_ID          "none"

/* Basic (SI) Units:
 */
#define slxUNIT_LENGTH_M_ID      "m"
#define slxUNIT_LENGTH_MM_ID     "mm"
#define slxUNIT_LENGTH_MUM_ID    "mum"
#define slxUNIT_LENGTH_NM_ID     "nm"
#define slxUNIT_MASS_ID          "kg"
#define slxUNIT_TIME_ID          "s"
#define slxUNIT_PLANE_ANGLE_ID   "rad"
#define slxUNIT_SOLID_ANGLE_ID   "sr"
#define slxUNIT_TEMP_ID          "K"
#define slxUNIT_ELEC_CURRENT_ID  "A"

/* IAU Recognized Units:
 */
#define slxUNIT_FREQ_ID          "Hz"
#define slxUNIT_ENERGY_ID        "J"
#define slxUNIT_POWER_ID         "W"
#define slxUNIT_ELEC_POT_ID      "V"
#define slxUNIT_FORCE_ID         "N"
#define slxUNIT_PRESSURE_ID      "Pa" 

/* Additional 'pseudo' units allowed
 */
#define slxUNIT_DEGS_OF_ARC_ID   "deg"  
#define slxUNIT_MINS_OF_ARC_ID   "arcmin"  
#define slxUNIT_SECS_OF_ARC_ID   "arcsec"  
#define slxUNIT_MAGNITUDE_ID     "mag"
#define slxUNIT_FLUX_FREQ_ID     "W/m^2/s"
#define slxUNIT_FLUX_WAVE_ID     "Jy"
#define slxUNIT_MJD_ID           "mjd"

/* The Unit Tables are global
 */
extern int     slxUnitTable[];
extern char    *slxUnitIdTable[];

/* Various possible conversions of FITS values when unit is defined
 * in the Dictionary.
 */
typedef enum {
    slxCONV_NONE = 0,
    slxCONV_SECS_TO_TIME,
    slxCONV_SECS_TO_DAYTIM,
    slxCONV_MJD_TO_DAYTIM,
    slxCONV_DEG_TO_DEGREE,
    slxCONV_DEG_TO_HOURANG,
    slxCONV_RAD_TO_DEGREE,
    slxCONV_RAD_TO_HOURANG,
    slxCONV_ARCM_TO_DEGREE,
    slxCONV_ARCM_TO_HOURANG,
    slxCONV_KELVIN_TO_CELSIUS	
} slxUNIT_CONVERSIONS;
#define slxCONV_INACTIVE_ID           "CONV_INACTIVE"
#define slxCONV_SECS_TO_TIME_ID       "CONV_SECS_TO_TIME"
#define slxCONV_SECS_TO_DAYTIM_ID     "CONV_SECS_TO_DAYTIM"
#define slxCONV_MJD_TO_DAYTIM_ID      "CONV_MJD_TO_DAYTIM"
#define slxCONV_DEG_TO_DEGREE_ID      "CONV_DEG_TO_DEGREE"
#define slxCONV_DEG_TO_HOURANG_ID     "CONV_DEG_TO_HOURANG"
#define slxCONV_RAD_TO_DEGREE_ID      "CONV_RAD_TO_DEGREE"
#define slxCONV_RAD_TO_HOURANG_ID     "CONV_RAD_TO_HOURANG"
#define slxCONV_ARCM_TO_DEGREE_ID     "CONV_ARCM_TO_DEGREE"
#define slxCONV_ARCM_TO_HOURANG_ID    "CONV_ARCM_TO_HOURANG"
#define slxCONV_KELVIN_TO_CELSIUS_ID  "CONV_KELVIN_TO_CELSIUS"

/* Names of tags for the conversion of values and writing into
 * the Comment Field of a FITS Keyword generated
 */
#define slxCONV_NONE_ID     "NONE"
#define slxCONV_TIME_ID     "TIME"
#define slxCONV_DAYTIM_ID   "DAYTIM"
#define slxCONV_DEGREE_ID   "DEGREE"
#define slxCONV_HOURANG_ID  "HOURANG"
#define slxCONV_CELSIUS_ID  "CELSIUS"

/* Structure to contain one Dictionary Record
 */
typedef struct _slxDIC_HANDLE {
    vltUINT8              keywFound;
    /* The name as in the Dictionary 
     */
    slxDIC_RECORD_ENTRY   parName;
    /* The keyword searched for if the Dictionary only contained the
     * generic keyword
     */
    slxDIC_RECORD_ENTRY   searchKeyword;
    /* Class as in the Dictionary */
    slxDIC_RECORD_ENTRY   parClass;   
    /* Classes resolved */
    slxKEYWORD_CLASS_VEC  parClassVec;
    slxDIC_RECORD_ENTRY   parContext;
    slxDIC_RECORD_ENTRY   dicParType;
    slxPAR_TYPE           dicParamType;
    slxDIC_RECORD_ENTRY   valFormat;
    slxDIC_RECORD_ENTRY   parUnit;
    slxDIC_UNITS          parUnitId;
    slxDIC_RECORD_ENTRY   comField;
    slxUNIT_CONVERSIONS   conversion;
    /* The "comFieldFormat" is used when a conversion is requested in
     * connection with a keyword. It points to a string ready to be
     * used by sprintf()
     */
    char                  *comFieldFormat;

    char                  *description;

    /* Internal use
     */ 
    vltINT32              no;
    struct _slxDIC_HANDLE *next;
    struct _slxDIC_HANDLE *prev;
} slxDIC_HANDLE;

ccsCOMPL_STAT slxLookUpKeyword(const slxDICTIONARY  dicName,
                               const char           *keywordName,
                               const vltUINT16      recordNumber,
                               slxDIC_HANDLE        *dicInfo,
                               ccsERROR             *error);

ccsCOMPL_STAT slxInstallDicBuf(slxDYN_BUF          *dicBuf,
			       miscHASH_TABLE      dicHashTable,
			       slxDIC_HANDLE       *dicList,
			       ccsERROR            *error);

void slxSortDicList(slxDIC_HANDLE *dicList);

ccsCOMPL_STAT slxLoadDicHashList(miscHASH_TABLE        dicHashTable,
				 const char            *dictionary,
				 slxDIC_HANDLE         *dicList,
				 ccsERROR              *error);

ccsCOMPL_STAT slxLoadDicHash(miscHASH_TABLE       dicHashTable,
                             const char           *dictionary,
                             ccsERROR             *error);

ccsCOMPL_STAT slxLookUpKeywordHashX(const miscHASH_TABLE  dicHashTable,
				    const slxKEYW_LINE    keyword,
				    slxDIC_HANDLE         **dicInfo,
				    ccsERROR              *error);

ccsCOMPL_STAT slxLookUpKeywordHash(const slxDICTIONARY  dictionary,
                                   const slxKEYW_LINE   keyword,
                                   slxDIC_HANDLE        **dicInfo,
                                   ccsERROR             *error);

ccsCOMPL_STAT slxResetDicRecord(slxDIC_HANDLE  *dicInfo,
				ccsERROR       *error);


ccsCOMPL_STAT slxLookUpKeywordBuf(slxDYN_BUF        *dicBuf,
                                  const char        *keywordName,
                                  const vltUINT16   recordNumber,
                                  slxDIC_HANDLE     *dicInfo,
                                  ccsERROR          *error);


/*VLTSW20020630*/
ccsCOMPL_STAT slxGetDicRecSeq(const vltLOGICAL  init,
			      vltLOGICAL        *gotRecord,
			      slxDYN_BUF        *dicBuf,
			      slxDIC_HANDLE     *dicInfo,
			      ccsERROR          *error);
ccsCOMPL_STAT slxParseUnitConv(slxDIC_HANDLE      *dicInfo,
			       ccsERROR           *error);
ccsCOMPL_STAT slxSetUnitId(slxDIC_HANDLE      *dicInfo,
			   ccsERROR           *error);



ccsCOMPL_STAT slxFreeDictionary(miscHASH_TABLE  dicHashTable,
				ccsERROR        *error);

ccsCOMPL_STAT slxConvMethNo2Id(const slxUNIT_CONVERSIONS number,
			       slxKEYW_LINE              id,
			       ccsERROR                  *error);

ccsCOMPL_STAT slxGetDicRevision(const slxDICTIONARY    dictionary,
				const vltINT32         entryNo,
				slxDIC_RECORD_ENTRY    revision,
				ccsERROR               *error);

ccsCOMPL_STAT slxGetDicVersion(const slxDICTIONARY     dictionary,
			       slxDIC_RECORD_ENTRY     version,
			       ccsERROR                *error);

#ifdef __cplusplus
}
#endif

#endif /*!slxDictionary_H*/
