#ifndef slxFitsKeys_H
#define slxFitsKeys_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: slxFitsKeys.h,v 1.145 2005/07/27 08:56:48 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  19/03/96  created
*/

/************************************************************************
 * This header files contains macros for all officially recognised 
 * FITS keywords.
 *
 * All keywords are given without the possible indeces. To assemble the
 * keywords to a complete Hierarchical Short-FITS keyword, the function
 * slxAssembleKeyword2() can be used.
 *
 * In the individual context there may be defined special keywords. These 
 * must then be defined by the developer himself.
 *
 * NOTE: This file is currently under development, and the contents may
 *       be changed in future releases of SLX.
 *----------------------------------------------------------------------
 */

/* Standard FITS keywords
 */
#define slxSTD_AIRMASS    "AIRMASS"
#define slxSTD_BITPIX     "BITPIX"
#define slxSTD_BSCALE     "BSCALE"
#define slxSTD_BZERO      "BZERO"
#define slxSTD_COMMENT    "COMMENT"
#define slxSTD_DATAMAX    "DATAMAX"
#define slxSTD_DATAMIN    "DATAMIN"
#define slxSTD_DATE       "DATE"
#define slxSTD_DATE_OBS   "DATE-OBS"
#define slxSTD_DEC        "DEC"
#define slxSTD_END        "END"
#define slxSTD_EXPTIME    "EXPTIME"
#define slxSTD_EXTEND     "EXTEND"
#define slxSTD_INSTRUME   "INSTRUME"
#define slxSTD_LST        "LST"
#define slxSTD_NAXIS      "NAXIS"
#define slxSTD_OBJECT     "OBJECT"
#define slxSTD_OBSERVER   "OBSERVER"
#define slxSTD_ORIGIN     "ORIGIN"
#define slxSTD_RA         "RA"
#define slxSTD_SIMPLE     "SIMPLE"
#define slxSTD_TELESCOP   "TELESCOP"
#define slxSTD_UT         "UT"

/* World Coordinate System (WCS) Keywords
 */
#define slxWCS_CDELT      "CDELT"
#define slxWCS_CRVAL      "CRVAL"
#define slxWCS_CRPIX      "CRPIX"
#define slxWCS_CTYPE      "CTYPE"
#define slxWCS_EQUINOX    "EQUINOX"
#define slxWCS_MJD_OBS    "MJD-OBS"
#define slxWCS_PC00100    "PC00100"
#define slxWCS_PC00200    "PC00200"

/* Category FITS Keywords
 */
#define slxCAT_ADA        "ADA"
#define slxCAT_ARC        "ARC"
#define slxCAT_DET        "DET"
#define slxCAT_GEN        "GEN"
#define slxCAT_INS        "INS"
#define slxCAT_PRO        "PRO"
#define slxCAT_SIM        "SIM"
#define slxCAT_SUP        "SUP"
#define slxCAT_TEL        "TEL"

/* Subsystem Keywords
 */
#define slxSUB_ADAO       "ADAO"
#define slxSUB_ADC        "ADC"
#define slxSUB_ACTO       "ACTO"
#define slxSUB_AIRM       "AIRM"
#define slxSUB_AMBI       "AMBI"
#define slxSUB_CAT        "CAT"
#define slxSUB_CHIP       "CHIP"
#define slxSUB_COMP       "COMP"
#define slxSUB_CHCK       "CHCK"
#define slxSUB_CRTE       "CRTE"
#define slxSUB_DOME       "DOME"
#define slxSUB_EXPO       "EXPO"
#define slxSUB_FILT       "FILT"
#define slxSUB_FOCU       "FOCU"
#define slxSUB_GRAT       "GRAT"
#define slxSUB_GRIS       "GRIS"
#define slxSUB_GUID       "GUID"
#define slxSUB_INST       "INST"
#define slxSUB_LAMP       "LAMP"
#define slxSUB_LCHG       "LCHG"
#define slxSUB_MIRR       "MIRR"
#define slxSUB_MOS        "MOS"
#define slxSUB_OBS        "OBS"
#define slxSUB_OPTI       "OPTI"
#define slxSUB_OUT        "OUT"
#define slxSUB_POS        "POS"
#define slxSUB_PRIS       "PRIS"
#define slxSUB_PROG       "PROG"
#define slxSUB_SHUT       "SHUT"
#define slxSUB_SLIT       "SLIT"
#define slxSUB_SOFW       "SOFW"
#define slxSUB_TARG       "TARG"
#define slxSUB_TRAK       "TRAK"
#define slxSUB_VER        "VER"
#define slxSUB_WIN        "WIN"
#define slxSUB_WIND       "WIND"

/* Parameter Keywords
 */
#define slxPARA_ALT        "ALT"
#define slxPARA_ANG        "ANG"
#define slxPARA_AZ         "AZ"
#define slxPARA_CHECKSUM   "CHECKSUM"
#define slxPARA_DATE       "DATE"
#define slxPARA_DAYTIM     "DAYTIM"
#define slxPARA_DEC        "DEC"
#define slxPARA_DIST       "DIST"
#define slxPARA_DIMX       "DIMX"
#define slxPARA_DIMY       "DIMY"
#define slxPARA_FWHM       "FWHM"
#define slxPARA_ID         "ID"
#define slxPARA_LEN        "LEN"
#define slxPARA_MAX        "MAX"
#define slxPARA_MIN        "MIN"
#define slxPARA_MODE       "MODE"
#define slxPARA_NAME       "NAME"
#define slxPARA_NO         "NO"
#define slxPARA_NX         "NX"
#define slxPARA_NY         "NY"
#define slxPARA_PARM       "PARM"
#define slxPARA_PRES       "PRES"
#define slxPARA_RA         "RA"
#define slxPARA_RATEA      "RATEA"
#define slxPARA_RATED      "RATED"
#define slxPARA_RHUM       "RHUM"
#define slxPARA_ROT        "ROT"
#define slxPARA_SCALE      "SCALE"
#define slxPARA_SCALX      "SCALX"
#define slxPARA_SCALY      "SCALY"
#define slxPARA_SPEED      "SPEED"
#define slxPARA_ST         "ST"
#define slxPARA_STATUS     "STATUS"
#define slxPARA_TEMP       "TEMP"
#define slxPARA_TILTX      "TILTX"
#define slxPARA_TILTY      "TILTY"
#define slxPARA_TIME       "TIME"
#define slxPARA_TYPE       "TYPE"
#define slxPARA_UT         "TYPE"
#define slxPARA_VALUE      "VALUE"
#define slxPARA_WID        "WID"
#define slxPARA_WLEN       "WLEN"
#define slxPARA_X          "X"
#define slxPARA_Y          "Y"
#define slxPARA_ZENIT      "ZENIT"

#endif /*!slxFitsKeys_H*/
