/*******************************************************************************
* E.S.O. - VLT project
*
* slxCatFits.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  30/04/96  created
* - - - - - - - - APR2005 - - - - - - - - - - - - - - - - - - - - - - - 
* mnylund   19/07/04  VLTSW20040178 ccsInit/ccsExit is removed
* - - - - - - - - JAN2006 - - - - - - - - - - - - - - - - - - - - - - - 
* mnylund   21/07/05  VLTSW20050173 Support for slx under LCU
*
*/

/************************************************************************
*   NAME
*   slxCatFits - print out the contents of a FITS file header to stdout 
*
*   SYNOPSIS
*   slxCatFits <fits file> [<fits file>] [-printName]
* 
*   DESCRIPTION
*   The program reads in the header of a FITS file, reformats each line by 
*   adding newlines after each FITS line, and prints these to stdout.
*
*   It is possible to specify several files at the same time. If "-printName"
*   is given in, each block of FITS lines will be separated and the filenames
*   written.
*
*   FILES
*
*   ENVIRONMENT
*
*   COMMANDS
*
*   RETURN VALUES
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
* 
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxCatFits.c,v 1.145 2005/07/27 09:04:36 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);


/* System Headers 
 */
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
/* VLTSW20050173 */
#ifdef MAKE_LCU
    #include "vxWorks.h"
#endif
#include <sys/stat.h>

/* Local Headers 
 */
#include "slx.h"

#define slxCAT_FITS_SEPATOR \
"============================================================================"


/* Main
 */
int main (int argc, char *argv[])
{
    char          *linePtr;
    ccsERROR      error;
/*  VLTSW20040178
    ccsPROCNAME   procName;
*/
    slxDYN_BUF    hdBuf;
    slxFILENAME   completeName;
    slxKEYW_LINE  line;
    vltINT32      i, index, no;
    vltLOGICAL    printName = FALSE, asciiFile;

    if (argc < 2) 
	{
	printf("\nCorrect syntax is: \n");
	printf("\n%% slxCatFits <fits file> [<fits file>] [-printName]\n");
	printf("\n\n");
	exit(0);
	}

    no = argc;
    if (argc > 2)
	{
	if (strcmp(argv[(argc - 1)], "-printName") == 0)
	    {
	    printName = TRUE;
	    no = (argc - 1);
	    }
	}	
/*  VLTSW20040178 Remove ccsInit and only reset stack  
    strcpy(procName, "slxCatFits");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
	goto errorCond;
*/
    errResetStack(&error);

    if (slxInitDynBuf(&hdBuf, &error) == FAILURE)
	goto errorCond;

    for (i = 1; i < no; i++)
	{
	strcpy(completeName, argv[i]);
	if (slxLoadFitsHeaderBuf(completeName, &hdBuf, &asciiFile, 
				 &error) == FAILURE)
	    goto errorCond;
	if (printName)
	    {
	    printf("\n%s", slxCAT_FITS_SEPATOR);
	    printf("\nFITS File: %s", completeName);
	    printf("\n%s\n", slxCAT_FITS_SEPATOR);
	    }       
	if (!asciiFile)
	    {
	    index = 0;
	    memset(line, 0, sizeof(line));
	    while (index < hdBuf.bytesStored)
		{
		strncpy(line, (hdBuf.dynBuf + index), slxMAX_FITS_LI_LEN);
		*(line + (slxMAX_FITS_LI_LEN - 1)) = '\n';
		index += slxMAX_FITS_LI_LEN;
		if (*line != ' ')
		    printf("%s", line);
		}
	    }
	else
	    {
	    if (slxGetNextLineBuf2(&hdBuf, TRUE, &linePtr, &error) == FAILURE)
		goto errorCond;
	    do
		{
		if (slxGetNextLineBuf2(&hdBuf, FALSE, &linePtr, 
				       &error) == FAILURE)
		    goto errorCond;
		slxTrimString(linePtr, " \n");
		if (!slxSkipLine(linePtr))
		    printf("%s\n", linePtr);
		}
	    while (*linePtr != '\0');
	    }
	}

    slxResetDynBuf(&hdBuf, &error);
    return(0);

  errorCond:
/*  VLTSW20040178
    slxPrintError(&error);
*/
    errPrintStderr(&error);
    slxResetDynBuf(&hdBuf, &error);
/*  VLTSW20040178 Replace ccsExit with errResetStack
#ifndef NOCCS
    ccsExit(&error);
#endif
*/
    errResetStack(&error);
    return(1);
}


/*___oOo___*/
