/*****************************************************************************
* E.S.O. - VLT project
*
* slxCreateWriteFits2.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  09/04/96  created
* - - - - - -MAR2002 - -- - - -
* epozna    27/05/02  fixed bug in va_list/va_end :slxCreateWriteFits2
*/

/************************************************************************
*   NAME
*   slxCreateWriteFits2 - creates FITS line and writes this to an image file
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxCreateWriteFits2(const int             fd,
*                                     const miscHASH_TABLE  dictionary,
*                                     const slxFITS_VALUE   value,
*                                     ccsERROR              *error, ..., NULL)
*
* 
*   DESCRIPTION
*   The function creates a FITS line by calling the function 
*   "slxCreateFits2()", and appends this in the file specified by the file 
*   descriptor. The function assumes that the file pointer points to the 
*   correct place in the file, and simply writes the new FITS line at this 
*   position.
*
*   For further information about the generation of the FITS keywords, see the
*   man page for "slxCreateFits2()".
*
* 
*   fd    IN  File descriptor for the image data file which is opened and ready
*             to be written in. File pointer must point to the correct location
*             in the file.
*
*   For an explanation of the rest of the parameters, consult the man page for
*   the function "slxCreateFits2()".
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successful and the FITS line was 
*             written successfully in the file.
*   FAILURE   If something went wrong. The following error codes can be 
*             returned:
*
*             slxERR_WRONG_FD       The error descriptor is wrong
*
*             slxERR_WRITE_FILE_FD  An error occurred while writing in a file
*                                   referenced by the file descriptor.
*
*             The function calls slxCreateFITS() and may return errors 
*             generated by this function.
*
*   CAUTIONS 
*   See man page for "slxCreateFits2()".
*
*   EXAMPLES
*
*   SEE ALSO
*   slxCreateFits2()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxCreateWriteFits2.c,v 1.145 2005/07/27 09:09:50 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <unistd.h>
#include <sys/stat.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxCreateWriteFits2(const int             fd,
				  const miscHASH_TABLE  dictionary,
				  const slxFITS_VALUE   value,
				  ccsERROR              *error, ...)
{ 
    struct stat     buf;
    va_list         argList;
    ccsCOMPL_STAT   stat;
    slxKEYW_LINE    fitsLine;


    /* Check the file descriptor
     */
    if (fstat(fd, &buf) != 0)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_WRONG_FD, __FILE__, NULL,
                  modUNIX, fd);
        return FAILURE;
        }

    /* Create the FITS line
     */
    va_start(argList, error);
    stat = slxCreateFits2_v(dictionary, value, TRUE, fitsLine, error, argList);
    va_end(argList);
    if (stat == FAILURE)
	return FAILURE;

    /* Write it in the file
     */
    if (write(fd, fitsLine, (int)slxMAX_FITS_LI_LEN) < slxMAX_FITS_LI_LEN)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE_FD, __FILE__, NULL,
                  modUNIX, fd);
        return FAILURE;
        }
    return SUCCESS;
}

/*
 *****************************************************************************
 */

/*___oOo___*/


