/******************************************************************************
* E.S.O. - VLT project
*
* slxDetectFormatLine.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  26/03/95  created
*/

/************************************************************************
*   NAME
*   slxDetectFormatLine - detects the format of a set-up line
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxDetectFormat(const slxKEYW_LINE  keywordLine,
*				  slxSETUP_FORMAT     *type,
*                                 ccsERROR            *error)
*
* 
*   DESCRIPTION
*   The function checks the format of a set-up to determine whether the format
*   is the normal FITS or the Short-FITS. The type is written to the variable
*   pointed to by "type".
* 
*
*   keywordLine IN      The keyword line.
*
*   type	IN	Points to variable (provided from calling application)
*    			in which the result of the check is stored. Can be 
*			either "slxNORMAL_FITS", "slxSHORT_FITS" or 
*			"slxUNKNOWN_FORMAT".
*			
*   error       OUT     CCS error structure in which a possible error 
*                       occurring is returned.
*
*
*   RETURN VALUES 
*   SUCCESS	In case the operation was successful.
*
*   FAILURE	In case the operation went wrong. 
*
*
*   CAUTION
*   Since the analysis of the keyword is done only on basis on this one line
*   the detection might be wrong if the keyword line contains ambigous
*   information.
*
*   SEE ALSO
*   slxShortToNormalFITS(), slxNormalToShortFITS(), slxConvertToNormal(),
*   slxConvertToShort()   
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxDetectFormatLine.c,v 1.145 2005/07/27 09:11:05 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>

/* Local Headers
 */
#include "slx.h" 

/* Function definition
 */
ccsCOMPL_STAT slxDetectFormatLine(const slxKEYW_LINE  keywordLine,
			          slxSETUP_FORMAT     *type,
                                  ccsERROR            *error)

{

    /* The criteria to detect the Normal hierarchical format is
     * to find the string "HIERARCH ESO" as the first part of the
     * line. 
     * 
     * For Short-FITS the critetia is to find a dot (".") on the
     * fourth position and a ";" in the line.
     */
    
    /* Normal FITS?
     */          
    if ((strncmp(keywordLine, slxFITS_ESO_PREFIX, 
		strlen(slxFITS_ESO_PREFIX)) == 0) ||
	strncmp(keywordLine, "COMMENT", strlen("COMMENT")) == 0)
	{
	*type = slxNORMAL_FITS;
	} 
    else 
	{
	*type = slxSHORT_FITS;
	}

    return SUCCESS;
}

/*
 *****************************************************************************
 */


/*___oOo___*/






