/******************************************************************************
* E.S.O. - VLT project
*
* slxDynBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/10/96  created
* epozna    26/03/01  VLTSW20000108 ('\0' at the end of dynBuf->dynBuf 
*                                    in function slxStoreBufDynBuf  )
*/

/************************************************************************
*   NAME
*   slxDynBuf, slxInitDynBuf, slxDynBufAlloc, slxResetDynBuf,
*   slxSoftResetDynBuf, slxStoreBufDynBuf, slxDynBufReadToCrt,
*   slxRewindDynBuf, slxSizeDynBuf, slxAllocDynBuf,
*   slxGetCharDynBuf, slxSetCharDynBuf, slxTruncateDynBuf - tools for 
*   handling SLX Dynamic Buffers
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxInitDynBuf(slxDYN_BUF  *dynBuf,
*			        ccsERROR   *error)
*
*   ccsCOMPL_STAT slxDynBufAlloc(slxDYN_BUF       *dynBuf,
*                                const vltUINT32  size,
*                                ccsERROR         *error)
*
*   ccsCOMPL_STAT slxResetDynBuf(slxDYN_BUF  *dynBuf,
*                                ccsERROR    *error)
*
*   ccsCOMPL_STAT slxSoftResetDynBuf(slxDYN_BUF  *dynBuf,
*                                    ccsERROR    *error)
*
*   ccsCOMPL_STAT slxStoreBufDynBuf(slxDYN_BUF        *dynBuf,
*                                   const char        *buf,
*                                   const vltUINT32   bufLen,
*                                   ccsERROR          *error) 
*
*   ccsCOMPL_STAT slxDynBufReadToCrt(slxDYN_BUF    *dynBuf,
*	  		             char          **buffer,
*                                    vltUINT32     *bufLen,
*		 	             ccsERROR      *error)
*
*   ccsCOMPL_STAT slxRewindDynBuf(slxDYN_BUF     *dynBuf,
*                                 const vltINT32 bytes,
*                                 ccsERROR       *error)
*
*   char *slxDynBuf(const slxDYN_BUF     *dynBuf)
*
*   vltUINT32 slxSizeDynBuf(const slxDYN_BUF     *dynBuf)
*
*   vltUINT32 slxAllocDynBuf(const slxDYN_BUF     *dynBuf)
*
*   char slxGetCharDynBuf(const slxDYN_BUF  *dynBuf,
*                         const vltINT32    index)
*
*   ccsCOMPL_STAT slxSetCharDynBuf(slxDYN_BUF     *dynBuf,
*                                  const vltINT32 index,
*                                  char           chr,
*                                  ccsERROR       *error)
*
*   ccsCOMPL_STAT slxTruncateDynBuf(slxDYN_BUF     *dynBuf,
*                                   const vltINT32 bytes,
*                                   ccsERROR       *error)
*
*   ccsCOMPL_STAT slxDynBufAddEOS(slxDYN_BUF     *dynBuf,
*                                 ccsERROR       *error)
*
*   ccsCOMPL_STAT slxGetXBytesBuf(slxDYN_BUF     *dynBuf,
*			          slxDYN_BUF_OP  initialize,
*			          vltINT32       reqLen,
*			          char           *buffer,
*			          vltINT32       *bufLen,
*			          ccsERROR       *error)
* 
*   DESCRIPTION
*   slxInitDynBuf()
*   The function is used to initialize the SLX Dynamic Buffer. Note this 
*   function _must_ be called before starting out using the Dynamic Buffer, 
*   but should only be called once in the lifetime of this buffer.
*   If it is necessary to reset the buffer use slxResetDynBuf().
*
*   dynBuf   IN   Pointer to SLX Dynamic Buffer structure to handle e.g. a
*                 setup in memory.
*  
*   slxDynBufAlloc()
*   The function is used to allocate a number of bytes for a SLX Dynamic
*   Buffer. If the buffer already is allocated, the size is adjusted to
*   the specified one. If the buffer is enlarged the contents will
*   remain untouched after the reallocation.
*
*   If the size already allocated is the same as requested nothing is done.
*
*   New memory allocated will contain NULL's.
*
*   dynBuf   IN   Pointer to SLX Dynamic Buffer structure to handle e.g. a
*                 setup in memory.
*
*   size     IN   Size to which the buffer should be adjusted.
*
*   slxResetDynBuf()
*   The function is used to reset/initialize the SLX Dynamic Buffer. Possibly
*   allocated memory is freed - so be sure that it is desirable to delete
*   the data contained in the buffer.
*
*   dynBuf   IN   Pointer to SLX Dynamic Buffer structure to handle e.g.
*                 setup in memory.
*
*   slxSoftResetDynBuf()
*   The function is used to reset/initialize the SLX Dynamic Buffer. Possibly
*   allocated memory remains untouched.
*
*   dynBuf   IN   Pointer to SLX Dynamic Buffer structure to handle e.g.
*                 setup in memory.
*
*   slxStoreBufDynBuf()
*   The function appends the contents of the buffer pointed to by "buf" in the
*   end of the SLX Dynamic Buffer, "dynBuf". If the Dynamic Buffer cannot
*   contain the new data, the buffer size is adjusted aytomatically.
*
*   dynBuf   IN   Pointer to SLX Dynamic Buffer structure to handle e.g. a
*                 setup in memory.
*
*   buf      IN   Buffer to be stored in the SLX Dynamic Buffer.
*
*   bufLen   IN   Length of the buffer to store
*
*   slxDynBufReadToCrt()
*   The function reads in bytes from the SLX Dynamic Buffer untill it meets
*   a CRT ('\n') or the end of the buffer is reached.
*
*   The buffer read is stored in local memory.
*
*   dynBuf       IN     SLX Dynamic Buffer to handle a file in memory. It is
*                       assumed that the Dynamic Buffer is initialized 
*                       properly (see slxGetNextLineBuf()).
*
*   buffer       OUT    Pointer to the beginning of the next line.
*
*   bufLen       OUT    Length of the buffer read.
*
*   slxRewindDynBuf()
*   Rewind the Dynamic Buffer a certain number of bytes from the current
*   internal reference.
*
*   slxDynBuf()
*   Return pointer to the actual (character) buffer of the slxDYN_BUF 
*   structure.
*
*   slxSizeDynBuf()
*   Return the size (bytes stored) in the Dynamic Buffer.
*
*   slxAllocDynBuf()
*   Return number of bytes allocated in the Dynamic Buffer.
*
*   slxGetCharDynBuf()
*   Return a specific character given by the "index". First character has
*   number 1.   
*
*   slxSetCharDynBuf()
*   Set a character in the Dynamic Buffer at the "index" position. First 
*   character in the Dynamic Buffer has number 1.
*
*   slxTruncateDynBuf()
*   Truncate the Dynamic Buffer starting from the last byte. the number of
*   bytes to truncate is indicated by "bytes".
*
*   slxGetXBytesBuf()
*   Get the next X bytes from an SLX Dynamic Buffer.
*
*   RETURN VALUES 
*
*   CAUTIONS 
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxDynBuf.c,v 1.145 2005/07/27 09:11:41 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);


/* System Headers
 */
#include <string.h>
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

ccsCOMPL_STAT slxInitDynBuf(slxDYN_BUF  *dynBuf,
			    ccsERROR   *error)
{
    memset((char *)dynBuf, 0, sizeof(slxDYN_BUF));

    return SUCCESS;
}

ccsCOMPL_STAT slxDynBufAlloc(slxDYN_BUF       *dynBuf,
                             const vltUINT32  size,
                             ccsERROR         *error)
{
    char  *newBuf;

    if (dynBuf->bytesAlloc == size)
	return SUCCESS;
    else if (!dynBuf->bytesAlloc)
	{
	if ((dynBuf->dynBuf = calloc(size, sizeof(vltUINT8))) == NULL)
	    {
	    errSysAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL,
		      modUNIX);
	    return FAILURE;
	    }
	/*VLTSW20000108 (just to make sure) */
	memset(dynBuf->dynBuf, '\0',size); 
	}
    else 
	{
	if ((newBuf = realloc(dynBuf->dynBuf, size)) == NULL)
	    {
	    errSysAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL,
		      modUNIX);
	    return FAILURE;
	    }
	dynBuf->dynBuf = newBuf;

	if (!dynBuf->bytesStored)
	    memset(dynBuf->dynBuf, 0, dynBuf->bytesAlloc);
	}
    dynBuf->bytesAlloc = size;

    return SUCCESS;
}

ccsCOMPL_STAT slxResetDynBuf(slxDYN_BUF   *dynBuf,
			     ccsERROR     *error)
{

    if (dynBuf->bytesAlloc == 0)
	{
	dynBuf->dynBuf = (char *)NULL;
	dynBuf->bytesStored = 0;	
	dynBuf->count = 0;
	}
    else
	{
	free(dynBuf->dynBuf);
        memset((char *)dynBuf, 0, sizeof(slxDYN_BUF));
	}

    return SUCCESS;
}

ccsCOMPL_STAT slxSoftResetDynBuf(slxDYN_BUF   *dynBuf,
			         ccsERROR     *error)
{
    dynBuf->bytesStored = 0;	
    dynBuf->count = 0;

    return SUCCESS;
}


ccsCOMPL_STAT slxStoreBufDynBuf(slxDYN_BUF        *dynBuf,
                                const char        *buf,
                                const vltUINT32   bufLen,
                                ccsERROR          *error) 
{
    char  *tmpBuf;

    /* Allocate more memory if necessary
     */

    /*  VLTSW20000108 make sure it ends with '\0'  */
    if ((bufLen + dynBuf->bytesStored) >= (dynBuf->bytesAlloc) )           /* SPR:  >=  */
      {
	 
	if ((tmpBuf = realloc(dynBuf->dynBuf, 
			      (bufLen + dynBuf->bytesStored +1))) == NULL) /* SPR: +1*/
	  {
	    errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL);
	    return FAILURE;
	  }	
	
	tmpBuf[bufLen + dynBuf->bytesStored ]='\0';                  /* SPR: just  make sure */
	dynBuf->dynBuf = tmpBuf;
	dynBuf->bytesAlloc = (bufLen + dynBuf->bytesStored+1);             /*SPR:  +1 */
	
      }

    /* Store the new bytes in the Dynamic Buffer
     */
    memcpy((dynBuf->dynBuf + dynBuf->bytesStored), buf, bufLen);
    dynBuf->bytesStored += bufLen;    
    
    return SUCCESS;
}

ccsCOMPL_STAT slxDynBufReadToCrt(slxDYN_BUF    *dynBuf,
 	                         char          **buffer,
                                 vltUINT32     *bufLen,
	 	                 ccsERROR      *error)
{
    char        *chrPtr;
    vltINT32    len;

    chrPtr = *buffer = (dynBuf->dynBuf + dynBuf->count + 1);  
    len = 0;
    while ((*chrPtr != '\n') && (*chrPtr != '\0') && 
	   ((chrPtr - dynBuf->dynBuf) <= dynBuf->bytesStored))
	{
	len++;
	chrPtr++;
	}

    *bufLen = len;
    dynBuf->count = (dynBuf->count + len);
    if (dynBuf->count > dynBuf->bytesStored)
	dynBuf->count = dynBuf->bytesStored;

    return SUCCESS;
}

ccsCOMPL_STAT slxRewindDynBuf(slxDYN_BUF      *dynBuf,
			      const vltINT32  bytes,
			      ccsERROR        *error)
{
    if (bytes > dynBuf->count)
	dynBuf->count = 0;
    else
	dynBuf->count -= bytes;

    return SUCCESS;
}

char *slxDynBuf(const slxDYN_BUF     *dynBuf)
{
    return dynBuf->dynBuf;
}

vltUINT32 slxSizeDynBuf(const slxDYN_BUF     *dynBuf)
{
    return dynBuf->bytesStored;
}

vltUINT32 slxAllocDynBuf(const slxDYN_BUF     *dynBuf)
{
    return dynBuf->bytesAlloc;
}

char slxGetCharDynBuf(const slxDYN_BUF  *dynBuf,
		      const vltINT32    index)
{
    if ((index - 1) < 0)
	return '\0';
    else if (index > dynBuf->bytesStored)
	return '\0';
    else
	return *(dynBuf->dynBuf + (index - 1));
}

ccsCOMPL_STAT slxSetCharDynBuf(slxDYN_BUF     *dynBuf,
			       const vltINT32 index,
			       char           chr,
			       ccsERROR       *error)
{
    if ((index - 1) < 0)
	{
	errAdd(error, slxMODULE_NAME, slxERR_DYN_BUF_ACTION, __FILE__, NULL, 
	       "Set Character in SLX Dynamic Buffer", "Index too small");
	return FAILURE;
	}
    else if (index > dynBuf->bytesStored)
	{
	errAdd(error, slxMODULE_NAME, slxERR_DYN_BUF_ACTION, __FILE__, NULL, 
	       "Set Character in SLX Dynamic Buffer", "Index too large");
	return FAILURE;
	}
    else
	*(dynBuf->dynBuf + (index - 1)) = chr;

    return SUCCESS;
}

ccsCOMPL_STAT slxTruncateDynBuf(slxDYN_BUF     *dynBuf,
				const vltINT32 bytes,
				ccsERROR       *error)
{
    vltINT32 tmpBytes = bytes;


    if (tmpBytes < 0)
	tmpBytes = 0;

    if (tmpBytes > slxSizeDynBuf(dynBuf))
	{
	errAdd(error, slxMODULE_NAME, slxERR_DYN_BUF_ACTION, __FILE__, NULL, 
	       "Truncate SLX Dynamic Buffer",
	       "No. of bytes to truncate too large");
	return FAILURE;
	}
    if (tmpBytes < 0)
	{
	errAdd(error, slxMODULE_NAME, slxERR_DYN_BUF_ACTION, __FILE__, NULL, 
	       "Truncate SLX Dynamic Buffer",
	       "No. of bytes to truncate less than zero");
	return FAILURE;
	}
    if (tmpBytes == 0)
	return SUCCESS;

    *(slxDynBuf(dynBuf) + (slxSizeDynBuf(dynBuf) - tmpBytes)) = '\0';
    
    /*if(slxSetCharDynBuf(dynBuf, (slxSizeDynBuf(dynBuf) - tmpBytes + 1), '\0',
			 error) == FAILURE)
	return FAILURE;*/

    dynBuf->bytesStored -= tmpBytes;

    return SUCCESS;
}

ccsCOMPL_STAT slxDynBufAddEOS(slxDYN_BUF     *dynBuf,
			      ccsERROR       *error)
{
  if (slxStoreBufDynBuf(dynBuf, " ", 1, error) == FAILURE)
    return FAILURE;
  *(slxDynBuf(dynBuf) + (slxSizeDynBuf(dynBuf) - 1)) = '\0';

  return SUCCESS;
}

/* Reads X bytes from the SLX Dynamic Buffer and stores them in
 * the buffer pointed to by "buffer". When there are no more bytes in the
 * Dynamic Buffer, "buffer" will have the length 0.
 * If less bytes are available than requested, only the amount of bytes
 * available are returned in the buffer.
 */
ccsCOMPL_STAT slxGetXBytesBuf(slxDYN_BUF     *dynBuf,
			      slxDYN_BUF_OP  initialize,
			      vltINT32       reqLen,
			      char           *buffer,
			      vltINT32       *bufLen,
			      ccsERROR       *error)
{    
    vltINT32   len;

    if (initialize == slxINIT_DYN_BUF)
	{
	dynBuf->count = 0;
	return SUCCESS;
	}

    if ((vltINT32)(dynBuf->bytesStored - dynBuf->count) < reqLen)
	{
	len = (dynBuf->bytesStored - dynBuf->count);
	if (len == 0)
	    {
	    *buffer = '\0';
	    *bufLen = 0;
	    }
	else
	    {
	    strncpy(buffer, (dynBuf->dynBuf + dynBuf->count), len);
	    *bufLen = len;
	    dynBuf->count += len;
	    }
	}
    else
	{
	strncpy(buffer, (dynBuf->dynBuf + dynBuf->count), reqLen);
	*bufLen = reqLen;
	dynBuf->count += reqLen;
	}        
    
    return SUCCESS;
}

/*
 *****************************************************************************
 */


/*___oOo___*/

