/******************************************************************************
* E.S.O. - VLT project
*
* slxUtils.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  13/06/95  Created
* epozna    11/10/00  VLTSW2000078 : resolve path starting with ./ ; manpage update 
* epozna    22/01/01  VLTSW20000334: added function slxYankExtensionGenLast, 
*                                    manpage update
* epozna    16/01/02  VLTSW20010753: use getcwd instead of PWD
* epozna    31/01/02  update slxResolvePath to handle ../ and ./ inside the path
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   07/11/03  VLTSW20030038
* mnylund   02/02/04  Problems with APR2004, static variable initialized 
*                     is initialized to ccsFALSE, VLTSW20040034
*                     Update of implementation of copy routine in slxTrimString.
*                     Check that no attempt is performed to copy a string into 
*                     itself, VLTSW20040034
* - - - - - - - - - - APR2005 - - - - - - - - - - - - - - - - - - - 
* mnylund   23/07/04  Compilation warning removed for the SUN platform.
*                     warning: implicit declaration of function `gettimeofday'
* - - - - - - - - - - JAN2006 - - - - - - - - - - - - - - - - - - - - - - - 
* mnylund   22/07/05  VLTSW20050173 Support for slx under LCU
*/

/************************************************************************
*   NAME
*   slxUtils, slxGetPath, slxGetFileName, slxGetExtension, 
*   slxGenDayTim, slxGenDayTim2, slxGenDayTimPrec, slxGenDayTimCcsTime, 
*   slxGenDayTimBasic, slxGetUTDate, slxGetUTTime, slxGenUTC,
*   slxSkipLine, slxStrToUpper, slxStripQuotes, slxStrDup,
*   slxBuildInsFileName, slxBuildInsUserDir, slxAssembleKeyword,
*   slxAssKey, slxIsShortFITS, slxGetLastChar, slxSetLastChar,
*   slxKeywClass2No, slxGetValueEnvVar, slxBitPix2NoBytes, slxGetKeywordIndex,
*   slxReplaceIndex, slxResolvePath, slxStr2Logical, 
*   slxCheckExtension, slxCheckExtension2,
*   slxYankExtension, slxYankExtensionGen, slxYankExtensionGenLast, slxYankLastPath,
*   slxYankTrailNewLines, slxSplitFileName, slxDetermineType, slxTrimString,
*   slxSplit - various small (usefull) utilities
* 
*
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxGetPath(const slxFILENAME  fileName, 
*                            slxFILENAME        path,
*                            ccsERROR           *error)
*
*   ccsCOMPL_STAT slxGetFileName(const slxFILENAME  completeName, 
*                                slxFILENAME        fileName,
*                                ccsERROR           *error)
*
*   char *slxGetExtension(const slxFILENAME fileName)
*        Returns the extention of the given file name, i.e. the characters
*        after the last dot in the filename. 
*        Note :NULL is returned when there is no dot in the filename or the last dot 
*              is the first character of the filename,e.g.:
*              "/home/ins1b/jknudstr/.fhtHome"
*              "/home/ins1b/jknudstr/.fhtHome/DICTIONARY/ALL"
*
*   void slxYankExtension(slxFILENAME fileName, const slxFILENAME extension)
*        Cats down the given extention from the filename.
*        It is not recommended to include path in the 'fileName'.
*        Note that when filename does not end with the given extention the function 
*        has no effect. 
*        The extention can be given with or without the dots, e.g.:  "fits", or ".fits".
*        Warning: The method looks for the FIRST occurance of the 'extention' in the 'filename'!
*                 Therefore if the given filename is :'file.fitsname.fits' and the extention
*                 is 'fits' the resulting filename is :'file'.
*
*   void slxYankExtensionGen(slxFILENAME fileName)
*       Removes the extention from the 'fileName'.
*       It is not recommended to include path in the 'fileName'.
*       Warning:It is not recommended to use this function.
*               The method searches for the FIRST dot in the filename, and cats 
*               down the remaining characters including the first dot.
*               
*   void slxYankExtensionGenLast(slxFILENAME fileName)   
*       It is a more robust version of slxYankExtensionGen catting down the extention of 
*       the given 'fileName' if it has any extention.     
*
*   void slxGenDayTim(slxKEYW_LINE  dayTime)
*   void slxGenDayTim2(const time_t  timeSecs, 
*                      slxKEYW_LINE  dayTime)
*   ccsCOMPL_STAT slxGenDayTimPrec(const vltINT32  precision,
*                                  slxKEYW_LINE    dayTime,
*                                  ccsERROR        *error)
*   void  slxGenDayTimCcsTime(const ccsTIMEVAL  *time,
*                             const vltINT32    precision,
*                             slxKEYW_LINE      dayTime)
*   void slxGenDayTimBasic(const unsigned long tv_sec,
*                          const long          tv_usec,
*                          const vltINT32      precision,
*                          slxKEYW_LINE        dayTime)
*
*   void slxGetUTDate(slxUT_DATE utDate)
*
*   void slxGetUTTime(vltUINT32 *utTime)
*
*   void slxGenUTC(const unsigned long tv_sec,
*                  const long          tv_usec,
*                  const vltINT32      precision,
*                  slxKEYW_LINE        utcTime)
*
*   int slxSkipLine(const char *line)
*
*   void slxStrToUpper(char *str)
*
*   void slxStripQuotes(char *str)
* 
*   char *slxStrDup(const char *string)
*
*   ccsCOMPL_STAT slxBuildInsFileName(const slxFILENAME  insPath1,
*				      const slxFILENAME  insPath2,
*				      const slxFILENAME  insPath3,
*				      const slxFILENAME  fileName,
*				      slxFILENAME        completeName,
*				      ccsERROR           *error)
*
*   ccsCOMPL_STAT slxBuildInsUserDir(slxFILENAME  insUserDir,
*		   	 	     char         *user,
*				     ccsERROR     *error)
*
*   ccsCOMPL_STAT slxBuildImageFileName(const slxFILENAME  fileName,
*				        slxFILENAME        completeName,
*				        ccsERROR           *error)
*
*   ccsCOMPL_STAT slxAssembleKeyword(const slxCATEGORY   category,
*				     const slxSUBSYSTEM  subSystem,
*				     const slxPARAMETER  parameter,
*				     slxKEYW_LINE        keyword,
*				     ccsERROR            *error)
*
*   char *slxAssKey(const char *key1, const int index1, ...)
*
*   vltLOGICAL slxIsShortFITS(const slxKEYW_LINE  keyword)
*
*   char slxGetLastChar(char *string)
*
*   void slxSetLastChar(char  *string,
*                       char  chr)
*
*   ccsCOMPL_STAT slxKeywClass2No(const char        *classStr, 
*			          slxKEYWORD_CLASS  *classNo,
*			          ccsERROR          *error)
*
*   ccsCOMPL_STAT slxGetValueEnvVar(const char      *envVar,
*			     	    char            **destBuf,
*				    ccsERROR        *error)
*
*   ccsCOMPL_STAT slxBitPix2NoBytes(const vltINT32  bitPix,
*		 	 	    vltINT32        *noOfBytes,
*				    ccsERROR        *error)
*
*   ccsCOMPL_STAT slxGetKeywordIndex(const slxKEYW_LINE  keyword, 
*	 	 		     vltINT32            *index,
*				     ccsERROR            *error)
*
*   void slxReplaceIndex(slxKEYW_LINE  keyword, vltINT32 *index)
*
*   ccsCOMPL_STAT slxResolvePath(const slxFILENAME  orgPath,
*			         char               **completePath,
*			         ccsERROR           *error)
*
*   vltLOGICAL slxStr2Logical(const char  *strLogical)
*
*   void slxYankTrailNewLines(char *str)
*
*   void slxYankLastPath(slxFILENAME  path)
*
*   ccsCOMPL_STAT slxSplitFileName(const slxFILENAME  completeName,
*                                  slxFILENAME        path,
*                                  slxFILENAME        fileName,
*                                  slxFILENAME        extension,
*                                  ccsERROR           *error)
*
*   void slxDetermineType(const char   *value,
*		          slxPAR_TYPE  *type)
*
*   void slxTrimString(char *string, char *trimChars)
*
*   ccsCOMPL_STAT slxSplit(const char       *separators, 
*		           const char       *stringPtr, 
*		           const vltINT32   maxNoOfSubStr, 
*		           slxKEYW_LINE     subStrings[], 
*		           vltINT32         *noOfSubStr, 
*		           ccsERROR         *error) 
*
*   DESCRIPTION 
*   slxGetPath()
*   Returns the path for a filename. Fails in case the path is not correctly
*   formatted.
*  
*   If a filename is not entered with the absolute path (first character 
*   different from '/'), the path will be an empty buffer ("\0").
*
* 
*   fileName   IN   Name of file including the absolute path.
*
*   path       OUT  The path for the filename.
*
*   slxGenDayTim()
*   slxGenDayTim2()
*   slxGenDayTimPrec()     (only available when CCS is available)
*   slxGenDayTimCcsTime()  (only available when CCS is available)
*   slxGenDayTimBasic()
*   Generate the string for the DAYTIM keyword as (example):
*
*   DAYTIM  "1996-10-15T16:21:23"
*
*   slxGetUTDate()
*   The function generates the present UT date as a string in the format:
*
*   'YYYY-MM-DDThh:mm:ss.sss'
*
*   This format is used e.g. in the FITS header of image data files, and in 
*   the Set-Up Files.
*   
*
*   fitsDate   OUT   The generated date as a string. The buffer must be 
*                    allocated by the calling application.
*
*   slxGetUTTime()
*   The function generates the UT time (seconds since midnight) and returns
*   this figure in the "utTime" parameter. This time reference is e.g. used 
*   for the FITS header of image data, and for the observation set-up files.
*
*   utTime    OUT  The UT time - seconds since midnight.
*
*   slxGenUTC()
*   Convert seconds and microseconds given in to the format:
*
*   "HH:MM:SS.SSS"
*
*   slxSkipLine()
*   The function is used when reading in VLT files, which typical has 
*   a comment initiated with a "#".
*
*   The function returns 1 if the first character of the line is one of the
*   following characters: "#", " ", "\n", "\0" or "\t" , otherwise 0 is 
*   returned.
*
*   line  IN  Pointer to string that shall be checked.
*
*   slxStringToUpper()
*   The function converts a string containing lower-case letters or a mixture
*   of lower-case and upper-case to consist of only uppercase letters. If a
*   non ASCII character is contained in the string, this is ignored (remains
*   unchanged). The string must be NULL terminated.
*
*   str   IN   Pointer to the string that shall be upper-cased.
*
*
*   slxStripQuotes()
*   Strings in setup files are enclosed in quotes. The function strips these
*   off using the same character buffer for storing the processed string.
*   The string must be NULL terminated.
*
*   If the string is not contained in quotes, the function simply returns
*   without changing the string.
*
*   str   IN   Pointer to the string that shall be stripped.
*
*   ccsCOMPL_STAT slxAssembleKeyword()
*   Assemble keyword and store it n a buffer provided by the calling process.
*   This function is obsolete; use rather slxAssKey().
*
*   char *slxAssKey(const char *key1, const int index1, ..., NULL)
*   char *slxAssKey2(const char *key1, ..., NULL)
*   Assembles a number of sub-keywords to a Short-FITS style one. An arbitrary 
*   number of keywords can be given in. 
*   
*   For slxAssKey() a keyword must always be followed by an index. If this is 
*   zero no index is appended to the keyword. For slxAssKey2() it is not
*   necessary to give in an index.
*
*   slxKeywClass2No()
*   Converts a Keyword Class descriptor entered as a string to a number 
*   type slxKEYWORD_CLASS).
*
*   ccsCOMPL_STAT slxGetValueEnvVar()
*   Gets the value of the specified environment variable.
*
*   ccsCOMPL_STAT slxGetKeywordIndex()
*   Gets the possible index of the keyword given in. E.g. if the keyword is
*   "FILT5", index will be 5.
*
*   void slxReplaceIndex()
*   Function that replace a possible index in a keyword with "i" and return
*   the index if defined
*
*   ccsCOMPL_STAT slxResolvePath()
*   Function to resolve a path of entries that may be environment variables
*   and indirect reference to HOME. E.g., the function can resolve:
*   "~/$MY_VAR/MY_DIR/file", "~<user>/MY_DIR" or "$HOME/$MY_VAR/MY_DIR/file".
*   A colon separated list of variables will be resolved as well.
*   Note that function can resolve pathes that starts with or includes 
*   patterns: ./ or ../
*   Warning :no space is allowed in the input path name! 
*
*   ccsCOMPL_STAT slxSplitFileName()
*   The function splits up a filename in its three basic parts:
*   path (if given), filename and extension (if given). Note that for the
*   the latter the extension is considered as the string after the last 
*   period found in the filename without the path.
*
*   The filename can be given as e.g. "/diskb/user/vlt/myDir/myFile.ext",
*   "$HOME/vlt/myDir/myFile.ext", "~/vlt/myDir/myFile.ext", "myFile.ext" or
*   "myFile". If the path or extension are not available, these parameters
*   will be string of length 0.
*
*   void slxDetermineType(const char   *value,
*		          slxPAR_TYPE  *type)
*   The function tries to determine the type of a string given in.
*   The "type" parameter will be set to either slxINTEGER, slxDOUBLE or
*   slxSTRING in case of a string or a logical value.
*
*   void slxTrimString(char *string, char *trimChars)
*   Trim a string for leading and trailing charecters according to the 
*   characters given in the "trimChars" string. The Trim Chars could be e.g. 
*   "{} ". In this case the string would be trimmed for all leading "{"'s,
*   "}"'s and spaces.
*
*   ccsCOMPL_STAT slxSplit(const char       *separators, 
*		           const char       *stringPtr, 
*		           const vltINT32   maxNoOfSubStr, 
*		           slxKEYW_LINE     subStrings[], 
*		           vltINT32         *noOfSubStr, 
*		           ccsERROR         *error) 
*   Splits a string into its basic components using the characters given in
*   "separators" as criteria for the splitting. The separators could e.g. be
*   " ,", whereby the string is split according to spaces and commas.
*
*   The user must provide the array of slxKEYW_LINE buffers in which the
*   sub-strings will be written.
*
*   void slxYankLastPath(slxFILENAME  path)
*   Remove the last path or filename in the path given.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxUtils.c,v 1.145 2005/07/27 09:24:46 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <ctype.h>
#include <string.h>
#ifdef MAKE_LCU
    #include "vxWorks.h"
    #include "types/vxTypes.h"
#endif
#include <sys/stat.h>
#include <time.h>
#include <unistd.h> /* VLTSW20010753 getcwd*/

/* VLTSW20050173 */
/* #ifdef LINUX  */
#if defined(LINUX) && !defined(MAKE_LCU)
    #include <sys/time.h>
#endif

/* VLTSW20030038 */
#include <math.h>

/* Local Headers
 */
#include "slx.h"
#ifndef NOCCS
#include "tims.h"
#endif
#define __EXTENSIONS__
#include <stdlib.h>


/* Function definitions
 */
ccsCOMPL_STAT slxGetPath(const slxFILENAME fileName, 
			 slxFILENAME       path, 
			 ccsERROR          *error)
{
    char      *chrPtr;
    vltUINT16 nameLen;

    nameLen = strlen(fileName);
    if ((chrPtr = strrchr(fileName, '/')) == NULL)
        {
	/* Apparently no path
	 */
        *path = '\0';
        return SUCCESS;
        }
 
    strncpy(path, fileName, (size_t)(chrPtr - fileName + 1));
    *(path + ((chrPtr - fileName) + 1)) = '\0';
   
    return SUCCESS;
}

ccsCOMPL_STAT slxGetFileName(const slxFILENAME  completeName, 
                             slxFILENAME        fileName,
                             ccsERROR           *error)
{
    char  *chrPtr, *namePtr;

    namePtr = (char *)completeName;

    while ((chrPtr = strstr(namePtr, "/")) != NULL)
	namePtr = (chrPtr + 1);
    strcpy(fileName, namePtr);
    
    return SUCCESS;
}

void slxGenDayTim(slxKEYW_LINE  dayTime)
{
    time_t         timeSecs;
    
    timeSecs = time(NULL);
    slxGenDayTim2(timeSecs, dayTime);
}

void slxGenDayTim2(const time_t  timeSecs, slxKEYW_LINE  dayTime)
{
    char           day[16], month[16], date[16], timeStr[16], year[16];
    char           date2[16];
    struct tm      *timeNow;
    slxKEYW_LINE   tmpBuf;
    vltINT32       intVal;
    
    timeNow = gmtime(&timeSecs);
    strcpy(tmpBuf, asctime(timeNow));
    
    /* Now, reformat:
     *
     * "Sun Sep 16 01:03:52 1973\n"
     *
     * into:
     *
     * "1973-09-16T01:03:52"
     */ 
    sscanf(tmpBuf, "%s %s %s %s %s", day, month, date, timeStr, year);

    /* Convert Month as string to number
     */
    if (strstr(month, "Jan") != NULL)
	sprintf(month, "01");
    else if (strstr(month, "Feb") != NULL)
	sprintf(month, "02");
    else if (strstr(month, "Mar") != NULL)
	sprintf(month, "03");
    else if (strstr(month, "Apr") != NULL)
	sprintf(month, "04");
    else if (strstr(month, "May") != NULL)
	sprintf(month, "05");
    else if (strstr(month, "Jun") != NULL)
	sprintf(month, "06");
    else if (strstr(month, "Jul") != NULL)
	sprintf(month, "07");
    else if (strstr(month, "Aug") != NULL)
	sprintf(month, "08");
    else if (strstr(month, "Sep") != NULL)
	sprintf(month, "09");
    else if (strstr(month, "Oct") != NULL)
	sprintf(month, "10");
    else if (strstr(month, "Nov") != NULL)
	sprintf(month, "11");
    else 
	sprintf(month, "12");
    
    /* If date is less than 10 put a 0 in front
     */
    intVal = atoi(date);
    if (intVal < 10)
	sprintf(date2, "0%d", intVal);
    else
	sprintf(date2, "%d", intVal);
   
    /* Remove possible newline character in the year
     */
    if (slxGetLastChar(year) == '\n')
	slxSetLastChar(year, '\0');

    /* Print the DAYTIM: "1973-09-16T01:03:52"
     */
    sprintf(dayTime, "%s-%s-%sT%s", year, month, date2, timeStr);
}

#if defined(HP_COMP)
    #define slx_gettimeofday(tp) gettimeofday((tp),NULL)
#endif


/* Compilation warning removed by MNY 2004-07-23
#if defined(SUN_COMP)
    #if !defined(_SYS_TIME_H) || !defined(crTIMEVAL_TIMEZONE_DEFINED)
        struct timeval {
	    long  tv_sec;		
	    long  tv_usec;	
        };
        int gettimeofday(struct timeval *tp);
    #else 
        #if defined(CCS_LIGHT) 
            int gettimeofday(struct timeval *tp);
        #endif
    #endif
    #define slx_gettimeofday(tp) gettimeofday(tp)
#endif
*/

/* VLTSW20050173 */
#if defined(SUN_COMP) && !defined(MAKE_LCU)
    #if !defined(_SYS_TIME_H) || !defined(crTIMEVAL_TIMEZONE_DEFINED)
        struct timeval {
	    long  tv_sec;		
	    long  tv_usec;	
        };
        int gettimeofday(struct timeval *tp, void *);
    #else 
        #if defined(CCS_LIGHT) 
            int gettimeofday(struct timeval *tp, void *);
        #else 
           int gettimeofday(struct timeval *tp, void *);
         #endif
    #endif
    #define slx_gettimeofday(tp) gettimeofday(tp,NULL)
#endif

/* VLTSW20050173 */
#if defined(LINUX) && !defined(MAKE_LCU)
    #define slx_gettimeofday(tp) gettimeofday(tp,NULL)   
#endif

/* VLTSW20050173 */
#if defined(MAKE_LCU)
    #define slx_gettimeofday(tp) timsGetUTC(tp,NULL,error)   
#endif

ccsCOMPL_STAT slxGenDayTimPrec(const vltINT32  precision,
			       slxKEYW_LINE    dayTime,
			       ccsERROR        *error)
{
/* VLTSW20050173 */
#if defined(MAKE_LCU) 
    ccsTIMEVAL time;
#else
    struct timeval  time;
#endif
    
    slx_gettimeofday(&time);
    slxGenDayTimBasic(time.tv_sec, time.tv_usec, precision, dayTime);
    
    return SUCCESS;
}


#ifndef NOCCS
void slxGenDayTimCcsTime(const ccsTIMEVAL  *time,
			 const vltINT32    precision,
			 slxKEYW_LINE      dayTime)
{
    slxGenDayTimBasic(time->tv_sec, time->tv_usec, precision, dayTime);
}
#endif 

void slxGenDayTimBasic(const unsigned long tv_sec,
		       const long          tv_usec,
		       const vltINT32      precision,
		       slxKEYW_LINE        dayTime)
{
    slxKEYW_LINE  format, tmpBuf;

    /* SPR VLTSW20030038 Rounding error */
    /* if roundin up the leading 1 must be added */
    if (tv_usec / 1.e6 >= 1.-0.5/pow(10,precision))
	slxGenDayTim2(tv_sec+1, dayTime);
    else
	slxGenDayTim2(tv_sec, dayTime);
    if (precision > 0)
	{
	sprintf(format, "%%.%df", precision);
	sprintf(tmpBuf, format, (tv_usec / 1e6));
	strcpy(tmpBuf, (tmpBuf + 1));
	strcat(dayTime, tmpBuf);
	}
    
}

void slxGetUTDate(slxUT_DATE utDate)
{ 
/* VLTSW20050173 */
#if defined(MAKE_LCU)
    ccsTIMEVAL timeNow; 
    ccsERROR* error;
    errResetStack(error);
#else
    struct timeval timeNow;
#endif

    slx_gettimeofday(&timeNow);
    slxGenDayTimBasic(timeNow.tv_sec, timeNow.tv_usec, 3, utDate);
}

#ifndef NOCCS
ccsCOMPL_STAT slxGetUtcDif(const ccsTIMEVAL   *utcFirst,
                           const ccsTIMEVAL   *utcLast,
                           vltUINT32          *timeDif,
                           ccsERROR           *error)
{
    vltUINT32   timeDifLoc;


    if (utcFirst->tv_sec < utcLast->tv_sec)
	{
	timeDifLoc = (utcLast->tv_sec - utcFirst->tv_sec); 
	timeDifLoc *= 1e6;
	timeDifLoc += (utcLast->tv_usec - utcFirst->tv_usec);
	}
    else
	{
	timeDifLoc = (utcLast->tv_usec - utcFirst->tv_usec);
	}

    *timeDif = timeDifLoc;

    return SUCCESS;
}
#endif


void slxGetUTTime(vltUINT32 *utTime)
{
   time_t         timeSecs;
   struct tm      *timeNow;
    
   timeSecs = time(NULL);
   timeNow = gmtime(&timeSecs);
   *utTime = timeNow->tm_sec;
   *utTime += (timeNow->tm_min * 60);
   *utTime += (timeNow->tm_hour * 3600);
}

void slxGenUTC(const unsigned long tv_sec,
	       const long          tv_usec,
	       const vltINT32      precision,
	       slxKEYW_LINE        utcTime)
{
    vltINT32      secsLeft, hours, mins;
    slxKEYW_LINE  tmpBuf, format;

    hours    = (vltINT32)(tv_sec / 3600);
    secsLeft = (tv_sec - (hours * 3600));
    mins     = (secsLeft / 60);
    secsLeft = (secsLeft - (mins * 60));

    if (hours < 10)
	sprintf(tmpBuf, "0%d", hours);
    else
	sprintf(tmpBuf, "%d", hours);
    strcpy(utcTime, tmpBuf);

    if (mins < 10)
	sprintf(tmpBuf, ":0%d", mins);
    else
	sprintf(tmpBuf, ":%d", mins);
    strcat(utcTime, tmpBuf);

    if (secsLeft < 10)
	sprintf(tmpBuf, ":0%d", secsLeft);
    else
	sprintf(tmpBuf, ":%d", secsLeft);
    strcat(utcTime, tmpBuf);

    if (precision > 0)
	{
	sprintf(format, "%%.%df", precision);
	sprintf(tmpBuf, format, (tv_usec / 1e6));
	strcat(utcTime, (tmpBuf + 1));
	}    
}


int slxSkipLine(const char  *line)
{
    if ((*line == '#') || (*line == ' ') || (*line == '\n') ||
        (*line == '\0') || (*line == '\t'))
        return 1;
    else
        return 0;
}


void slxStrToUpper(char *str)
{
    vltUINT16 i;
    
    for (i = 0; i < strlen(str); i++)
        *(str + i) = (char)toupper((int)*(str + i));
}


void slxStripQuotes(char *str)
{
    char  *chrPtr;
    char  *strPtr;
    
    /* Worst-case string which becomes:
     *
     * |   "   kjkdjd kjkjk   kjkj  "  ;   |
     * |kjkdjd kjkjk   kjkj|
     */
    strPtr = str;
    if ((chrPtr = strchr(str, '\"')) != NULL) /* " */
	{
	/* Skip possible blanks after the quote
	 */
	chrPtr++;
	while (*chrPtr == ' ')
	    chrPtr++;
	/* Copy until quote or NULL terminator
	 */
	while ((*chrPtr != '\0') && (*chrPtr != '\"')) /* " */
	    {
	    *strPtr = *chrPtr;
	    strPtr++;
	    chrPtr++;
	    }
	/* If the string only contains blanks or is of length 0, strPtr still
	 * points to the beginning of the string
	 */
	if (strPtr != str)
	    {
	    strPtr--;
	    /* Yank possible trailing blanks
	     */
	    while (*strPtr == ' ')
		strPtr--;
	    strPtr++;
	    }
	*strPtr = '\0';
	}
}

/* The strdup() call does not exist for VxWorks
 */   
char *slxStrDup(const char *string)
{
    char *strPtr;

    if ((strPtr = (char *)malloc((size_t)(strlen(string) + 1))) == NULL)
        return NULL;

    strcpy(strPtr, string);

    return strPtr;
}

ccsCOMPL_STAT slxBuildInsFileName(const slxFILENAME  insPath1,
				  const slxFILENAME  insPath2,
				  const slxFILENAME  insPath3,
				  const slxFILENAME  fileName,
				  slxFILENAME        completeName,
				  ccsERROR           *error)
{
    char                  *chrPtr;    
    slxFILENAME           tmpName;


    if (slxGetValueEnvVar(slxINS_ROOT, &chrPtr, error) == FAILURE)
	return FAILURE;

    strcpy(tmpName, chrPtr);
    if ((*(tmpName + (strlen(tmpName) - 1)) != '/') &&
	(*insPath1 != '/'))
	strcat(tmpName, "/");

    if (*insPath1 != '\0')
	strcat(tmpName, insPath1);
    if ((*(tmpName + (strlen(tmpName) - 1)) != '/') &&
	(*insPath2 != '/'))
	strcat(tmpName, "/");

    if (*insPath2 != '\0')
	strcat(tmpName, insPath2);
    if ((*(tmpName + (strlen(tmpName) - 1)) != '/') &&
	(*insPath3 != '/'))
	strcat(tmpName, "/");

    if (*insPath3 != '\0')
	strcat(tmpName, insPath3);   
    if (*(tmpName + strlen(tmpName) - 1) != '/')	
	strcat(tmpName, "/");

    if (*fileName != '\0')
	strcat(tmpName, fileName);

    if (slxResolvePath(tmpName, &chrPtr, error) == FAILURE)
	return FAILURE;
    
    slxCOPY_MAX_N_BYTES(completeName, chrPtr, slxFILENAME_LEN, error);

    return SUCCESS;
}

ccsCOMPL_STAT slxBuildInsUserDir(slxFILENAME  insUserDir,
				 char         *user,
				 ccsERROR     *error)
{
    char  *chrPtr;

    if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	{
	errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
	       NULL, slxINS_ROOT);
	return FAILURE;
	}

    strcpy(insUserDir, chrPtr);
    if (slxGetLastChar(insUserDir) != '/')
	strcat(insUserDir, "/");

    if (*user == '\0')
	{
	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }
	strcat(insUserDir, chrPtr);
	}
    else
	strcat(insUserDir, user);	

    if (slxGetLastChar(insUserDir) != '/')
	strcat(insUserDir, "/");

    return SUCCESS;
}


/* Print the contents of the CCS Error Structure to Standard Error
 */
ccsCOMPL_STAT slxPrintError(ccsERROR  *error)
{
    return(errPrint(error));
}


/* Assemble a keyword properly
 */
ccsCOMPL_STAT slxAssembleKeyword(const slxCATEGORY   category,
				 const slxSUBSYSTEM  subSystem,
				 const slxPARAMETER  parameter,
				 slxKEYW_LINE        keyword,
				 ccsERROR            *error)
{
    *keyword = '\0';

    if (*category != '\0')
	{
	strcpy(keyword, category);
	strcat(keyword, ".");
	}

    if (*subSystem != '\0')
	{
	strcat(keyword, subSystem);
	strcat(keyword, ".");
	}

    strcat(keyword, parameter);
    
    return SUCCESS;
}

/* Assemble a keyword - return pointer to internal static buffer
 */
char *slxAssKey(const char *key1, const int index1, ...)
{
    static slxKEYW_LINE  keyword;
    char                 *argPtr;
    int                  ival;
    slxKEYW_LINE         tmpBuf;
    va_list              ap;

    *keyword = '\0';
    if (index1 && (*key1 != '\0'))
	sprintf(keyword, "%s%d", key1, index1);
    else if (*key1 != '\0')
	strcpy(keyword, key1);
   
    va_start(ap, index1);
    argPtr = va_arg(ap, char *);
    while (argPtr != NULL)
	{
	if (*argPtr != '\0')
	    {
	    if (*keyword != '\0')
		strcat(keyword, ".");
	    strcat(keyword, argPtr);
	    ival = va_arg(ap, int);
	    /* Add index if larger than 0
	     */
	    if (ival)
		{
		sprintf(tmpBuf, "%d", ival);
		strcat(keyword, tmpBuf);
		}
	    }
	else
	    ival = va_arg(ap, int);
	argPtr = va_arg(ap, char *);
	}
    va_end(ap);

    return keyword;
}

char *slxAssKey2(const char *key1, ...)
{
    static slxKEYW_LINE  keyword;
    char                 *argPtr;
    va_list              ap;

    *keyword = '\0';
    if (*key1 != '\0')
	strcpy(keyword, key1);
   
    va_start(ap, key1);
    argPtr = va_arg(ap, char *);
    while (argPtr != NULL)
	{
	if (*argPtr != '\0')
	    {
	    if (*keyword != '\0')
		strcat(keyword, ".");
	    strcat(keyword, argPtr);
	    }
	argPtr = va_arg(ap, char *);
	}
    va_end(ap);

    return keyword;
}


vltLOGICAL slxIsShortFITS(const slxKEYW_LINE  keyword)
{
    char  *chrPtr;

    /* Expect that there is a point '.' within the first word.        
     */
    chrPtr = (char *)keyword;
    while ((*chrPtr != ' ') && (*chrPtr != '\0') && (*chrPtr != '.'))
	chrPtr++;

    if (*chrPtr == '.')
	return TRUE;
    else
	return FALSE;
}

char slxGetLastChar(char *string)
{
  if (string != NULL)
    {
    if (*string != '\0')
      return (*(string + (strlen(string) - 1)));
    else
      return 0;
    }
  else
    return 0;
}

void slxSetLastChar(char   *string, 
		    char   chr)
{
    vltINT32 len;
   
    if (slxGetLastChar(string) != chr)
	{
	len = strlen(string);
	*(string + len) = chr;
	*(string + (len + 1)) = '\0';
	}
}

ccsCOMPL_STAT slxKeywClass2No(const char        *classStr, 
			      slxKEYWORD_CLASS  *classNo,
			      ccsERROR          *error)
{
    if (strstr(classStr, slxSETUP_CLASS_STR) != NULL)
	*classNo = slxSETUP_CLASS;
    else if (strstr(classStr, slxHEADER_CLASS_STR) != NULL)
	*classNo = slxHEADER_CLASS;
    else if (strstr(classStr, slxCONFIG_CLASS_STR) != NULL)
	*classNo = slxCONFIG_CLASS;
    else if (strstr(classStr, slxPRIVATE_CLASS_STR) != NULL)
	*classNo = slxPRIVATE_CLASS;
    else if (strstr(classStr, slxTEMPLATE_CLASS_STR) != NULL)
	*classNo = slxTEMPLATE_CLASS;
    else if (strstr(classStr, slxMAINT_HEADER_CLASS_STR) != NULL)
	*classNo = slxMAINT_HEADER_CLASS;
    else if (strstr(classStr, slxCONF_LOG_CLASS_STR) != NULL)
	*classNo = slxCONF_LOG_CLASS;
    else if (strstr(classStr, slxCOND_LOG_CLASS_STR) != NULL)
	*classNo = slxCOND_LOG_CLASS;
    else if (strstr(classStr, slxOPS_LOG_CLASS_STR) != NULL)
	*classNo = slxOPS_LOG_CLASS;
    else if (strstr(classStr, slxREDUC_LOG_CLASS_STR) != NULL)
	*classNo = slxREDUC_LOG_CLASS;
    else if (strstr(classStr, slxALL_CLASS_STR) != NULL)
	*classNo = slxALL_CLASS;
    else
	{
	*classNo = slxANY_CLASS;
	errAdd(error, slxMODULE_NAME, slxERR_UNKNOWN_DIC_CLASS, __FILE__,
               "%s", classStr);	
	return FAILURE;
	}

    return SUCCESS;
}

/* Gets the value of the specified environment variable, and copies it to the
 * specified buffer. It is checked that the length of the environment variable
 * does not exceed the length of the "destBuf".
 */
ccsCOMPL_STAT slxGetValueEnvVar(const char      *envVar,
				char            **destBuf,
				ccsERROR        *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL  initialized=ccsFALSE;
    static slxDYN_BUF  varBuf;
    char               *chrPtr;
    slxFILENAME        tmpBuf;


    if (!initialized)
	{
	if (slxInitDynBuf(&varBuf, error) == FAILURE)
	    return FAILURE;
	initialized = TRUE;
	}
    if (slxSoftResetDynBuf(&varBuf, error) == FAILURE)
	return FAILURE;
    
    *destBuf = NULL;
    slxCOPY_MAX_N_BYTES(tmpBuf, envVar, sizeof(slxFILENAME), error);
    if ((chrPtr = getenv(tmpBuf)) == NULL)
	{
	errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
	       "%s", tmpBuf);
	return FAILURE;
	}
    if (slxStoreBufDynBuf(&varBuf, chrPtr, strlen(chrPtr), error) == FAILURE)
	return FAILURE; 
    if (slxStoreBufDynBuf(&varBuf, "", 1, error) == FAILURE)
	return FAILURE; 
    varBuf.bytesStored--;   

    *destBuf = varBuf.dynBuf;


    return SUCCESS;
}

ccsCOMPL_STAT slxBitPix2NoBytes(const vltINT32  bitPix,
				vltINT32        *noOfBytes,
				ccsERROR        *error)
{
    switch(bitPix)
	{
	case 8:
	    *noOfBytes = sizeof(char);
	    break;
	case 16:
	    *noOfBytes = sizeof(vltINT16);
	    break;
	case 32:
	case -32:
	    *noOfBytes = sizeof(vltINT32);
	    break;
	case -64:
	    *noOfBytes = 8;
	    break;
	default:
	    break;
	}

    return SUCCESS;
}

ccsCOMPL_STAT slxGetKeywordIndex(const slxKEYW_LINE  keyword, 
				 vltINT32            *index,
				 ccsERROR            *error)
{
    char  *chrPtr;

    chrPtr = (char *)(keyword + (strlen(keyword) - 1));
    while (isdigit((int)*chrPtr))
	chrPtr--;
    chrPtr++;
    
    *index = atoi(chrPtr);

    return SUCCESS;
}


ccsCOMPL_STAT slxResolvePath(const slxFILENAME   orgPath,
			     char                **completePath,
			     ccsERROR            *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL   initialized=ccsFALSE;
    static slxDYN_BUF   complPath;
    char                *chrPtr, *pathPtr, *tmpPtr;
    char                *nullTerm = "\0";
    slxFILENAME         tmpPath;
    vltINT32            len;


    vltINT32            currLen=0;
    vltINT32            k=0;
    char                c;
    
    if (!initialized)
	{
	if (slxInitDynBuf(&complPath, error) == FAILURE)
	    return FAILURE;
	if (slxDynBufAlloc(&complPath, 256, error) == FAILURE)
	    return FAILURE;
	initialized = TRUE;
	}
    if (slxSoftResetDynBuf(&complPath, error) == FAILURE)
	return FAILURE;

    *completePath = NULL;
    pathPtr = (char *)orgPath;
    chrPtr = strchr(pathPtr, '/');
    do
	{

	if (*pathPtr == '$')
	    {	
	    if (chrPtr != NULL)    
		len = ((chrPtr - pathPtr) - 1);
	    else
		len = strlen(pathPtr);
	    strncpy(tmpPath, (pathPtr + 1), len);
	    *(tmpPath + len) = '\0';
	    if (slxGetValueEnvVar(tmpPath, &tmpPtr, error) == FAILURE)
		return FAILURE;
	    if (slxStoreBufDynBuf(&complPath, tmpPtr, strlen(tmpPtr), 
				  error) == FAILURE)
		return FAILURE;
	    }
	else if (*pathPtr == '~')
	    {
	    /* Path of the format: "~/MY_DIR/" or "~<user>/"
	     */
	    slxCOPY_MAX_N_BYTES(tmpPath, "HOME", sizeof(slxFILENAME),
				error);
	    if (slxGetValueEnvVar(tmpPath, &tmpPtr, error) == FAILURE)
		return FAILURE;
	    if (slxStoreBufDynBuf(&complPath, tmpPtr, strlen(tmpPtr), 
				  error) == FAILURE)
		return FAILURE;
	    }

	else if (strstr(pathPtr, "./") == pathPtr)
	    {
	    
	    if(strlen(pathPtr)==strlen((char *)orgPath))
		{
                /* path starts with ./
		 * replace it with real path
		 * VLTSW2000078
		 */
/* VLTSW20050173 */
#ifndef MAKE_LCU
		if (realpath("./",tmpPath) == NULL)
		    return FAILURE;
#endif
		
		if (slxStoreBufDynBuf(&complPath, tmpPath, strlen(tmpPath), 
				      error) == FAILURE)
		    return FAILURE;
		}
	    else
		{
		/* path contains  ./
		 * remove last /
		 */
		currLen=slxSizeDynBuf(&complPath);
		if(slxSetCharDynBuf(&complPath,currLen, '\0',error)!=SUCCESS)
		    {
		    return FAILURE;
		    }
		complPath.bytesStored=currLen-1;
		}
	    }

	else if (strstr(pathPtr, "../") == pathPtr)
	    {

	    
	    if(strlen(pathPtr)==strlen((char *)orgPath))
		{
		/*
		 * path starts with ../
		 * replace ../ with the directory under the current path 
		 *
		 * VLTSW20010753 
		 * if (slxGetValueEnvVar("PWD", &tmpPtr, error) == FAILURE)
		 * return FAILURE;	     
		 * strcpy(tmpPath, tmpPtr);
		 */
		if (getcwd(tmpPath, sizeof(slxFILENAME) - 1) == NULL)
		    {
		    errAdd(error, slxMODULE_NAME, slxERR_GENERIC_W, __FILE__, 
			   NULL, "Failure while calling getcwd");
		    return FAILURE;
		    }
		
		slxYankLastPath(tmpPath);
		if (slxStoreBufDynBuf(&complPath, tmpPath , strlen(tmpPath ),
				      error) == FAILURE)
		    return FAILURE;
		
		}

	    else
		{
		/*
		 * path contains ../
		 * remove the previous element from the path
		 */
	
		/*declare previous element*/
		currLen=slxSizeDynBuf(&complPath);
		k=0;
		do 
		    { 
		    k++;
		    c=slxGetCharDynBuf(&complPath,currLen-k);   
		    } 
		while (c!='/' && k<=currLen);
		
		/*remove previous element */
		if(slxSetCharDynBuf(&complPath,currLen-k, '\0',error)!=SUCCESS)
		    {
		    return FAILURE;
		    }
		complPath.bytesStored=currLen-k-1;
		}
	    }  
	else
	    {
	    if (chrPtr != NULL)
		len = (chrPtr - pathPtr);
	    else
		len = strlen(pathPtr);
	    strncpy(tmpPath, pathPtr, len);
	    *(tmpPath + len) = '\0';
	    if (slxStoreBufDynBuf(&complPath, tmpPath, strlen(tmpPath), 
				  error) == FAILURE)
		return FAILURE;
	    }

	if (slxStoreBufDynBuf(&complPath, "/", 1, error) == FAILURE)
	    return FAILURE;

	if (chrPtr != NULL)
	    {
	    if (*chrPtr != '\0')
		pathPtr = (chrPtr + 1);
	    else
		pathPtr = chrPtr;
	    }
	else 
	    pathPtr = nullTerm;

	if (*pathPtr == ':')
	    {
	    if (slxStoreBufDynBuf(&complPath, ":", 1, error) == FAILURE)
		return FAILURE;
	    pathPtr++;
	    }
	}
    while (((chrPtr = strchr(pathPtr, '/')) != NULL) || (*pathPtr != '\0'));

    /* Since we cannot know if a filename is contained in the path, we
     * should not allow slash in the end of the complete path
     */
    if (*(complPath.dynBuf + complPath.bytesStored - 1) == '/')
	complPath.bytesStored--;

    /* Make sure the path name is terminated with '\0'
     */
    if (complPath.bytesStored == complPath.bytesAlloc)
	{
	if (slxStoreBufDynBuf(&complPath, " ", 1, error) == FAILURE)
	    return FAILURE;
	*(complPath.dynBuf + (complPath.bytesStored - 1)) = '\0';
	}
    else
	*(complPath.dynBuf + complPath.bytesStored) = '\0';

    *completePath = complPath.dynBuf;

    return SUCCESS;
}


/* Function that replace a possible index in a keyword with "i" and return
 * the index if defined
 */
void slxReplaceIndex(slxKEYW_LINE  keyword, vltINT32 *index)
{
    char      tempBuf[128];
    vltINT32  i = 0;
    
    while (isdigit((int)*(keyword + (strlen(keyword) - 1))))
	{
	*(tempBuf + i++) = *(keyword + (strlen(keyword) - 1));
	*(keyword + (strlen(keyword) - 1)) = '\0';    
	}
    *(tempBuf + i) = '\0';
    *index = atoi(tempBuf);
    if (*index)
	strcat(keyword, "i");        
}

ccsCOMPL_STAT slxProbeReplaceIndeces(const slxKEYW_LINE  keyword,
				     slxKEYW_LINE        subStKeyw,
				     ccsERROR            *error)
{
    char           *chrPtr;
    slxKEYW_LINE   keyElements[slxMAX_NO_OF_KEY_ELS];
    slxVALUE       value;
    slxCOMMENT     comment;
    vltINT32       index, i; 
    vltINT32       noOfKeyElements;
    vltLOGICAL     didSubstitution;

    
    *subStKeyw = '\0';

    if (slxSplitShortFitsDyn(keyword, slxMAX_NO_OF_KEY_ELS, keyElements, 
			     &noOfKeyElements,
			     value, comment, error) == FAILURE)
	return FAILURE;

    didSubstitution = FALSE;
    for (i = 0; i < noOfKeyElements; i++)
	{
	chrPtr = keyElements[i];
	if (isdigit((int)slxGetLastChar(chrPtr)) && (*chrPtr != '\0'))
	    {
	    slxReplaceIndex(chrPtr, &index);
	    didSubstitution = TRUE;
	    }
	}

    if (didSubstitution)
	{
	strcpy(subStKeyw, keyElements[0]);
	for (i = 1; i < noOfKeyElements; i++)
	    {
	    strcat(subStKeyw, ".");
	    strcat(subStKeyw, keyElements[i]);
	    }
	}
    else
	{
	slxCOPY_MAX_N_BYTES(subStKeyw, keyword, sizeof(slxKEYW_LINE), error);
	}

    return SUCCESS;
}

vltLOGICAL slxStr2Logical(const char  *strLogical)
{
    if (strchr(strLogical, 'T') != NULL)
	return TRUE;
    else
	return FALSE;
}

char *slxGetExtension(const slxFILENAME fileName)
{
    char *chrPtr, *chrPtr2;

    if ((chrPtr = strrchr(fileName, '.')) == NULL)
	return NULL;
    else
	{
	/* Check that the 'extension' found is not part of the filename	   
	 * like e.g.:
	 *
	 * "/home/ins1b/jknudstr/.fhtHome/DICTIONARY/ALL"
	 *
	 * Check also the following:
	 *
	 * "/home/ins1b/jknudstr/.fhtHome"
	 *
	 * In this case there is no extension.
	 */
	chrPtr2 = strrchr(fileName, '/');
	if (chrPtr2 > chrPtr)
	    return NULL;
	else if (*(chrPtr - 1) == '/')
	    return NULL;
	else
	    return (chrPtr + 1);
	}
}

void slxCheckExtension(slxFILENAME fileName, const char *extension)
{
    if (strstr(fileName, extension) == NULL)
	strcat(fileName, extension);
}

vltLOGICAL slxCheckExtension2(slxFILENAME fileName, const char *extension)
{
   char *chrPtr;
   
   chrPtr = slxGetExtension(fileName);
   if ((*extension == '\0') && (chrPtr == NULL))
       return TRUE;
   else if (chrPtr == NULL)
       return FALSE;
   else if (strcmp(extension, chrPtr) == 0)
       return TRUE;
   else
       return FALSE;
}

void slxYankExtension(slxFILENAME fileName, const slxFILENAME extension)
{    
    char         *chrPtr;
    slxFILENAME  extLoc;

    if (*extension != '.')
	sprintf(extLoc, ".%s", extension);
    else
	sprintf(extLoc, "%s", extension);
    if ((chrPtr = strstr(fileName, extLoc)) != NULL)
	*chrPtr = '\0';
}

void slxYankExtensionGen(slxFILENAME fileName)
{
    char  *chrPtr;

    if ((chrPtr = strchr(fileName, '.')) != NULL)
	*chrPtr = '\0';
}

void slxYankExtensionGenLast(slxFILENAME fileName)
{    
  int      pos;
  char     *ext;
 
  /* return if the filename is null */
  if (fileName==NULL)
    return;

  /* declare extention */
  if ((ext=slxGetExtension(fileName))==NULL)
    return;
  else
    {
      pos=strlen(fileName)-strlen(ext)-1;
      fileName[pos]='\0';
    }

}

void slxYankTrailNewLines(char *str)
{
    while (slxGetLastChar(str) == '\n')
	*(str + (strlen(str) - 1)) = '\0';
}

ccsCOMPL_STAT slxSplitFileName(const slxFILENAME  completeName,
			       slxFILENAME        path,
			       slxFILENAME        fileName,
			       slxFILENAME        extension,
			       ccsERROR           *error)
{
    char         *chrPtr;
    slxFILENAME  completeNameLoc;

    if (slxResolvePath(completeName, &chrPtr, error) == FAILURE)
	return FAILURE;
    slxCOPY_MAX_N_BYTES(completeNameLoc, chrPtr, sizeof(slxFILENAME), error); 

    if (slxGetPath(completeNameLoc, path, error) == FAILURE)
	return FAILURE;

    *fileName = '\0';
    if (slxGetFileName(completeNameLoc, fileName, error) == FAILURE)
	return FAILURE;
    slxYankExtensionGen(fileName);

    chrPtr = slxGetExtension(completeNameLoc);
    if (chrPtr != NULL)
	{
	slxCOPY_MAX_N_BYTES(extension, chrPtr, sizeof(slxFILENAME), error);
	}
    else
	*extension = '\0';

    return SUCCESS;
}

/* Note, logical is returned as string
 */
void slxDetermineType(const char   *value,
		      slxPAR_TYPE  *type)
{
    vltDOUBLE  dblVal;
    vltINT32   intVal;
    
    /* Check first if integer
     */
    if (sscanf(value, "%d", &intVal) == 1)
	*type = slxINTEGER;
    else if (sscanf(value, "%lf", &dblVal) == 1)
	*type = slxDOUBLE;
    else
	*type = slxSTRING;
}

void slxTrimString(char *string, char *trimChars)
{
    char        *chrPtr;
    vltLOGICAL  run;

    if (*string != '\0')
	{
	/* Remove initiating trim characters
	 */
	run = TRUE;
	chrPtr = string;
	do
	    {
	    if (strchr(trimChars, *chrPtr) != NULL)
		chrPtr++;
	    else
		run = FALSE;
	    }
	while ((*chrPtr != '\0') && run);
/* MNY Problems with APR2004 release. 
       Check that string is not copying itself, VLTSW20040034 
	strcpy(string, chrPtr);
*/
	if (string != chrPtr)
	    strcpy(string, chrPtr);

	/* Remove trailing trim characters
	 */
	if (*chrPtr != '\0')
	    {
	    chrPtr = (string + (strlen(string) - 1));
	    run = TRUE;
	    do
		{
		if (strchr(trimChars, *chrPtr) != NULL)
		    {
		    *chrPtr = '\0';
		    chrPtr--;
		    }
		else
		    run = FALSE;
		}
	    while ((*chrPtr != '\0') && run);
	    }
	}
}

ccsCOMPL_STAT slxSplit(const char       *separators, 
		       const char       *stringPtr, 
		       const vltINT32   maxNoOfSubStr, 
		       slxKEYW_LINE     subStrings[], 
		       vltINT32         *noOfSubStr, 
		       ccsERROR         *error) 
{ 
    char     *strPtr, *nextField; 
    vltINT32 i; 
    
    strPtr = (char *)stringPtr;     
    for (i = 0; i < maxNoOfSubStr; i++) 
	{ 
	if (slxGetNextFieldMulti(separators, strPtr, &nextField, 
				 &strPtr, error) != SUCCESS) 
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(subStrings[i], nextField, sizeof(slxKEYW_LINE),
			    error);
	if ((strPtr == NULL) || (strlen(strPtr) == 0))
	    { 
	    if (noOfSubStr != NULL)
		{
		if (*subStrings[0] == '\0')
		    *noOfSubStr = 0;
		else
		    *noOfSubStr = (i + 1);
		}
	    return SUCCESS; 
	    } 
	} 
    
    if (noOfSubStr != NULL) 
	*noOfSubStr = i; 
    
    return SUCCESS; 
} 

ccsCOMPL_STAT slxStopWatch(const vltINT32    watchNo,
			   const vltLOGICAL  start,
			   vltUINT32         *timeSec,
			   vltUINT32         *timeUSec,
			   ccsERROR          *error) 
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
/* VLTSW20050173 */
#if defined(MAKE_LCU)
    static ccsTIMEVAL  watchList[slxMAX_NO_OF_WATCHES];
    ccsTIMEVAL         timeNow;
#else
    static struct timeval  watchList[slxMAX_NO_OF_WATCHES];
    struct timeval         timeNow;
#endif

    if (!initialized)
	{
/* VLTSW20050173 */
#if defined(MAKE_LCU)
	memset(watchList, 0, (slxMAX_NO_OF_WATCHES * sizeof(ccsTIMEVAL)));
#else
	memset(watchList, 0, (slxMAX_NO_OF_WATCHES * sizeof(struct timeval)));
#endif
	}
    if ((watchNo < 0) || (watchNo > (slxMAX_NO_OF_WATCHES - 1)))
	{
	errAdd(error, slxMODULE_NAME, slxERR_GENERIC_W, __FILE__, 
	       NULL, "SLX Stop Watch number given is out of range");
	return FAILURE;
	}
    
    if (start)
	{
	slx_gettimeofday(&(watchList[watchNo]));
	/*gettimeofday(&(watchList[watchNo]), NULL);*/
	}
    else
	{ 
	slx_gettimeofday(&timeNow);
	/*gettimeofday(&timeNow, NULL);*/
	*timeSec = (timeNow.tv_sec - watchList[watchNo].tv_sec);
	if (timeNow.tv_usec >= watchList[watchNo].tv_usec)
	    *timeUSec = (timeNow.tv_usec - watchList[watchNo].tv_usec);
	else
	    {
	    *timeUSec = (1000000 + timeNow.tv_usec-watchList[watchNo].tv_usec);
	    (*timeSec)--;
	    }
	}

    return SUCCESS;
}

ccsCOMPL_STAT slxBuildImageFileName(const slxFILENAME  fileName,
				    slxFILENAME        completeName,
				    ccsERROR           *error)
{
    char          *chrPtr;    
    struct stat   statBuf;
    slxFILENAME   tmpName;

    
    if (strstr(fileName, slxFITS_FILE_EXT) == NULL)
	{
	strcpy(tmpName, fileName);
	strcat(tmpName, slxFITS_FILE_EXT);
	}
    else
	strcpy(tmpName, fileName);

    /* Try to stat() the file as it is
     */
    if (stat(tmpName, &statBuf) == 0)
	{
	if (*tmpName != '/')
	    {
	    /* VLTSW20010753
	     *if ((chrPtr = getenv("PWD")) == NULL)
	     *	{
	     *	errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF,
	     *	       __FILE__, NULL, "PWD");
	     *	return FAILURE;
	     *	}
	     *strcpy(completeName, chrPtr);
	     ****
	     * as completeName passed as argument the relative size of it 
	     * is not calculateded properly by slxFILENAME
	     */
	    if (getcwd(completeName, sizeof(slxFILENAME) - 1) == NULL)
                {
                errAdd(error, slxMODULE_NAME, slxERR_GENERIC_W, __FILE__, 
                       NULL, "Failure while calling getcwd");
                return FAILURE;
                }
	    /*printf("2 %s\n", completeName);*/
	    slxSetLastChar(completeName, '/');
	    }
	else
	    *completeName = '\0';
	strcat(completeName, tmpName);
	}
    else if ((*tmpName == '~') || (*tmpName == '$'))
	{
	if (slxResolvePath(tmpName, &chrPtr, error) == FAILURE)
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(completeName, chrPtr, slxFILENAME_LEN, error);
	}
    else if ((*tmpName == '.') && (*(tmpName + 1) == '/'))
	{

	/* VLTSW20010753
	 *if ((chrPtr = getenv("PWD")) == NULL)
	 *    {
	 *    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
	 *	   NULL, "PWD");
	 *    return FAILURE;
	 *    }
	 *strcpy(completeName, chrPtr);
	 */
	 if (getcwd(completeName, sizeof(slxFILENAME) - 1) == NULL)
            {
            errAdd(error, slxMODULE_NAME, slxERR_GENERIC_W, __FILE__, 
                   NULL, "Failure while calling getcwd");
            return FAILURE;
            }
	 /* printf("3 %s\n", completeName);*/
	

	slxSetLastChar(completeName, '/');
	strcat(completeName, tmpName);
	}
    else if (*tmpName != '/')
	{
	/* Get the value of INS_ROOT
	 */
	
	if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_ROOT);
	    return FAILURE;
	    }
	strcpy(completeName, chrPtr);
	if ((*(completeName + (strlen(completeName) - 1)) != '/') &&
	    (*tmpName != '/'))
	    strcat(completeName, "/");

	/* Get value of INS_USER
	 */
	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }
	strcat(completeName, chrPtr);
	if ((*(completeName + (strlen(completeName) - 1)) != '/') &&
	    (*tmpName != '/'))
	    strcat(completeName, "/");
	strcat(completeName, "DETDATA/");

	strcat(completeName, tmpName);
	}
    else
	{
	/* It's the complete path
	 */
	strcpy(completeName, tmpName);
	}


    return SUCCESS;
}

void slxYankLastPath(slxFILENAME  path)
{
    char *chrPtr;
    if ((chrPtr = strrchr(path, '/')) != NULL)
	*chrPtr = '\0';
}

/*
 *****************************************************************************
 */

/*___oOo___*/



