#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxAddFITSBlocks.test.tcl,v 1.145 2005/07/27 12:19:25 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
# epozna    30/07/02  added VLTSW20020319 slxTestAddBlocksWithEnd

#************************************************************************
#   NAME
#   slxAddFITSBlocks.test.prog - test script for slxAddFITSBlocks()
# 
#   SYNOPSIS
#   slxAddFITSBlocks.test.prog
# 
#   DESCRIPTION
#   Used for testing the SLX function slxAddFITSBlocks().
#
#   The following tests are carried out
#   
#   1.1 Normal behaviour - complete one incomplete block
#
#   1.2 Normal behaviour - complete header with several blocks (add END)
#
#   1.3 Normal behaviour - create 3 blocks
#
#   1.4 Normal behaviour - update from 1 to 3 blocks + add END
#
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs

#set verbose 1
set tf slxAddFITSBlocks.test
set testPath [pwd]
set tmpPath ""
set slxTestAddBlocks ../bin/slxTestAddBlocks
set slxTestAddBlocksWithEnd ../bin/slxTestAddBlocksWithEnd

# PART 1.1 Normal behaviour - complete one incomplete block
# =========================================================
test slxAddFITSBlocks-1.1 {Complete one incomplete block} {
    exec cp $tf.1.1.bs $tf.1.1.temp
    exec chmod ugo=+rwx $tf.1.1.temp
    exec $slxTestAddBlocks $testPath/$tf.1.1.temp 0 NO
    set msg [exec diff $tf.1.1.temp $tf.1.1.rf]
    exec rm $tf.1.1.temp
    set msg
} ""



# PART 1.2 Normal behaviour - complete header with several blocks (add END)
# =========================================================================
test slxAddFITSBlocks-1.2 {Complete header with several blocks (add END)} {
    exec cp $tf.1.2.bs $tf.1.2.temp
    exec chmod ugo=+rwx $tf.1.2.temp
    exec $slxTestAddBlocks $testPath/$tf.1.2.temp 0 YES
    set msg [exec diff $tf.1.2.temp $tf.1.2.rf]
    exec rm $tf.1.2.temp
    set msg
} ""


# PART 1.3 Normal behaviour - create 3 blocks
# ===========================================
test slxAddFITSBlocks-1.3 {Normal behaviour create 3 blocks} {
    exec cp $tf.1.3.bs $tf.1.3.temp
    exec chmod ugo=+rwx $tf.1.3.temp 
    exec $slxTestAddBlocks $testPath/$tf.1.3.temp 3 NO
    set msg [exec diff $tf.1.3.temp $tf.1.3.rf]
    exec rm $tf.1.3.temp
    set msg
} ""



# PART 1.4 Normal behaviour - update from 1 to 3 blocks + add END
# ===============================================================
test slxAddFITSBlocks-1.4 {Update from 1 to 3 blocks + add END} {
    exec cp $tf.1.4.bs $tf.1.4.temp
    exec chmod ugo=+rwx $tf.1.4.temp
    exec $slxTestAddBlocks $testPath/$tf.1.4.temp 2 YES
    set msg [exec diff $tf.1.4.temp $tf.1.4.rf]
    exec rm $tf.1.4.temp
    set msg
} ""

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# EPO
# VLTSW20020319 slxTestAddBlocksWithEnd
# note: this functionaly makes sure that the END kw is present at the 
# end of block. Above test are updated for this case.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

# PART 1.5 Normal behaviour - complete one incomplete block
# =========================================================
test slxTestAddBlocksWithEnd-1.5 {VLTSW20020319:Complete one incomplete block} {
    exec cp $tf.1.1.bs $tf.1.5.temp
    exec chmod ugo=+rwx $tf.1.5.temp
    exec $slxTestAddBlocksWithEnd $testPath/$tf.1.5.temp 0 
    set msg [exec diff $tf.1.5.temp $tf.1.5.rf]
    exec rm $tf.1.5.temp
    set msg
} ""


# PART 1.6 Normal behaviour - complete header with several blocks (add END)
# =========================================================================
test slxTestAddBlocksWithEnd-1.6 {VLTSW20020319:Complete header with several blocks } {
    exec cp $tf.1.2.bs $tf.1.6.temp
    exec chmod ugo=+rwx $tf.1.6.temp
    exec $slxTestAddBlocksWithEnd $testPath/$tf.1.6.temp 0 
    set msg [exec diff $tf.1.6.temp $tf.1.2.rf]
    exec rm $tf.1.6.temp
    set msg
} ""



# new PART 1.7 Normal behaviour - add 3 blocks
# ===========================================
test slxTestAddBlocksWithEnd-1.7 {VLTSW20020319:Normal behaviour create 3 blocks} {
    exec cp $tf.1.3.bs $tf.1.7.temp
    exec chmod ugo=+rwx $tf.1.7.temp  
    #echo '3 $testPath/$tf.1.7.temp'
    exec $slxTestAddBlocksWithEnd $testPath/$tf.1.7.temp 3 
    set msg [exec diff $tf.1.7.temp $tf.1.7.rf]
    exec rm $tf.1.7.temp
    set msg
} ""



# PART 1.8 Normal behaviour - update from 1 to 3 blocks ( add END)
# ===============================================================
test slxTestAddBlocksWithEnd-1.8 {VLTSW20020319:Update from 1 to 3 blocks } {
    exec cp $tf.1.4.bs $tf.1.8.temp
    exec chmod ugo=+rwx $tf.1.8.temp
    exec $slxTestAddBlocksWithEnd $testPath/$tf.1.8.temp 2 
    set msg [exec diff $tf.1.8.temp $tf.1.4.rf]
    exec rm $tf.1.8.temp
    set msg
} ""
