#!/usr/local/bin/tcl
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCheckSyntaxLine.test.tcl,v 1.145 2005/07/27 10:37:38 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxCheckSyntaxLine.test.prog - test script for slxCheckSyntaxLine
# 
#   SYNOPSIS
#   slxCheckSyntaxLine.test.prog
# 
#   DESCRIPTION
#   The script test the function slxCheckSyntaxLine that tests the syntax 
#   of a set-up line in the Short-FITS format.
#
#   The script carries out the following tests:
#
#   1. Test of handling of failure in locating dictionary.
#      Error constant: slxERR_SYN_LOCATE_DICO.
#
#   2. Test of handling of failure in opening dictionary.
#      Test files used:  ESO-VLT-DIC.FORS, test.ref
#      Error constant: slxERR_SYN_OPEN_DICO.
#
#   3. Test of an error in the header of a dictionary.
#      Test files used: ESO-VLT-DIC.EMMI, test.ref
#      Error constant: slxERR_SYN_HEADER_DICO.
#
#   4. Test of correct functionining of the function. Test file is used that
#      provoke all detectable syntax errors.
#      Test files used: ESO-VLT-DIC.ISAAC, slxCheckSyntaxLine.test.4.bs.rf
#
#
#   The script writes an error report to standard output. 
#
#
#   FILES
#   ESO-VLT-DIC.ISAAC                    Correct dictionary containing the 
#                                        necessary definitions.
#
#   ESO-VLT-DIC.FORS                     rwx protected dictionary - cannot be 
#                                        opened.
#
#   ESO-VLT-DIC.EMMI                     Dictionary with an error in the 
#                                        dictionary header.
#   
#   test.ref                             Just a dummy set-up file to give an
#                                        input to the function.
#
#   slxCheckSyntaxLine.test.4.bs.rf  Input set-up file containing keywords
#                                        that will provoke all detectable 
#                                        syntax errors.
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
global gvar

source ../test/defs
source ../test/slxTestCommon.tcl

#set verbose 1
set tf slxCheckSyntaxLine.test
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM
set gvar(writeErrorMsg) 0




# PART 1: Failure in locating the dictionary
# ==========================================
test slxCheckSyntaxLine-1 {Failure in locating dictionary} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup test.ref UVES" msg]} { 
	set msg [slxTestTestString "Error opening dictionary" $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""



# PART 2: Failure in opening dictionary
# =====================================
exec chmod -rwx $env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.FORS
test slxCheckSyntaxLine-2 {Attempt to open rwx protected dictionary} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup test.ref FORS -t" msg]} {
	set msg [slxTestTestString "Error opening dictionary" $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""
exec chmod +rw $env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.FORS


# PART 4: Test of basic functioning
# =================================
test slxCheckSyntaxLine-4 {Basic functionality test} {
    exec slxCheckSyntaxSetup $tf.4.bs.rf ISAAC -t > $tf.4.temp
    set msg [exec diff $tf.4.temp $tf.4.rf]
    exec rm $tf.4.temp
    set msg
} ""

# --- oOo ---
