#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxConvertToShort.test.tcl,v 1.145 2005/07/27 10:48:52 vltsccm Exp $" 
# 
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxConvertToShort.test.tcl - test program for slxConvertToShort()
#
#   SYNOPSIS
#   slxConvertToShort.test.tcl
#
#   DESCRIPTION
#   This script tests the slxConvertToShort program, as follows:
#	1.1 not existing FITS header file, INS_SETUPPATH not set - SKIPPED!!!
#	1.3 read-protected FITS header file
#	1.6 read-only FITS header file
#	1.7 "FITS header" file is not in FITS format
#
#	2.1 Normal execution of slxConvertToShort
#
#   To run these tests, the program slxSetupToShortFITS is used.  The tests
#   for the latter program should therefore be run before this one.
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   FILES
#   The following files are required and accessed under ~slx/test:
#	- slxFitsHeader.test.bs (tests 1.3, 1.6, 2.1)
#	- slxConvertToShort.test.tcl (test 1.7)
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs

set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM

catch {unset env(INS_SETUPPATH)} msg


# This test program will use the services of the program "slxSetupToShortFITS".
# It is therefore expected that the test suite for the latter was run 
# successfully before.
set tp slxSetupToShortFITS


# For more info, see also the test suite for "slxSetupToShortFITS".

# PART 1: produce errors
# ======================


# IMPL.: Temporary skipped - how is it possible to unset INS_SETUPPATH
#        when "exec" is used?
if {1 == 0} {
# ----------------------------------------------------------------------------
# Not existing file, and INS_SETUPPATH is not set
# This is an error produced by slxLocateSetupFile
# For all other errors that can come from slxLocateSetupFile: see
#     slxLocateSetupFile.test.tcl
test slxConvertToShort-1.1 {not existing FITS header file, INS_SETUPPATH not set} {
	catch "exec $tp sureUcantFindThis1 -t" msg
	set msg
}   "\nCouldn't convert the file - check if filename is correct!\n
slxERR_INS_SETUPPATH : Environment variable INS_SETUPPATH is not set"}

# ----------------------------------------------------------------------------
# Read protected file
set tf slxFitsHeader.test
test slxConvertToShort-1.3 {read protected FITS header file} {
    set env(INS_SETUPPATH) [pwd]
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp
    catch "exec chmod ugo=+rwx $tf.temp"
    chmod -r $tf.temp
    catch "exec $tp $tf.temp -t" msg
    unset env(INS_SETUPPATH)
    catch "exec chmod ugo=+rwx $tf.temp"
    exec rm $tf.temp
    set msg
}   "\nCouldn't convert the file - check if filename is correct!

slxERR_OPEN_FILE : Could not open file \"slxFitsHeader.test.temp\""


# ---------------------------------------------------------------------------
# How the hell do you produce a read-error???
#test slxConvertToShort-1.4 {file read error} {
# ....

# ---------------------------------------------------------------------------
# How the hell do you produce a write-error on the temporary file???
#test slxConvertToShort-1.5 {temporary file write error} {
# ....

# ---------------------------------------------------------------------------
# error renaming temporary file: if original file is read only
set tf slxFitsHeader.test
test slxConvertToShort-1.6 {read-only setupfile} {
    set env(INS_SETUPPATH) [pwd]
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp
    chmod 0444 $tf.temp
    catch "exec $tp $tf.temp -t" msg
    unset env(INS_SETUPPATH)
    catch "exec chmod ugo=+rwx $tf.temp"
    exec rm -f $tf.temp
    set msg
}   "\nCouldn't convert the file - check if filename is correct!

slxERR_OPEN_FILE : Could not open file \"slxFitsHeader.test.temp\""


# ---------------------------------------------------------------------------
# file is not in appropriate format
# This is an error produced by slxNormalToShortFITS
set tf slxConvertToShort.test.tcl
test slxConvertToShort-1.7 {"setupfile" is not in FITS format} {
    set env(INS_SETUPPATH) [pwd]
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf $tf.temp
    catch "exec $tp $tf.temp -t" msg
    unset env(INS_SETUPPATH)
    catch "exec chmod ugo=+rwx $tf.temp"
    exec rm $tf.temp
    set msg
}   "\nCouldn't convert the file - check if filename is correct!

slxERR_FILE_FORMAT : File \"slxConvertToShort.test.tcl.temp\" does not have the correct format"


# PART 2: test functionality
# ==========================

# ---------------------------------------------------------------------------
# file is to be found in $INS_SETUPPATH
# For setup file in other directories/paths: see slxLocateSetupFile.test.tcl
test slxConvertToShort-2.1 {file is to be found in $INS_SETUPPATH} {
    set tf slxFitsHeader.test
    set env(INS_SETUPPATH) [pwd]
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp 
    catch "exec chmod ugo=+rwx $tf.temp"
    if {[catch "exec $tp $tf.temp -t" msg] == 0} {
	set msg [exec diff $tf.temp $tf.rf]
    } 
    unset env(INS_SETUPPATH)
    catch "exec chmod ugo=+rwx $tf.temp"
    exec rm $tf.temp
    set msg
}   ""


# --- oOo ---
