#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxDetectFormat.test.tcl,v 1.145 2005/07/27 10:54:41 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxDetectFormat.test.prog - program for slxDetectFormat()
# 
#   SYNOPSIS
#   slxDetectFormat.test.prog
# 
#   DESCRIPTION
#   Used for testing test programs for the SLX module.
#
#   The following tests are carried out
#   
#   1.1 Check normal file (Short-FITS)
#
#   1.2 Check normal file (Normal-FITS)
#
#   1.3 Check file (neither Short-FITS nor Normal-FITS)
#
#   2.1 Non-existing set-up file specified
#
#   2.2 Error opening set-up file
#
#
#   The script writes an error report to standard output. 
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#

source ../test/defs
source ../test/slxTestCommon.tcl

#set verbose 1
set tf slxDetectFormat.test
set testPath [pwd]
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM



# PART 1.1 Check normal file (Short-FITS)
# =======================================
test slxDetectFormat-1.1 {Detect format Short-FITS} {
    if {[catch "exec -keepnewline slxWhichFormat $tf.1.1.bs.rf -t > $tf.1.1.temp " msg] == 0} {
        set msg [exec diff $tf.1.1.temp $tf.1.1.rf]
        exec rm $tf.1.1.temp
        } 
} ""



# PART 1.2 Check normal file (Normal-FITS)
# ========================================
test slxDetectFormat-1.2 {Detect format Normal-FITS} {
    if {[catch "exec -keepnewline slxWhichFormat -t > $tf.1.2.temp " msg] == 0} {
        set msg [exec diff $tf.1.2.temp $tf.1.2.rf]
        } 
} ""
exec rm $tf.1.2.temp


# PART 1.3 Check file (neither Short-FITS nor Normal-FITS)
# ========================================================
test slxDetectFormat-1.3 {Detect format - not Short, not Normal FITS} {
    if {[catch "exec -keepnewline slxWhichFormat $tf.1.3.bs -t > $tf.1.3.temp " msg] == 0} {
        set msg [exec diff $tf.1.3.temp $tf.1.3.rf]
        exec rm $tf.1.3.temp
        } 
} ""



# PART 2.1 Non-existing set-up file specified
# ===========================================
test slxDetectFormat-2.1 {Non-existing set-up file} {
   if {[catch "exec -keepnewline slxWhichFormat nOnExIsT -t" msg]} {
       set testString "Could not find \"nOnExIsT\" according to INS_SETUPPATH "
       set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""


# PART 2.2 Error opening set-up file
# ==================================
test slxDetectFormat-2.2 {Error opening set-up file} {
    exec chmod -rw $tf.2.2.bs
    catch "exec -keepnewline slxWhichFormat $tf.2.2.bs -t" msg
    exec chmod +rw $tf.2.2.bs
    set testString "Could not open file"
    set msg [slxTestTestString $testString $msg]
    set msg
} ""


