/******************************************************************************
* E.S.O. - VLT project
*
* slxDumpBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  23/01/95  created
*/

/************************************************************************
*   NAME
*   slxDumpBuf - dump contents of the SLX Dynamic Buffer
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxDumpBuf(slxDYN_BUF  *dynBuf,
*                            char        *fileName,
*                            ccsERROR    *error)
* 
* 
*   DESCRIPTION
*   Function to save the contents of the SLX Dynamic Buffer in a file.
*
*   dynBuf       IN  Pointer to SLX Dynamic Buffer.
*
*   fileName     IN  Name of file in which data will be stored.
*
*   error        OUT CCS error structure indicating a possible error 
*                    occurring.
*
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. The following error codes can be
*             be generated by the function:
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxDumpBuf.c,v 1.145 2005/07/27 10:24:26 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);


/* 
 * System Headers
 */

#include <stdio.h>


/* 
 * Local Headers
 */
#include "slx.h"

/* 
 * Function definition
 */

ccsCOMPL_STAT slxDumpBuf(slxDYN_BUF  *dynBuf,
			 char        *fileName,
			 ccsERROR    *error)
{
    FILE             *filePtr;
    slxKEYW_LINE     keywordLine;



    if ((filePtr = fopen(fileName, "w")) == NULL)
	return FAILURE;

    if (slxGetNextLineBuf(dynBuf, slxINIT_DYN_BUF, keywordLine,
			  error) == FAILURE)
	return FAILURE;

    do
	{
	if (slxGetNextLineBuf(dynBuf, slxDO_NOT_INIT, keywordLine,
			      error) == FAILURE)
	    return FAILURE;

	if (keywordLine[0] != '\0')
	    {
	    if (fprintf(filePtr, "%s", keywordLine) < 0)
		return FAILURE;
	    }
	}
    while (keywordLine[0] != '\0');

    fclose(filePtr);

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/

