#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxLocateLoadSaveFile.test.tcl,v 1.145 2005/07/27 11:02:57 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
# epozna    31/07/02  added test 1.6 for VLTSW20000108 (file end with comments) 
#

#**************************************************************************
#   NAME
#   slxLocateLoadSave.test.prog - test program for slxLocateSetupFile()
#
#   SYNOPSIS
#   slxLocateLoadSave.test.prog
#
#   DESCRIPTION
#
#   FILES
#
#   CAUTIONS
#
#-----------------------------------------------------------------------------
source ../test/defs
source ../test/slxTestCommon.tcl


cd ../test
set testDir [pwd]
set testFile1 slxLocateLoadSave.bs
set slxTestLoadSaveFile ../bin/slxTestLoadSaveFile
#VLTSW20000108
set slxTestDynBuf ../bin/slxTestDynBuf

# TEST 1.1: Load an existing file - full path
# ===========================================
test slxLocateLoadSave-1.1 {Load an existing file} {
	catch "exec $slxTestLoadSaveFile \"\" \
		$testDir/slxLocateLoadSave.bs \
		$testDir/slxLocateLoadSave.1.1.temp" msg
	set msg
} ""

# TEST 1.2: Save with complete path
# =========================================
test slxLocateLoadSave-1.2 {Save with complete path} {
    set msg [exec diff $testDir/slxLocateLoadSave.bs \
	    $testDir/slxLocateLoadSave.1.1.temp] 
    set msg
} ""
catch "exec rm $testDir/slxLocateLoadSave.1.1.temp"

# TEST 1.3: Replace existing file
# =========================================
catch "exec chmod ugo=+rwx $testDir/slxLocateLoadSave.1.3.temp"
catch "exec cp $testDir/slxLocateLoadSave.bs \
	$testDir/slxLocateLoadSave.1.3.temp"
catch "exec chmod ugo=+rwx $testDir/slxLocateLoadSave.1.3.temp"
set env(TEST_VAR) $testDir
test slxLocateLoadSave-1.3 {Replace existing file} {
    catch "exec $slxTestLoadSaveFile \
	    TEST_VAR \
	    $testDir/slxLocateLoadSave.bs \
	    $testDir/slxLocateLoadSave.1.3.temp" msg
    set msg
} ""
catch "exec rm $testDir/slxLocateLoadSave.1.3.temp"


# TEST 1.4: Save in first path in env var$$$
# =========================================
set env(TEST_VAR) $testDir:$testDir/../src:$testDir/../bin
test slxLocateLoadSave-1.4 {Save in first path in env var} {    
    catch "exec $slxTestLoadSaveFile \
	    TEST_VAR \
	    $testDir/slxLocateLoadSave.bs \
	    slxLocateLoadSave.1.4.temp" msg
    set msg
} ""
catch "exec rm $testDir/slxLocateLoadSave.1.4.temp"


# TEST 1.5: Load an existing file - env var
# ===========================================
test slxLocateLoadSave-1.5 {VLTSW20000108:Load an existing file - env var} {
	catch "exec $slxTestLoadSaveFile \
		 TEST_VAR \
		 slxLocateLoadSave.bs \
		 slxLocateLoadSave.1.5.temp" msg
	set msg
} ""
catch "exec rm $testDir/slxLocateLoadSave.1.5.temp"



# TEST 1.6: Load an existing file with comment at the end (VLTSW20000108)
# ===========================================
test slxLocateLoadSave-1.6 {VLTSW20000108 Load a file ends with comment} {
	exec $slxTestDynBuf > slxTestDynBuf.tmp
        set msg [exec diff $testDir/slxTestDynBuf.tmp $testDir/slxTestDynBuf.rs] 
        set msg
} ""
catch "exec rm $testDir/slxTestDynBuf.tmp"



# TEST 2.1: Load a non-existing file
# ========================================
test slxLocateLoadSave-2.1 {Load a non-existing file} {
    catch "exec $slxTestLoadSaveFile \
	    TEST_VAR \
	    nonExisting \
	    dontCare" msg
    set testString "Could not locate file \"nonExisting\""
    set msg [slxTestTestString $testString $msg]
    set msg
} ""

# TEST 2.2: Save in a non-existing directory
# ==========================================
test slxLocateLoadSave-2.2 {Save in a non-existing directory} {
    catch "exec $slxTestLoadSaveFile \
	    \"\" \
	    $testDir/slxLocateLoadSave.bs \
	    thisDirDoesntExis/newFile" msg
    set testString "The specified Environment Variable \"\" could"
    set msg [slxTestTestString $testString $msg]
    set msg
} ""


# --- oOo ---
