#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxParseSetup.test.tcl,v 1.145 2005/07/27 11:11:46 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxParseSetup.test.prog - test program for slxParseSetup()
#
#   SYNOPSIS
#   slxParseSetup.test.prog
#
#   DESCRIPTION
#   This script tests the slxParseSetup program, as follows:
#	1.1 unknown destination
#	1.2 unknown file, INS_SETUPPATH not set - SKIPPED!!!
#	1.3 unknown file, INS_SETUPPATH set
#	1.4 file in wrong format
#	1.5 read protected file
#	1.6 timeout on final reply
#
#	2.1 Normal execution of slxParseSetup - send TEL category across
#	2.2 Normal execution of slxParseSetup - send INS category across
#	2.3 Normal execution of slxParseSetup - 2 replies per command, don't
#	    wait for final reply
#	2.4 Normal execution of slxParseSetup - 2 replies per command, delay
#	    in between replies
#
#   To run these tests, the program slxTestParseSetup is used, together with
#   a partner program that will receive the commands and return replies 
#   (slxTestRecvCmd).
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   FILES
#   The following files are required and accessed under ~slx/test:
#	- slxParseSetup.test2.1.bs (tests 1.1, 1.5, 1.6, 2.1, 2.2, 2.3, 2.4)
#	- slxParseSetup.test.prog (test 1.4)
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs

#set VERBOSE 1

# testP is the name of the binary we use for testing our function
set testP slxTestParseSetup
set timeOut 10
set env(INS_SETUPPATH) [pwd]

if {[lindex $testP 0] == "no"} {error $testP}

set msgSend [exec which msgSend]
if {[lindex $msgSend 0] == "no"} {error $msgSend}

set tf slxParseSetup.test2.1.bs

# In case of errors, exec $testP does an exit(1), so from Tcl
# we have to catch this error

#####################################################################
############ IMPORTANT !!!!
#####################################################################
# IMPL.: This test is cancelled temporarily due to CCS light ########
#####################################################################
return

# 26.07.95/JKN: $env(RTAPENV) -> "" because of CCS-Light

# PART 1: argument testing
# ========================
catch {unset env(INS_SETUPPATH)} msg

#-------------------------------------------------------------------------
# Produce error for destination not known
test slxParseSetup-1.1 {unknown destination} {
	set file $tf;				# setup filename
	set cat TEL;				# category
	set wte   ""; 			        # environment name
	set proc unknown;			# process name
	set wait 0;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	set msg
} "slxERR_SET_RECV_FILT : Could not adjust Message Receive Filter for\
\"/unknown\""

#-------------------------------------------------------------------------
# Let's start up the partner program which will receive the setup messages
# We'll ask for 2 replies per command, with 3 seconds between replies.
# Stdout and stderr are redirected to a file;
set pId [exec slxTestRecvCmd 2 3 >&result.temp &]
sleep 3
# Give it the time to register with CCS
set counter 0
while {[catch "exec $msgSend -v -n "" slxTestRecvCmd PING \"\""]} {
    sleep 1
    if {[incr counter] == $timeOut} {
        error "slxTestRecvCmd did not start up in $timeOut seconds ?!?!"
    }
}

# IMPL.: Temporary skipped - how is it possible to unset INS_SETUPPATH
#        when "exec" is used?
if {1 == 0} {
# Produce error for unknown file
test slxParseSetup-1.2 {unknown file, INS_SETUPPATH not set} {
	set file SureYouCantFindThisOne;	# setup filename
	set cat TEL;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 0;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	set msg
}  {slxERR_INS_SETUPPATH : Environment variable INS_SETUPPATH is not set}
}

#-------------------------------------------------------------------------
# Produce error for unknown file
test slxParseSetup-1.3 {unknown file, INS_SETUPPATH set} {
	set env(INS_SETUPPATH) [pwd]
	set file SureYouCantFindThisOne;	# setup filename
	set cat TEL;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 0;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	unset env(INS_SETUPPATH)
	set msg
}  {slxERR_LOCATE_FILE : Could not find "SureYouCantFindThisOne" according to INS_SETUPPATH\
or path given with filename}

#-------------------------------------------------------------------------
# Produce error for file in wrong format
test slxParseSetup-1.4 {file in wrong format} {
    set env(INS_SETUPPATH) [pwd]
    exec cp slxParseSetup.test.prog $tf.temp 
    exec chmod ugo=+rwx $tf.temp  
    set file $tf.temp;			# setup filename
    set cat TEL;				# category
    set wte	"";			# environment name
    set proc slxTestRecvCmd;		# process name
    set wait 0;				# wait flag
    set to 1000;				# timeout value
    catch "exec $testP $file $cat $wte $proc $wait $to" msg
    unset env(INS_SETUPPATH)
    exec rm $tf.temp
    set msg
}  "slxERR_FILE_FORMAT : File \"[pwd]/$tf.temp\" does not have the correct format"

#-------------------------------------------------------------------------
# Produce error for file read-protected
test slxParseSetup-1.5 {read protected file} {
	set env(INS_SETUPPATH) [pwd]
	chmod -r $tf
	set file $tf;				# setup filename
	set cat TEL;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 0;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	unset env(INS_SETUPPATH)
	chmod +r $tf
	set msg
}  "slxERR_OPEN_FILE : Could not open file \"[pwd]/$tf\""

#-------------------------------------------------------------------------
# Produce error for timeout on final reply
# Remark that the partner program will do an exit(1) here, as it won't find
# the proper message queue ID for the reply (slxTestParseSetup will die
# before the partner can send a reply)
test slxParseSetup-1.6 {timeout on final reply} {
	set env(INS_SETUPPATH) [pwd]
	set file $tf;				# setup filename
	set cat TEL;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 1;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	unset env(INS_SETUPPATH)
	sleep 3;		# untill partner quits
	exec rm result.temp
	set msg
}  {ccsERR_MSG_TIMEOUT : Timeout on message receive}

# PART 2: functionality test
# ==========================
set env(INS_SETUPPATH) [pwd]

#-------------------------------------------------------------------------
set pId [exec slxTestRecvCmd 1 > result.temp &]
sleep 3
# Give it the time to register with CCS
# Remark that slxTestRecvCmd's stdout will contain already some output (i.e.
# the message body going with PING, in other words an empty line

set counter 0
while {[catch "exec $msgSend -v -n "" slxTestRecvCmd PING \"\""]} {
    sleep 1
    if {[incr counter] == $timeOut} {
        error "slxTestRecvCmd did not start up in $timeOut seconds ?!?!"
    }
}

test slxParseSetup-2.1 {send TEL category across} {
	set file $tf;				# setup filename
	set cat TEL;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 0;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	# terminate the partner proc with an EXIT command - this flushes output
	exec $msgSend -v -n "" slxTestRecvCmd EXIT " "
	
	if {![string compare $msg "Expected replies = 0"]} {
		set msg [exec diff result.temp slxParseSetup.test2.1.rf]
	}
#	exec rm result.temp
	set msg
}  {}

#-------------------------------------------------------------------------
set pId [exec slxTestRecvCmd 1 > result.temp &]
sleep 3
# Give it the time to register with CCS
set counter 0
while {[catch "exec $msgSend -v -n "" slxTestRecvCmd PING \"\""]} {
    sleep 1
    if {[incr counter] == $timeOut} {
        error "slxTestRecvCmd did not start up in $timeOut seconds ?!?!"
    }
}

test slxParseSetup-2.2 {send INS category across} {
	set file $tf;				# setup filename
	set cat INS;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 0;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	# terminate the partner proc with an EXIT command - this flushes output
	exec $msgSend -v -n "" slxTestRecvCmd EXIT " "
	
	if {![string compare $msg "Expected replies = 0"]} {
		set msg [exec diff result.temp slxParseSetup.test2.2.rf]
	}
	exec rm result.temp
	set msg
}  {}

#-------------------------------------------------------------------------
set pId [exec slxTestRecvCmd 2 > result.temp &]
sleep 3
# Give it the time to register with CCS
set counter 0
while {[catch "exec $msgSend -v -n "" slxTestRecvCmd PING \"\""]} {
    sleep 1
    if {[incr counter] == $timeOut} {
        error "slxTestRecvCmd did not start up in $timeOut seconds ?!?!"
    }
}

test slxParseSetup-2.3 {2 replies per command, don't wait for final reply} {
	set file $tf;				# setup filename
	set cat INS;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 0;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	# terminate the partner proc with an EXIT command - this flushes output
	exec $msgSend -v -n "" slxTestRecvCmd EXIT " "
	
	if {![string compare $msg "Expected replies = 1"]} {
		set msg [exec diff result.temp slxParseSetup.test2.2.rf]
	}
	exec rm result.temp
	set msg
}  {}

#-------------------------------------------------------------------------
set pId [exec slxTestRecvCmd 2 > result.temp &]
sleep 3
# Give it the time to register with CCS
set counter 0
while {[catch "exec $msgSend -v -n "" slxTestRecvCmd PING \"\""]} {
    sleep 1
    if {[incr counter] == $timeOut} {
        error "slxTestRecvCmd did not start up in $timeOut seconds ?!?!"
    }
}

test slxParseSetup-2.4 {2 replies per command, wait for final reply} {
	set file $tf;				# setup filename
	set cat INS;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 1;				# wait flag
	set to 1000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	# terminate the partner proc with an EXIT command - this flushes output
	exec $msgSend -v -n "" slxTestRecvCmd EXIT " "
	
	if {![string compare $msg "Expected replies = 0"]} {
		set msg [exec diff result.temp slxParseSetup.test2.2.rf]
	}
	exec rm result.temp
	set msg
}  {}

#-------------------------------------------------------------------------
set pId [exec slxTestRecvCmd 2 1 > result.temp &]
sleep 3
# Give it the time to register with CCS
set counter 0
while {[catch "exec $msgSend -v -n "" slxTestRecvCmd PING \"\""]} {
    sleep 1
    if {[incr counter] == $timeOut} {
        error "slxTestRecvCmd did not start up in $timeOut seconds ?!?!"
    }
}

test slxParseSetup-2.5 {2 replies per command, delay in between replies} {
	set file $tf;				# setup filename
	set cat INS;				# category
	set wte	"";			# environment name
	set proc slxTestRecvCmd;		# process name
	set wait 1;				# wait flag
	set to 2000;				# timeout value
	catch "exec $testP $file $cat $wte $proc $wait $to" msg
	# terminate the partner proc with an EXIT command - this flushes output
	exec $msgSend -v -n "" slxTestRecvCmd EXIT " "
	
	if {![string compare $msg "Expected replies = 0"]} {
		set msg [exec diff result.temp slxParseSetup.test2.2.rf]
	}
	exec rm result.temp
	set msg
}  {}

