#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxResolvePath.test.tcl,v 1.145 2005/07/27 12:31:55 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# epozna     30/07/02  created
#

#************************************************************************
#   NAME
#   slxResolvePath.test.tcl - test script for slxResolvePath()
# 
#   SYNOPSIS
#   slxResolvePath.test.tcl
# 
#   DESCRIPTION
#   Used for testing the SLX function slxResolvePath().
#
#   The following tests are carried out
#   
#   1.1 Resolve Path:  ./ at the beginning of the path 
#
#   1.2 Resolve Path:  ../../ at the beginning of the path
#
#   1.3 Resolve Path:  ../ and ./ in the middle of the path
#
#   1.4 Resolve Path: environment variable 
#
#   1.5 Resolve Path: ~
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
source ../test/slxTestCommon.tcl

set tf slxResolvePath.test
set homedir [pwd]
set slxResolvePath ../bin/slxTestResolvePath

# PART 1.1: test of resolving current directory
# ==========================================================
test slxResolvePath-1.1 {VLTSW20000078: current directory} {
    #echo  [pwd] 
    exec $slxResolvePath ./ [pwd] 

    #echo  [pwd]/myfile
    exec $slxResolvePath ./myfile [pwd]/myfile
} ""

# PART 1.2: test of resolving ../../ directory
# ==========================================================
test slxResolvePath-1.2 {VLTSW20000078: ../../ beginning of path } {
    #test of direcory
    set path ../../
    cd $path
    set tmpdir [pwd]
    cd $homedir
    #echo tmpdir: $tmpdir
    #echo currdir: [pwd]
    exec $slxResolvePath $path $tmpdir
   
    # test of file
    # the occurance of '//' in the result is acceptable
    exec $slxResolvePath $path/myfile $tmpdir//myfile

} ""


# PART 1.3: test of resolving ./ and ../ inside the path
# ==========================================================
test slxResolvePath-1.3 {VLTSW20000078: ./ and ../ inside the path } {
    #test of direcory
    set path ../test/./../
    cd $path
    set tmpdir [pwd]
    cd $homedir
    #echo tmpdir: $tmpdir
    #echo currdir: [pwd]
    exec $slxResolvePath $path $tmpdir
   
    # test of file
    exec $slxResolvePath $path/myfile $tmpdir//myfile

} ""

# PART 1.4: test of resolving environment variable
# ==========================================================
test slxResolvePath-1.3 {VLTSW20000078: environment variable } {
    
    set path "\$PWD"
    #echo $path
    exec $slxResolvePath $path [pwd]
   
    # test of file
    exec $slxResolvePath $path/myfile [pwd]/myfile

} ""

# PART 1.5: test of resolving symbol '~' in the path 
# ==========================================================
test slxResolvePath-1.3 {VLTSW20000078: environment variable } {
    
    exec $slxResolvePath ~ $HOME
   
    # test of file
    exec $slxResolvePath ~/myfile $HOME/myfile

} ""

# --- oOo ---
