/******************************************************************************
* E.S.O. - VLT project
*
* slxSetupToShortFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  08/09/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxSetupToShortFITS - convert a setup file to Short-FITS format
* 
*   SYNOPSIS
*   slxSetupToShortFITS <setup filename> [-t]
* 
*   DESCRIPTION
*   Converts a setup file in the Normal FITS format to the Short-FITS
*   format. The old filename is overwritten with the new.
*
*   <setup filename>  Can be complete name, including the path. If only the
*                      name is given, there is searched for the setup file
*                      according to the INS_SETUPPATH environment variable.
*
*   -t verbose switch; if specified, will send CCS error messages also to 
*	stderr
*
*   ENVIRONMENT 
*   INS_SETUPPATH    Defines the various directories where setup files are
*                    stored.
*
*
*   SEE ALSO
*   slxNormalToShortFITS(), slxConvertToShort()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxSetupToShortFITS.c,v 1.145 2005/07/27 12:22:35 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers 
 */
#include <stdio.h>

/* Local Headers 
 */
#include "slx.h"

/* Main
 */
int main (int argc, char *argv[])
{
    ccsERROR     error;
    ccsPROCNAME  procName;
    vltLOGICAL   verbose = ccsFALSE;


    if (argc == 1 || ((argc == 3) && (strcmp(argv[2],"-t") != 0)) || argc > 3)
        {
        fprintf(stderr,"\nCorrect syntax is:\n\n"
        		"slxSetupToShortFITS  <setup filename> [-t]\n\n");
        exit(1);
        }

    /* If there are 3 arguments, the last is for sure -t (see above) 
     */
    if (argc == 3) 
        verbose = ccsTRUE;
    
    strcpy(procName, argv[0]);
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
	{
	fprintf(stderr, "ccsInit failed.\n");
	exit(1);
	}

    if (slxConvertToShort(argv[1], &error) != SUCCESS)
        {
        fprintf(stderr, "\nCouldn't convert the file - ");
        fprintf(stderr, "check if filename is correct!\n\n");
        if (verbose)
            fprintf(stderr, "%s", error.runTimePar);     /* includes \n */
        ccsExit(&error);
        exit(1);
        }

    ccsExit(&error);
    exit(0);
}














