#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxSplitFITS.test.tcl,v 1.145 2005/07/27 11:13:29 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxSplitFITS.test.prog - test program for slxSplitFITS()
#
#   SYNOPSIS
#   slxSplitFITS.test.prog
#
#   DESCRIPTION
#   This script tests the slxSplitFITS program, as follows:
#	2.1 Normal execution of slxSplitFITS - parameter with string value
#	2.2 Normal execution of slxSplitFITS - parameter with logical value
#	2.3 Normal execution of slxSplitFITS - parameter with integer value
#	2.4 Normal execution of slxSplitFITS - parameter with double value
#
#   To run these tests, the program slxTestSplitFITS is used.
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs

#set VERBOSE 1

# testP is the name of the binary we use for testing our function
set testP ../bin/slxTestSplitFITS


# In case of errors, exec $testP does an exit(1), so from Tcl
# we have to catch this error

# PART 1: argument testing
# ========================

# ---------------------------------------------------------------------------
# Produce error for input string not exactly 80 chars
#test slxSplitFITS-1.1 {FITS input string is too long} {
#	set string "This string is beyond any doubt much too long as it is\
#	            supposed to be exactly 80 chars and it is far over it"
#	catch "exec $testP $string" msg
#	set msg
#}  {slxERR_????}

# ---------------------------------------------------------------------------
# Produce error for input string not exactly 80 chars
#test slxSplitFITS-1.2 {FITS input string is too short} {
#	set string "This string is beyond any doubt too short"
#	catch "exec $testP $string" msg
#	set msg
#}  {slxERR_????}

# PART 2: functionality test
# ==========================
test slxSplitFITS-2.1 {parameter with string value} {
	set string {"HIERARCH ESO cat subsystem parameter=         'any string '\
	            / comment           "}
	catch "exec $testP $string" msg
	set msg
}  {{cat} {subsystem} {parameter} {'any string '} {comment           }}

test slxSplitFITS-2.2 {parameter with logical value} {
	set string {"HIERARCH ESO cat subsystem parameter=                    T \
	            / comment           "}
	catch "exec $testP $string" msg
	set msg
}  {{cat} {subsystem} {parameter} {T} {comment           }}

test slxSplitFITS-2.3 {parameter with integer value} {
	set string {"HIERARCH ESO cat subsystem parameter=              1234567 \
	            / comment           "}
	catch "exec $testP $string" msg
	set msg
}  {{cat} {subsystem} {parameter} {1234567} {comment           }}

test slxSplitFITS-2.4 {parameter with double value} {
	set string {"HIERARCH ESO cat subsystem parameter=           123.456789 \
	            / comment           "}
	catch "exec $testP $string" msg
	set msg
}  {{cat} {subsystem} {parameter} {123.456789} {comment           }}

