#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxStoreLine.test.tcl,v 1.145 2005/07/27 11:29:33 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxStoreLine.test.prog - test script for slxStoreLine()
# 
#   SYNOPSIS
#   slxStoreLine.test.prog
# 
#   DESCRIPTION
#   Used for testing the SLX function slxStoreLine().
#
#   The following tests are carried out
#   
#   1.1 Normal behaviour - saving non-exisiting keyword in file
#
#   1.2 Normal behaviour - saving existing keyword in file
#
#   1.3 Normal behaviour - saving existing keyword including '\n'
#
#   2.1 Non-existing file specified
#
#   2.2 Read protected file specified (file could not be opened)
#
#   2.3 Format of keyword line is wrong - SKIPPED!!
#
#   2.4 Set-up file write protected (could not rename file)
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
source ../test/slxTestCommon.tcl


set tf slxStoreLine.test
set testPath [pwd]
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM
set slxTestStoreLine ../bin/slxTestStoreLine

# PART 1.1 Saving non-exisiting keyword in file
# ================================================================
test slxStoreLine-1.1 {Saving non-existing keyword} {
    exec cp $tf.1.1.bs.ins $tf.1.1.temp.ins
    exec chmod ugo=+rwx $tf.1.1.temp.ins 
    exec ../bin/slxTestStoreLine "NEW.KEYW.STOREIT        12345; # A comment"  $tf.1.1.temp.ins
    set msg [exec diff $tf.1.1.temp.ins $tf.1.1.rf.ins]
    exec rm $tf.1.1.temp.ins
    set msg
} ""


# PART 1.2 Normal behaviour - saving existing keyword in file
# ===========================================================
test slxStoreLine-1.2 {Saving existing keyword} {
    exec cp $tf.1.2.bs.ins $tf.1.2.temp.ins 
    exec chmod ugo=+rwx $tf.1.2.temp.ins 
    exec ../bin/slxTestStoreLine "INS.FOCU.ENC           1234; # A comment"  $tf.1.2.temp.ins

    set msg [exec diff $tf.1.2.temp.ins $tf.1.2.rf]
    exec rm $tf.1.2.temp.ins
    set msg
} ""



# PART 1.3 Normal behaviour - saving existing keyword including '\n'
# ==================================================================
test slxStoreLine-1.3 {Saving existing keyword including '\n'} {
    exec cp $tf.1.3.bs.ins $tf.1.3.temp.ins
    exec chmod ugo=+rwx $tf.1.3.temp.ins
    exec ../bin/slxTestStoreLine "INS.FOCU.ENC           1234; # A comment\n"  $tf.1.3.temp.ins

    set msg [exec diff $tf.1.3.temp.ins $tf.1.3.rf]
    exec rm $tf.1.3.temp.ins
    set msg
} ""



# PART 2.1 Non-existing file specified
# ====================================
test slxStoreLine-2.1 {Non-existing set-up file} {
    if {[catch "exec -keepnewline ../bin/slxTestStoreLine \"dont care\" nOnExIsT" msg]} {
	set testString "Could not find \"nOnExIsT\" according"
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""



# PART 2.2 read protected file specified (file could not be opened)
# =================================================================
exec chmod -rw $tf.2.2.bs 
test slxStoreLine-2.2 {Read protected set-up file} {
    if {[catch "exec -keepnewline ../bin/slxTestStoreLine \"dont care\" $tf.2.2.bs" msg]} {
        set testString "Could not open file \"slxStoreLine.test.2.2.bs\""
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""
exec chmod +rw $tf.2.2.bs 



# PART 2.4 Set-up file write protected (could not rename file)
# ============================================================
exec chmod -w $tf.2.4.bs.ins
test slxStoreLine-2.4 {Write protected set-up file} {
    if {[catch "exec -keepnewline ../bin/slxTestStoreLine \"INS.FOCU.ENC mmm\" $tf.2.4.bs.ins" msg]} {
	set testString "Could not open file \"slxStoreLine.test.2."
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
} ""
exec chmod +rw $tf.2.4.bs.ins

