#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxTclTestAuto.tcl,v 1.145 2005/07/27 11:48:17 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  22/09/95  created
#
#************************************************************************
#   NAME
#
#     slxTclTestAuto
#
#   SYNOPSIS
#
#     slxTclTestAuto
#
#   DESCRIPTION
#
#     Tcl procedure used to test automatically the slxTcl library using
#     a test GUI panel.
#
#   CAUTIONS
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
wm withdraw . ; # withdraw main window from screen

proc slxTclTestPuts { file {flag 0}} {
    global myVar

    # Show on standard output all the results (needed for tat)
    # filter out panel debug items and all comment lines contained in the
    # setup file header
#    puts [exec cat $file]
    set filter "/^#/d"
    puts [exec sed -e $filter $file]
    # Delete file
    exec rm $file
    if {$flag == 1} {
        puts "\nValues are:"
        puts "simple $myVar(simple)"
        puts "method $myVar(method)"
        puts "bits $myVar(bits)"
        puts "type $myVar(typeR)"
        puts "time $myVar(timeR)"
        puts "type $myVar(typeB)"
        puts "time $myVar(timeB)"
    }
}                                            

global myVar
global env

###############################
#  Start The Test Procedure
###############################

# Delete all output files possibly left
catch {eval exec rm [glob slxTclTest*.stp]}
# Initialization
set appl slxTclTest
eval exec $appl &;                # Start Test Main Panel
ptlibPanelExist     $appl ;       # Check if the test panel is active
ptlibGetCanvasNames $appl cv bcv; # get canvas from panel

# Set values
set myVar(simple) default
set myVar(method) none
set myVar(bits) 10
set myVar(typeR) dark
set myVar(timeR) 123.456789
set myVar(typeB) normal
set myVar(timeB) 45678.9
ptlibSetOptMenu  $appl $cv.method $myVar(method)
ptlibClickRCButton  $appl  $cv.simple
ptlibSetScale  $appl  $cv.setupCommon.cv.bits $myVar(bits)
ptlibClickRCButton  $appl  $cv.redArm.cv.$myVar(typeR)
ptlibSetEntry  $appl  $cv.redArm.cv.time $myVar(timeR)
ptlibClickRCButton  $appl  $cv.blueArm.cv.$myVar(typeB)
ptlibSetEntry  $appl  $cv.blueArm.cv.time $myVar(timeB)

# Step 1:Save setup in slxTclTest1.stp (no dictionary no template)
set file slxTclTest1.stp
puts "\n\nStep 1:Save setup in $file (no dictionary no template)\n\n"
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file 1

# Step 2:Save setup in slxTclTest2.stp (no dictionary)
set file slxTclTest2.stp
puts "\n\nStep 2:Save setup in $file (no dictionary)\n\n"
ptlibClickRCButton  $appl  $cv.template
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file

# Step 3:Save setup in slxTclTest3.stp
set file slxTclTest3.stp
puts "\n\nStep 3:Save setup in slxTclTest3.stp\n\n"
ptlibClickRCButton  $appl  $cv.dictionary
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file

# Step 4:Save setup in slxTclTest4.stp after changing all values
set file slxTclTest4.stp
puts "\n\nStep 4:Save setup in slxTclTest4.stp after changing all values\n\n"
set myVar(simple) !default
set myVar(method) compressed
set myVar(bits) 4
set myVar(typeR) normal
set myVar(timeR) 4567.890123
set myVar(typeB) dark
set myVar(timeB) 89.0123456
ptlibSetOptMenu  $appl $cv.method $myVar(method)
ptlibClickRCButton  $appl  $cv.simple
ptlibSetScale  $appl  $cv.setupCommon.cv.bits $myVar(bits)
ptlibClickRCButton  $appl  $cv.redArm.cv.$myVar(typeR)
ptlibSetEntry  $appl  $cv.redArm.cv.time $myVar(timeR)
ptlibClickRCButton  $appl  $cv.blueArm.cv.$myVar(typeB)
ptlibSetEntry  $appl  $cv.blueArm.cv.time $myVar(timeB)
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file 1

# Step 5:Load setup from slxTclTest3.stp and save it in slxTclTest5.stp
set file slxTclTest3.stp
puts "\n\nStep 5:Load setup from slxTclTest3.stp and save it in slxTclTest5.stp\n\n"
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.load
set file slxTclTest5.stp
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file

# Step 6:Save setup in slxTclTest6.stp disabling some keywords
set file slxTclTest6.stp
puts "\n\nStep 6:Save setup in slxTclTest5.stp disabling some keywords\n\n"
ptlibClickRCButton  $appl  $cv.redSelect
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file

# Step 7:Save setup in slxTclTest7.stp disabling other keywords
set file slxTclTest7.stp
puts "\n\nStep 7:Save setup in slxTclTest7.stp disabling other keywords\n\n"
ptlibClickRCButton  $appl  $cv.blueSelect
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file

# Step 8:Save setup in slxTclTest7.stp enabling again all keywords
set file slxTclTest8.stp
puts "\n\nStep 8:Save setup in slxTclTest8.stp enabling again all keywords\n\n"
ptlibClickRCButton  $appl  $cv.redSelect
ptlibClickRCButton  $appl  $cv.blueSelect
ptlibSetEntry  $appl  $cv.file $file
ptlibClickButton  $appl  $bcv.save
slxTclTestPuts $file

# Terminate
ptlibClickMenu  $appl ".${appl}.topMenu.menu.file"  Quit
exit
