/******************************************************************************
* E.S.O. - VLT project
*
* slxParseSetup.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  28/07/94  Created 1. version
*/

/************************************************************************
*------------------------------------------------------------------------
* Simply loops on receiving any command, and replying a number of times
* to that command (# times is given in 1st argument); the reply contains
* the original body.
* The second argument gives a wait-value (in s) before eventual multiple
* replies are sent
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxTestRecvCmd.c,v 1.145 2005/07/27 12:05:06 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* Local Headers
 */
#include "slx.h"

/* Program
 */
int main(int argc, char *argv[])
{
    ccsERROR           error, errmsg;     
    ccsPROCNAME        procName;
    char               sendBuf[slxMAX_CCS_BODY];
    char               *buffer;    
    msgHEADER          *message;
    msgRECEIVEFILTER   msgFilter;
    msgCMD             *recvCommand, orgCommand;      
    msgCMDID           recvCommandID, orgCommandId;  
    msgLENGTH          buflen;       
    msgPROCESSID       *recvID, orgId;   
    vltUINT8           msgType;
    int		i,cntr,nrReplies = 1;	/* default: only one reply */
    unsigned	seconds=0;		/* default: immediate multiple repl.s*/
    vltLOGICAL  lastReply;
    char	*name;


    /* First argument is number of replies to send to a command */
    if (argc > 1) sscanf (argv[1], "%d", &nrReplies);
    
    if (argc > 2) sscanf (argv[2], "%d", &seconds);
    
    /* application name is last part of pathname */
    if ((name = strrchr(argv[0],'/')) == NULL)
	{
	name = argv[0];		/* name does not contain path */
	} 
    else
	{
	name = name+1;		/* pointer to first char after last slash */
	}

    strcpy(procName, name);
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
	{
	fprintf (stderr,"ccsInit failed.\n");
	exit(1);
	}
    memcpy (&errmsg, &error, sizeof(ccsERROR));
    
    /* Set the message filter
     */
    msgFilter.accept = msgANY_MESSAGE;

    while (ccsTRUE)		/* loop forever */
	{
	message = (msgHEADER *)NULL;
	if (msgRecvMsg(&msgFilter, msgNO_TIMEOUT, &message, &error) 
	    == FAILURE)
	    {
	    fprintf(stderr,"%s",error.runTimePar);
	    exit(1);
	    }
	
	if (msgParseMsg(message, &msgType, &recvCommand, &recvCommandID, 
			&recvID, &lastReply, &buffer, &buflen, 
			&errmsg, &error) == FAILURE)
	    {
	    fprintf(stderr,"%s",error.runTimePar);
	    exit(1);
	    }
	
	if (msgType != msgTYPE_COMMAND)
	    {
	    fprintf(stderr,"Got a message of unexpected type");
	    exit(1);
	    }
	
	printf ("%s\n",buffer);		/* to stdout */
	memcpy(orgCommand, *recvCommand, sizeof(msgCMD));
	orgCommandId = recvCommandID;
	memcpy(&orgId, recvID, sizeof(msgPROCESSID));
	memcpy(sendBuf, buffer, buflen);
	
	if (strcmp(orgCommand,"EXIT") == 0)
	    cntr = 1;
	if (strcmp(orgCommand,"PING") == 0) 
	    cntr = 1;
	else
	    cntr = nrReplies;
	
	for (i = 1; i<=cntr; i++)
	    {
	    if (i != cntr) 
		lastReply = ccsFALSE;
	    else
		lastReply = ccsTRUE;
	    
	    /* next guarantees that stdout is flushed before a reply is
	     * sent to the one who send the EXIT command, so this sender
	     * can operate on the output as soon as he receives a reply
             */
	    if (strcmp(orgCommand,"EXIT") == 0) fclose (stdout);

	    if (msgSendReply(&orgId,
			     orgCommandId,
			     orgCommand,
			     sendBuf,
			     buflen,
			     lastReply,
			     &errmsg,
			     &error) == FAILURE)
		{
		fprintf(stderr,"%s",error.runTimePar);
		exit(1);
		}
	    if (cntr > 1) sleep(seconds);
	    }
	if (strcmp(orgCommand,"EXIT") == 0) exit(0);
	}
    }
/*
 *****************************************************************************
 */


/*___oOo___*/



