#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxYankExtention.test.tcl,v 1.145 2005/07/27 12:33:02 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# epozna     30/07/02  created to test VLTSW20000334
#

#************************************************************************
#   NAME
#   slxYankExtention.test.tcl - test script for slxYankExtention..()
# 
#   SYNOPSIS
#   slxYankExtention.test.tcl
# 
#   DESCRIPTION
#   Used for testing the SLX function slxYankExtention().
#
#   The following tests cases are carried out
#   
#   1.1  filename with no extentions : myfile
#   1.2  filename with extention     : myfile.fits
#   1.3  filename with more dots     : file_2000-07-31T18:09:39.97.fits
# 
#   For each case 5 tests are carried out:
#     a) declare the extention
#        using: slxGetExtension
#     b) remove the  extention that has been declared
#        using: slxYankExtension
#     c) remove the last extention (after the first dot)
#        using: slxYankExtensionGen
#     d) remove the  extention (after the last dot)
#        using: slxYankExtensionGenLast (VLTSW20000334)
#     e) try to remove a non existant extention
#        using: slxYankExtension
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
source ../test/slxTestCommon.tcl

set tf slxYankExtention.test
set homedir [pwd]
set slxYankExtention ../bin/slxTestYankExtention

# PART 1.1: filename with no extention
# ==========================================================
test slxYankExtention-1.1 {VLTSW20000334: file with no extention} {
     exec $slxYankExtention myfile 
} "''\n'myfile'\n'myfile'\n'myfile'\n'myfile'"

# PART 1.2: filename with extention (typical case)
# ==========================================================
test slxYankExtention-1.2 {VLTSW20000334: file with extention } {
   exec $slxYankExtention myfile.fits
} "'fits'\n'myfile'\n'myfile'\n'myfile'\n'myfile.fits'"


# PART 1.3: filename with more dotes 
# ==========================================================
test slxResolvePath-1.3 {VLTSW20000334: file with more dots  } {
    exec $slxYankExtention  file_2000-07-31T18:09:39.97.fits
} "'fits'\n'file_2000-07-31T18:09:39.97'\n'file_2000-07-31T18:09:39'\n'file_2000-07-31T18:09:39.97'\n'file_2000-07-31T18:09:39.97.fits'"


# --- oOo ---
